Type: Package
Package: TriMatch
Title: Propensity Score Matching of Non-Binary Treatments
Version: 1.0.2
Date: 2026-04-28
Authors@R: 
    person("Jason", "Bryer", , "jason@bryer.org", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-2454-0402"))
Description: Propensity score matching for non-binary treatments.
License: GPL (>= 2)
URL: https://jbryer.github.io/TriMatch/,
        https://github.com/jbryer/TriMatch/
BugReports: https://github.com/jbryer/TriMatch/issues/
Depends: ggplot2, R (>= 3.0), reshape2, scales
Imports: car, compiler, grid, gridExtra, plyr, PSAgraphics, psych,
        randomForest, stats, stringr
Suggests: bookdown, knitr, MASS, rmarkdown, xtable
VignetteBuilder: knitr
Encoding: UTF-8
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-04-28 16:19:14 UTC; jasonbryer
Author: Jason Bryer [aut, cre] (ORCID: <https://orcid.org/0000-0002-2454-0402>)
Maintainer: Jason Bryer <jason@bryer.org>
Repository: CRAN
Date/Publication: 2026-04-28 16:50:21 UTC
Built: R 4.5.2; ; 2026-05-01 19:39:51 UTC; unix

Type: Package
Package: TriMatch
Title: Propensity Score Matching of Non-Binary Treatments
Version: 1.0.2
Date: 2026-04-28
Authors@R: 
    person("Jason", "Bryer", , "jason@bryer.org", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-2454-0402"))
Description: Propensity score matching for non-binary treatments.
License: GPL (>= 2)
URL: https://jbryer.github.io/TriMatch/,
        https://github.com/jbryer/TriMatch/
BugReports: https://github.com/jbryer/TriMatch/issues/
Depends: ggplot2, R (>= 3.0), reshape2, scales
Imports: car, compiler, grid, gridExtra, plyr, PSAgraphics, psych,
        randomForest, stats, stringr
Suggests: bookdown, knitr, MASS, rmarkdown, xtable
VignetteBuilder: knitr
Encoding: UTF-8
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-04-28 16:19:14 UTC; jasonbryer
Author: Jason Bryer [aut, cre] (ORCID: <https://orcid.org/0000-0002-2454-0402>)
Maintainer: Jason Bryer <jason@bryer.org>
Repository: CRAN
Date/Publication: 2026-04-28 16:50:21 UTC
Built: R 4.6.0; ; 2026-05-01 22:51:09 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(merge,triangle.matches)
S3method(merge,triangle.psa)
S3method(plot,balance.plots)
S3method(plot,triangle.matches)
S3method(plot,triangle.psa)
S3method(print,balance.plots)
S3method(print,triangle.plot)
S3method(print,trimatch.summary)
S3method(summary,balance.plots)
S3method(summary,triangle.matches)
S3method(summary,triangle.psa)
S3method(summary,unmatched)
export(OneToN)
export(balance.plot)
export(boxdiff.plot)
export(covariateBalance)
export(data.frame.to.list)
export(distances.plot)
export(loess3.plot)
export(maximumTreat)
export(multibalance.plot)
export(parallel.plot)
export(perpPt)
export(segment1)
export(segment2)
export(trimatch)
export(trips)
export(unmatched)
import(PSAgraphics)
import(compiler)
import(ggplot2)
import(gridExtra)
import(plyr)
import(reshape2)
import(stats)
importFrom(car,Anova)
importFrom(graphics,plot)
importFrom(grid,editGrob)
importFrom(grid,gTree)
importFrom(grid,getGrob)
importFrom(grid,grid.draw)
importFrom(grid,grid.layout)
importFrom(grid,grid.newpage)
importFrom(grid,grob)
importFrom(grid,grobHeight)
importFrom(grid,grobWidth)
importFrom(grid,pushViewport)
importFrom(grid,upViewport)
importFrom(grid,viewport)
importFrom(grid,vpPath)
importFrom(grid,vpTree)
importFrom(psych,describe)
importFrom(psych,describeBy)
importFrom(randomForest,randomForest)
importFrom(scales,percent)
importFrom(stringr,str_count)
importFrom(stringr,str_detect)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  message = FALSE,
  warning = FALSE,
  error = FALSE
)
options(digits = 2)

## ----setup--------------------------------------------------------------------
library(TriMatch)
data("tutoring")
data("nmes")

## -----------------------------------------------------------------------------
names(tutoring)

## -----------------------------------------------------------------------------
table(tutoring$treat, tutoring$Course, useNA="ifany")

## -----------------------------------------------------------------------------
formu <- ~ Gender + Ethnicity + Military + ESL + EdMother + EdFather + 
Age + Employment + Income + Transfer + GPA
tutoring.tpsa <- trips(tutoring, tutoring$treat, formu)

## -----------------------------------------------------------------------------
plot(tutoring.tpsa)

## -----------------------------------------------------------------------------
tutoring.matched <- trimatch(tutoring.tpsa, exact=tutoring[,c("Course")])

## -----------------------------------------------------------------------------
tutoring.matched.caliper <- trimatch(tutoring.tpsa, 
exact=tutoring[,c("Course")], method=NULL)

## -----------------------------------------------------------------------------
tutoring.matched.2to1 <- trimatch(tutoring.tpsa, 
exact=tutoring[,c("Course")], method=OneToN, M1=2, M2=1)
tutoring.matched.3to2 <- trimatch(tutoring.tpsa, 
exact=tutoring[,c("Course")], 
method=OneToN, M1=3, M2=2)

## ----triangleplot, fig.cap = 'Traingle Plot'----------------------------------
print(plot(tutoring.matched, rows=c(50), line.alpha=1, draw.segments=TRUE))

## -----------------------------------------------------------------------------
summary(unmatched(tutoring.matched))

## -----------------------------------------------------------------------------
summary(unmatched(tutoring.matched.caliper))

## -----------------------------------------------------------------------------
summary(unmatched(tutoring.matched.2to1))

## -----------------------------------------------------------------------------
summary(unmatched(tutoring.matched.3to2))

## ----multibalance, fig.cap = 'Multiple Covariate Balance Plot of Absolute Standardized Effect Sizes Before and After Propensity Score Adjustment'----
print(multibalance.plot(tutoring.tpsa) + ggtitle("Covariate Balance Plot"))

## ----balance, fig.cap = 'Covariate Balance Plots', cache = TRUE---------------
bplots <- balance.plot(tutoring.matched, tutoring[,all.vars(formu)], 
		legend.position="none", x.axis.labels=c("C","T1","T1"), x.axis.angle=0)
print(plot(bplots, cols=3, byrow=FALSE))

## -----------------------------------------------------------------------------
matched.out <- merge(tutoring.matched, tutoring$Grade)
names(matched.out)
head(matched.out)

## -----------------------------------------------------------------------------
s1 <- summary(tutoring.matched, tutoring$Grade)
names(s1)
s1$friedman.test
s1$t.tests

## -----------------------------------------------------------------------------
s2 <- summary(tutoring.matched.caliper, tutoring$Grade)
s3 <- summary(tutoring.matched.2to1, tutoring$Grade)
s4 <- summary(tutoring.matched.3to2, tutoring$Grade)

print("Max Treat"=s1, "Caliper"=s2, "2-to-1"=s3, "3-to-2"=s4)

## ----boxplots, fig.cap = 'Boxplot of Differences', fig.show = 'hold', out.width = '30%'----
boxdiff.plot(tutoring.matched, tutoring$Grade, 
			 ordering=c("Treat2","Treat1","Control")) + 
	ggtitle("Maximum Treatment Matching")
boxdiff.plot(tutoring.matched.caliper, tutoring$Grade, 
			 ordering=c("Treat2","Treat1","Control")) +
	ggtitle("Caliper Matching")
boxdiff.plot(tutoring.matched.2to1, tutoring$Grade, 
			 ordering=c("Treat2","Treat1","Control")) +
	ggtitle("2-to-1-to-n Matching")

## ----loess, fig.cap = 'Loess Plot for Caliper Matching'-----------------------
loess3.plot(tutoring.matched.caliper, tutoring$Grade, ylab="Grade", 
			points.alpha=.1, method="loess")

## -----------------------------------------------------------------------------
data(nmes)
nmes <- subset(nmes, select = c(packyears, smoke, LASTAGE, MALE, RACE3, beltuse, educate, marital, SREGION, POVSTALB, HSQACCWT, TOTALEXP))

## -----------------------------------------------------------------------------
nmes <- na.omit(nmes)

## -----------------------------------------------------------------------------
nmes$smoke <- factor(nmes$smoke, levels=c(0,1,2), labels=c("Never","Smoker","Former"))
nmes$LogTotalExp <- log(nmes$TOTALEXP + 1)
(medPY <- median(nmes[nmes$smoke != "Never",]$packyears))
table(nmes$smoke, nmes$packyears > medPY)
nmes$smoke2 <- ifelse(nmes$smoke == "Never", "Never", 
ifelse(nmes$packyears > 17, "Heavy", "Moderate"))
table(nmes$smoke, nmes$smoke2, useNA="ifany")

## ----packyearsAndTotalExp, fig.cap = 'Relationship Between Pack Year and Total Expenditures by Current Smoking Status', fig.show = 'hold', out.width = '50%'----
ggplot(nmes[nmes$smoke != "Never",], aes(x=log(packyears+1), color=smoke, fill=smoke)) +
	  	geom_density(alpha=.1) + 
	  	theme(legend.position="none", plot.margin=rep(unit(0, "cm"), 4)) +
	  	xlab("") + ylab("Density")
ggplot(nmes[nmes$smoke != "Never",], aes(x=log(packyears+1), y=LogTotalExp, color=smoke, fill=smoke)) + 
	  	geom_point(alpha=.2) + 
	  	geom_smooth(method="loess", formula = y ~ x) +
	  	scale_color_hue("") + scale_fill_hue("") +
	  	theme(legend.position=c(.9,1), plot.margin=rep(unit(0, "cm"), 4)) + 
	  	xlab("log(Pack Year)") + ylab("log(Total Expenditures)")

## -----------------------------------------------------------------------------
nmes$LastAge5 <- cut(nmes$LASTAGE, 
breaks=quantile(nmes$LASTAGE, probs=seq(0,1,1/5)),
include.lowest=TRUE, orderd_result=TRUE)

## -----------------------------------------------------------------------------
formu <- ~ LASTAGE + MALE + RACE3 + beltuse + educate + marital + 
SREGION + POVSTALB

## -----------------------------------------------------------------------------
tpsa.smoke <- trips(nmes, nmes$smoke, formu)
tpsa.packyears <- trips(nmes, nmes$smoke2, formu)

## ----nmestriangleplot, fig.cap = 'Triangle Plots for NMES', fig.show = 'hold', out.width = '50%'----
p.smoke <- plot(tpsa.smoke, sample=c(.05), edge.alpha=.1) + ggtitle("Treatment Variable: Current Smoking Status")
p.packyears <- plot(tpsa.packyears, sample=c(.05), edge.alpha=.1) + ggtitle("Treatment Variable: Lifetime Smoking Frequency")
p.smoke
p.packyears

## ----cache = TRUE-------------------------------------------------------------
tmatch.smoke <- trimatch(tpsa.smoke, 
exact=nmes[,c("LastAge5","MALE","RACE3")])
tmatch.packyears <- trimatch(tpsa.packyears, 
exact=nmes[,c("LastAge5","MALE","RACE3")])

## -----------------------------------------------------------------------------
summary(unmatched(tmatch.smoke))
summary(unmatched(tmatch.packyears))

## ----nmesbalance, fig.cap = 'Multiple Covariate Balance Plots for NMES', fig.show = 'hold', out.width = '50%'----
p.smoke <- multibalance.plot(tpsa.smoke) + ggtitle("Treatment Variable: Current Smoking Status")
p.packyears <- multibalance.plot(tpsa.packyears) + ggtitle("Treatment Variable: Lifetime Smoking Frequency")
p.smoke
p.packyears

## ----nmesboxplots, fig.cap = 'Boxplot of Differences for NMES', fig.show = 'hold', out.width = '50%'----
boxdiff.plot(tmatch.smoke, nmes$LogTotalExp, ordering=c("Smoker","Former","Never")) + 
	ggtitle("Treatment Variable: Current Smoking Status")
boxdiff.plot(tmatch.packyears, nmes$LogTotalExp, ordering=c("Heavy","Moderate","Never")) +
	ggtitle("Treatment Variable: Lifetime Smoking Frequency")

## ----eval=FALSE---------------------------------------------------------------
# sum.smoke <- summary(tmatch.smoke, nmes$LogTotalExp, ordering=c("Smoker","Former","Never"))
# sum.packyears <- summary(tmatch.packyears, nmes$LogTotalExp, ordering=c("Heavy","Moderate","Never"))
# print("Current Smoking Status" = sum.smoke, "Smoking Frequency" = sum.packyears)

## ----eval=FALSE---------------------------------------------------------------
# sum.smoke$t.tests
# sum.packyears$t.test

require(TriMatch)
require(gridExtra)
data(nmes)

nmes <- nmes[!is.na(nmes$packyears),]
nmes <- subset(nmes, select=c(packyears, smoke, LASTAGE, MALE, RACE3, 
				beltuse, educate, marital, SREGION, POVSTALB, HSQACCWT, TOTALEXP))

# Remove any rows with missing values. Consistent with Imai and van Dyk's analysis.
nmes <- na.omit(nmes)

nmes$smoke <- factor(nmes$smoke, levels=c(0,1,2), labels=c("Never","Smoker","Former"))
table(nmes$smoke, useNA="ifany")

# We'll create a log(TOTALEXP) varaible
nmes$POSEXP <- 1*(nmes$TOTALEXP>0)
nmes$LogTotalExp <- log(nmes$TOTALEXP + 1)

# Alternative way of defining treatments, heavy smokers 
# (i.e. packyears > median(packyears)), moderate smokers, and non-smokers.
(medPY <- median(nmes[nmes$smoke != "Never",]$packyears))
table(nmes$smoke, nmes$packyears > medPY)
nmes$smoke2 <- ifelse(nmes$smoke == "Never", "Never", 
					  ifelse(nmes$packyears > 17, "Heavy", "Moderate"))
table(nmes$smoke2, useNA="ifany")

# We see our control group is the same, but there is some shifting of heavy
# and moderate smokers and former and current smokers.
table(nmes$smoke, nmes$smoke2, useNA="ifany")

# log(packyears) and log(TotalExp) with density of log(packyears)
grid.newpage()
pushViewport(viewport(layout = grid.layout(2, 1, heights=unit(c(1,3), "null"))))
vplayout <- function(x, y) viewport(layout.pos.row = x, layout.pos.col = y)
print(ggplot(nmes[nmes$smoke != "Never",], 
			 aes(x=log(packyears+1), color=smoke, fill=smoke)) + 
	  	geom_density(alpha=.1) + 
	  	theme(legend.position="none", plot.margin=rep(unit(0, "cm"), 4)) +
	  	xlab("") + ylab("Density"), 
	  vp=vplayout(1,1))
print(ggplot(nmes[nmes$smoke != "Never",], 
			 aes(x=log(packyears+1), y=LogTotalExp, color=smoke, fill=smoke)) + 
	  	geom_point(alpha=.2) + 
	  	geom_smooth(method="loess") +
	  	scale_color_hue("") + scale_fill_hue("") +
	  	theme(legend.position=c(.9,1), plot.margin=rep(unit(0, "cm"), 4)) + 
	  	xlab("log(Pack Year)") + ylab("log(Total Expenditures)"),
	  vp=vplayout(2,1))

# From Imai and van Dyk (2004, pp. 857-858):
# Our analysis includes the following subject-level covariates: age at the times 
# of the survey (19-94), age when the individual started smoking, gender 
# (male, female), race (white, black, other), marriage status (married, widowed, 
# divorced, separated, never married), education level (college graduate, some 
# college, high school graduate, other), census region (Northeast, Midwest, 
# South, West), poverty status (poor, near poor, low income, middle income, 
# high income), and seat belt usage (rarely, sometimes, always/almost always).

# Imai and van Dyk observed that there appeared to be a relationship between age
# and medical expenditures. We will create a new categorical age variable using
# quintiles to use for partial exact matching.
nmes$LastAge5 <- cut(nmes$LASTAGE, 
					 breaks=quantile(nmes$LASTAGE, probs=seq(0,1,1/5)),
					 include.lowest=TRUE, orderd_result=TRUE)
table(nmes$smoke, nmes$LastAge5, useNA="ifany")
table(nmes$smoke2, nmes$LastAge5, useNA="ifany")

# Formula for estimating propensity scores. Note that we do not specify the
# dependent varaible (treatment indicator) as the trips function will replace
# the dependent varaible for each model.
formu <- ~ LASTAGE + MALE + RACE3 + beltuse + educate + marital + SREGION + POVSTALB

tpsa.smoke <- trips(nmes, nmes$smoke, formu)
head(tpsa.smoke)
#We'll plot 5% random triplets to get a sence of the matches
plot(tpsa.smoke, sample=c(.05), edge.alpha=.1)

# Using our second treatment varaible
tpsa.packyears <- trips(nmes, nmes$smoke2, formu)
head(tpsa.packyears)
plot(tpsa.packyears, sample=c(.05), edge.alpha=.1)

tmatch.smoke <- trimatch(tpsa.smoke, exact=nmes[,c("LastAge5","MALE","RACE3")], nmatch=10)
tmatch.packyears <- trimatch(tpsa.packyears, exact=nmes[,c("LastAge5","MALE","RACE3")], nmatch=10)

# Summary of unmatched rows
summary(unmatched(tmatch.smoke))
summary(unmatched(tmatch.packyears))

##### Checking Balance #####
#Effect size balance plot
multibalance.plot(tpsa.smoke)
multibalance.plot(tpsa.packyears)

bplots <- balance.plot(tmatch.smoke, nmes[,all.vars(formu)], 
					   legend.position="none", x.axis.angle=90)
plot(bplots, cols=3, byrow=TRUE, plot.sequence=c(3:8,1:2))

bplots2 <- balance.plot(tmatch.packyears, nmes[,all.vars(formu)], 
						legend.position="none", x.axis.angle=90)
plot(bplots2, cols=3, byrow=TRUE, plot.sequence=c(3:8,1:2))

sum.smoke <- summary(tmatch.smoke, nmes$LogTotalExp, 
					 ordering=c("Smoker","Former","Never"))
sum.packyears <- summary(tmatch.packyears, nmes$LogTotalExp, 
						 ordering=c("Heavy","Moderate","Never"))
print("Current Smoking Status"=sum.smoke, "Smoking Frequency"=sum.packyears)

sum.smoke$t.tests
sum.packyears$t.test

loess3.plot(tmatch.smoke, nmes$LogTotalExp, points.alpha=.01, method="loess", 
			ylab="log(Total Expenditures)")

loess3.plot(tmatch.packyears, nmes$LogTotalExp, points.alpha=.01, method="loess", 
			ylab="log(Total Expenditures)")

boxdiff.plot(tmatch.smoke, nmes$LogTotalExp, ordering=c("Smoker","Former","Never"))
boxdiff.plot(tmatch.packyears, nmes$LogTotalExp, ordering=c("Heavy","Moderate","Never"))

##### Alternative views of the Loess Plot #####
# We can use the propensity scores from other models.
# 0=Never, 1=Moderate, Heavy is imputed
loess3.plot(tmatch.packyears, nmes$LogTotalExp, points.alpha=.01, method="loess", 
			model=1, ylab="log(Total Expenditures)")
# 0=Never, 1=Heavy, Moderate is imputed
loess3.plot(tmatch.packyears, nmes$LogTotalExp, points.alpha=.01, method="loess", 
			model=2, ylab="log(Total Expenditures)")
# 0=Heavy, 1=Moderate, Never is imputed
loess3.plot(tmatch.packyears, nmes$LogTotalExp, points.alpha=.01, method="loess", 
			model=3, ylab="log(Total Expenditures)")

library(TriMatch)
library(grid)
data(tutoring)

names(tutoring)
table(tutoring$treat, useNA='ifany')
table(tutoring$treat, tutoring$Course, useNA='ifany')

formu <- ~ Gender + Ethnicity + Military + ESL + EdMother + EdFather + Age +
	       Employment + Income + Transfer + GPA

# tutoring$Treat1 <- !tutoring$treat == 'Control'
# lr1 <- glm(Treat1 ~ Gender + Ethnicity + Military + ESL + EdMother + EdFather + Age +
# 		   	Employment + Income + Transfer + GPA, data=tutoring, family=binomial)
# summary(lr1)
# lr2 <- stepAIC(lr1)
# summary(lr2)
# formu <- ~ Gender + Employment + Transfer
# multibalance.plot(tutoring.tpsa, cols=all.vars(formu))

# Estimate the propensity scores for the three models
tutoring.tpsa <- trips(tutoring, tutoring$treat, formu)

# Some useful information is saved as attributes.
names(attributes(tutoring.tpsa))

# Prints a combined summary of the three logistic regression models
summary(tutoring.tpsa)

# Triangle plot of the propensity scores
plot(tutoring.tpsa, sample=c(200))

# In the three cases below we will use exact matching on the Course.
# Default maximumTreat
tutoring.matched <- trimatch(tutoring.tpsa, exact=tutoring[,c('Course')]) 
# Caliper matching
tutoring.matched.caliper <- trimatch(tutoring.tpsa, exact=tutoring[,c('Course')], 
									 method=NULL)
# 2-to-1-to-1 matching
tutoring.matched.2to1 <- trimatch(tutoring.tpsa, exact=tutoring[,c('Course')], 
								  method=OneToN, M1=2, M2=1)
# 3-to-2-to-1 matching
tutoring.matched.3to2 <- trimatch(tutoring.tpsa, exact=tutoring[,c('Course')], 
								  method=OneToN, M1=3, M2=2)

# Examine unmatched cases.
summary(unmatched(tutoring.matched))
summary(unmatched(tutoring.matched.caliper))
summary(unmatched(tutoring.matched.2to1))
summary(unmatched(tutoring.matched.3to2))

# Examine the differences in standardized propensity scores for each matched triplet.
# The caliper parameter allows us to see how many matched triplets we would loose
# if we reduced the caliper.
# TODO: Currently doesn't work
# distances.plot(tutoring.matched, caliper=.2) 
# distances.plot(tutoring.matched.caliper, caliper=.2) 

# We can overlay matched triplets on the triangle plot
plot(tutoring.matched, rows=c(1), line.alpha=1, draw.segments=TRUE)


##### Checking balance #########################################################
multibalance.plot(tutoring.tpsa) + ggtitle('Covariate Balance Plot')

# Continuous covariate
balance.plot(tutoring.matched, tutoring$Age, label='Age')

# Discrete covariate
balance.plot(tutoring.matched, tutoring$Ethnicity)

# Create a grid of figures.
bplots <- balance.plot(tutoring.matched, tutoring[,all.vars(formu)], 
		legend.position='none', x.axis.labels=c('C','T1','T1'), x.axis.angle=0)
bplots[['Military']] # We can plot one at at time.
summary(bplots) # Create a data frame with the statistical results
plot(bplots, cols=3, byrow=FALSE)
#print(bplots, cols=3, byrow=FALSE) # Will call summary and plot

##### Phase II #################################################################
matched.out <- merge(tutoring.matched, tutoring$Grade)
names(matched.out)
head(matched.out)

(s1 <- summary(tutoring.matched, tutoring$Grade))
(s2 <- summary(tutoring.matched.caliper, tutoring$Grade))
(s3 <- summary(tutoring.matched.2to1, tutoring$Grade))
(s4 <- summary(tutoring.matched.3to2, tutoring$Grade))

# Loess plot for the optimal matching method
loess3.plot(tutoring.matched, tutoring$Grade, ylab='Grade')
# Draw lines connecting each matched triplet
loess3.plot(tutoring.matched, tutoring$Grade, ylab='Grade', 
			points.alpha=.5, plot.connections=TRUE)
# Loess plot for the caliper matching method
loess3.plot(tutoring.matched.caliper, tutoring$Grade, ylab='Grade', 
			points.alpha=.1, method='loess')
# Loess plot for the 2-to-1 matching method
loess3.plot(tutoring.matched.2to1, tutoring$Grade, ylab='Grade', span=.9)
# Loess plot for the 3-to-2 matching method
loess3.plot(tutoring.matched.3to2, tutoring$Grade, ylab='Grade', span=.9)

boxdiff.plot(tutoring.matched, tutoring$Grade, 
			 ordering=c('Treat2','Treat1','Control')) + 
	ggtitle('Boxplot of Differences: Optimal Matching')
boxdiff.plot(tutoring.matched.caliper, tutoring$Grade, 
			 ordering=c('Treat2','Treat1','Control')) +
	ggtitle('Boxplot of Differences: Caliper Matching')
boxdiff.plot(tutoring.matched.2to1, tutoring$Grade, 
			 ordering=c('Treat2','Treat1','Control')) +
	ggtitle('Boxplot of Differences: 2-to-1-to-n Matching')
boxdiff.plot(tutoring.matched.3to2, tutoring$Grade, 
			 ordering=c('Treat2','Treat1','Control')) +
	ggtitle('Boxplot of Differences: 3-to-2-to-n Matching')


print(print('Optimal'=s1, 'Caliper'=s2, '2-to-1'=s3, '3-to-2'=s4), row.names=FALSE)

require(TriMatch)
data(tutoring)

formu <- ~ Gender + Ethnicity + Military + ESL + EdMother + EdFather + Age +
	Employment + Income + Transfer + GPA

table(tutoring$treat, useNA='ifany')

tutoring.tpsa <- trips(tutoring, tutoring$treat, formu)

tutoring.matched.1to1 <- trimatch(tutoring.tpsa, exact=tutoring[,c("Course")], 
								  method=OneToN, M1=1, M2=1)

tutoring.matched.1to1 <- tutoring.matched.1to1[!duplicated(tutoring.matched.1to1[,1]),]
tutoring.matched.1to1 <- tutoring.matched.1to1[!duplicated(tutoring.matched.1to1[,2]),]
tutoring.matched.1to1 <- tutoring.matched.1to1[!duplicated(tutoring.matched.1to1[,3]),]

nrow(tutoring.matched.1to1)
length(unique(tutoring.matched.1to1$Treat1))
length(unique(tutoring.matched.1to1$Treat2))
length(unique(tutoring.matched.1to1$Control))

summary(tutoring.matched.1to1, tutoring$Grade)

plot(tutoring.matched.1to1)
boxdiff.plot(tutoring.matched.1to1, tutoring$Grade)

#' Returns rows from [trips()] that were not matched by [trimatch()].
#' 
#' This function returns a subset of [trips()] that were not matched by
#' [trimatch()]. All data frame methods work with the returned object
#' but special `summary` function will provided relevant information.
#' 
#' @param tmatch the results of [trimatch()].
#' @return a data frame of unmatched rows.
#' @export
unmatched <- function(tmatch) {
	tpsa <- attr(tmatch, 'triangle.psa')
	unmatched <- tpsa[!(tpsa$id %in% c(tmatch[,1], tmatch[,2], tmatch[,3])),]
	class(unmatched) <- c('unmatched', 'triangle.psa', 'data.frame')
	attr(unmatched, 'triangle.psa') <- tpsa
	return(unmatched)
}

#' Provides a summary of unmatched subjects.
#' 
#' Will return as a list and print the percentage of total unmatched rows and
#' percent by treatment.
#' 
#' @param object results of [unmatched()].
#' @param digits number of digits to print.
#' @param ... currently unused.
#' @return a list of summary results.
#' @export
#' @method summary unmatched
summary.unmatched <- function(object, digits=3, ...) {
	tpsa <- attr(object, 'triangle.psa')
	results <- list()
	results$PercentUnmatched <- nrow(object) / nrow(tpsa)
	results$UnmatchedByTreat <- table(object$treat)
	results$PercentByTreat <- table(object$treat) / table(tpsa$treat)
	results$Unmatched <- table(object$treat)
	cat(paste0(nrow(object), ' (',
			prettyNum(nrow(object) / nrow(tpsa) * 100, digits=digits), 
			'%) of ', nrow(tpsa), ' total data points were not matched.'))
	cat('\nUnmatched by treatment:\n')
	tmp <- paste0(results$UnmatchedByTreat, ' (', prettyNum(results$UnmatchedByTreat / 
							table(tpsa$treat) * 100, digits=digits), '%)')
	names(tmp) <- names(results$UnmatchedByTreat)
	print(tmp, quote=FALSE)
	invisible(results)
}

#' Calculate covariate effect size differences before and after stratification.
#' 
#' This function is modified from the [PSAgraphics::cv.bal.psa()] function in the 
#' `PSAgrpahics` package.
#' 
#' Note: effect sizes are calculated as treatment 1 - treatment 0, or treatment B - treatment A.
#' 
#' @param covariates dataframe of interest
#' @param treatment binary vector of 0s and 1s (necessarily? what if character, or 1, 2?)
#' @param propensity PS scores from some method or other.
#' @param strata either a vector of strata number for each row of covariate, or 
#'        one number n in which case it is attempted to group rows by ps scores into 
#'        n strata of size approximately 1/n.  This does not seem to work well in the 
#'        case of few specific propensity values, as from a tree.
#' @param int either a number m used to divide [0,1] into m equal length subintervals, 
#'        or a vector of cut points between 0 an    1 defining the subintervals (perhaps 
#'        as suggested by loess.psa).  In either case these subintervals define strata, 
#'        so strata can be of any size.  
#' @param tree logical, if unique ps scores are few, as from a recursively partitioned 
#'        tree, then TRUE will force each ps value to define a stratum.  
#' @param minsize smallest allowable stratum-treatment size.  If violated, strata is 
#'        removed.
#' @param universal.psd If 'TRUE', forces standard deviations used to be unadjusted 
#'        for stratification.  
#' @param trM trimming proportion for mean calculations.
#' @param absolute.es logical, if 'TRUE' routine uses absolute values of all effect sizes.
#' @param trt.value allows user to specify which value is active treatment, if desired.
#' @param use.trt.var logical, if true then Rubin-Stuart method using only treatment 
#'        variance with be used in effect size calculations. 
#' @param verbose logical, controls output that is visibly returned.  
#' @param xlim limits for the x-axis.
#' @param plot.strata logical indicating whether to print strata.
#' @param ... currently unused. 
#' @author Robert M. Pruzek RMPruzek@@yahoo.com
#' @author James E. Helmreich James.Helmreich@@Marist.edu
#' @author KuangNan Xiong harryxkn@@yahoo.com
#' @export
covariateBalance <- function(covariates, treatment, propensity, strata = NULL, 
                     int = NULL, tree = FALSE, minsize = 2, universal.psd = TRUE,
	                  trM = 0, absolute.es = TRUE, trt.value = NULL, 
	                  use.trt.var = FALSE, verbose = FALSE, xlim = NULL, 
	                  plot.strata = TRUE, ...) {
	X <- covariates
	
	treat.lev <- sort(unique(treatment))
	
	if(is.null(trt.value)) { 
		trt.value=treat.lev[2] 
	}
	if(!is.null(trt.value)) { 
		if((trt.value!=treat.lev[2]) & (trt.value!=treat.lev[1])) {
			stop("WARNING: trt.value as defined does not match a treatment level")
		} 
		if(trt.value==treat.lev[1]) {
			treat.lev <-treat.lev[c(2,1)]
		}
	}
	
	######################################################## BEGIN C
	# Call cstrat.psa for definitions of strata
	cstrat <- cstrata.psa(treatment = treatment, 
						  propensity = propensity, 
						  strata = strata, int = int, 
						  tree = tree, 
						  minsize = minsize, 
						  graphic = FALSE)
	shom <- cstrat$Original.Strata
	som <- cstrat$Used.Strata
	psct <- cstrat$strata
	
	XX <- cbind(X, treatment, psct)
	
	names.cov <- colnames(X) 
	n.cov <- length(names.cov) 
	names.strata <- colnames(som)
	n.strata <- length(names.strata) 
	######################################################## END C
	
	######################################################## BEGIN D
	#Calculating balance effect sizes for stratified covraiates 
	#Initializing matrices used later.
	
	uess = matrix(0, nrow = n.cov, ncol = 2) 
	effect.size.ji = matrix(0, nrow = n.cov, ncol = n.strata) 
	effect.size.ji.adj = matrix(0, nrow = n.cov, ncol = n.strata) 
	var.cov.by.strattreat = matrix(0, nrow = n.cov, ncol = 2*n.strata) 
	mean.diff <- matrix(0, nrow = n.cov, ncol = n.strata) 
	mean.diff.adj = matrix(0, nrow = n.cov, ncol = n.strata) 
	sd.adj <- matrix(0, nrow = n.cov, ncol = n.strata)
	sd.un <- rep(0, n.cov) 
	mean.diff.unadj <- rep(0, nrow = n.cov) 
	
	#mean.diff is the mean difference between tr/cr for each covariate across strata 
	#sd.adj is the PS-adjusted pooled standard deviation for each covariate across strata 
	#mean.diff.adj is the mean differences adjusted by strata size 
	#var.cov.by.strattreat is the variance for each cell in strata*tr/ct table 
	#################################################################Z
	for (j in 1:n.cov) {
		for (i in 1:n.strata) {      
			#ha  is (size ith strata by 2)-matrix that picks off values of jth 
			#covariate and treatment in ith stratum 
			ha = XX[XX[, n.cov + 2] == names.strata[i], c(j, n.cov + 1)]
			mean.diff[j,i] = (mean(ha[ha[, 2] == treat.lev[2], 1], trim = trM) - 
			                  mean(ha[ha[, 2] == treat.lev[1] , 1], trim = trM)) 
			mean.diff.adj[j,i] = mean.diff[j,i] * som[3, i]/sum(som[3, ])
			if(use.trt.var) {
				var.cov.by.strattreat[j, i] = var(ha[ha[, 2] == treat.lev[2], 1] )
				var.cov.by.strattreat[j, i + n.strata] = var(ha[ha[, 2] == treat.lev[2],1] ) 
				sd.adj[j,i]=sd( ha[ha[,2]==treat.lev[2],1] )  
			} else {
				var.cov.by.strattreat[j,i] = var( ha[ha[,2]==treat.lev[1],1]) 
				var.cov.by.strattreat[j, i + n.strata] = var( ha[ha[,2]==treat.lev[2],1] ) 
				sd.adj[j,i] = sqrt((var.cov.by.strattreat[j, i] + 
										var.cov.by.strattreat[j, i + n.strata])/2)
			} 
	     }
		# uess[j,1] contains unadjusted ES for jth covariate by direct calculation 
		# mean.diff.unadj and sd.un are mean.diff and sd.adj for each covariate without 
		# propensity score adjustment. 
		mean.diff.unadj[j] = (mean(XX[XX[, n.cov+1] == treat.lev[2], j], trim = trM) - 
		                      mean(XX[XX[, n.cov+1] == treat.lev[1], j], trim = trM)) 
		if (use.trt.var) {
			sd.un[j] = sd(XX[XX[, n.cov + 1] == treat.lev[2] ,j ])
		} else {
			sd.un[j] = sqrt((var(XX[XX[, n.cov+1] == treat.lev[1],j]) + 
		                     var(XX[XX[, n.cov+1] == treat.lev[2], j]))/2) 
		}
		uess[j,1] = if(sd.un[j] > 0){ mean.diff.unadj[j] / sd.un[j] } else { 0 } 
	   
		#effect.size.ji provides the effect size mean.diff/sd.adj for each stratum for each covariate; 
		#these are shown as letters on graphic. 
		#effect.size.ji.adj is the middle step to calculate uess[j,2] which is the sum of mean.diff/psd 
		#in all strata 
		#weighted by strata sizes.  
		#uess[j,2] is going to be shown as weighted-avg of above dots from effect.sizeji 
		#in the final graphic. 
		
		if (universal.psd == TRUE) {
			sd.adj[j,] = sd.un[j]
		}
		for (i in 1:n.strata) {
			effect.size.ji[j,i] = if(sd.adj[j,i] > 0) {mean.diff[j,i]/sd.adj[j,i]}else{0} 
			effect.size.ji.adj[j,i] = if(sd.adj[j,i] > 0) {mean.diff.adj[j,i]/sd.adj[j,i]}else{0} 
		} 
		uess[j,2] = sum(effect.size.ji.adj[j,]) 
	}
	#####################################################################Z
	
	#Name dimensions of everything.
	 
	n.strata2 = n.strata*2
	sd.un <- matrix(sd.un, ncol = 1)
	rownames(sd.un) <- names.cov
	dimnames(uess) = list(names.cov, c("stES_unadj", "stES_adj")) 
	dimnames(mean.diff) = list(names.cov, names.strata) 
	dimnames(mean.diff.adj) = list(names.cov, names.strata) 
	dimnames(effect.size.ji) = list(names.cov, names.strata) 
	mean.diff.unadj <- matrix(mean.diff.unadj, ncol = 1)
	dimnames(mean.diff.unadj) = list(names.cov, "mean.diff_unadj") 
	dimnames(var.cov.by.strattreat) = list(names.cov, paste("cellvar", 1:n.strata2)) 
	
	#when absolute.es is set as true, take absolute values.
	if (absolute.es == TRUE) {
		effect.size.ji = abs(effect.size.ji) 
		mean.diff = abs(mean.diff) 
		mean.diff.adj = abs(mean.diff.adj) 
		mean.diff.unadj = abs(mean.diff.unadj) 
		uess = abs(uess) 
	} 
	
	se = order(uess[,1]) 
	se2 = order(uess[,1], decreasing = TRUE)
	sd.un = as.matrix(sd.un[se2, ])
	colnames(sd.un) <- "st.dev.unadj"
	ord.uess = uess[se,] 
	ord.uess.2 = uess[se2,] 
	#matrix is ordered according to unadjusted ES values 
	effect.size.ji1 = effect.size.ji[se, ] 
	effect.size.ji2 = effect.size.ji[se2, ] 
	mean.diff.adj = mean.diff.adj[se2, ] 
	mean.diff.unadj = mean.diff.unadj[se2, ] 
	#sd.adj.3 = sd.adj.3[se2, ]
	var.cov.by.strattreat = var.cov.by.strattreat[se2, ]
	mean.diff = mean.diff[se2, ] 
	colnames(effect.size.ji2) =  letters[1:n.strata] 
	colnames(som) = letters[1:n.strata]
	colnames(mean.diff) = letters[1:n.strata]
	colnames(mean.diff.adj) = letters[1:n.strata]
	colnames(var.cov.by.strattreat) = c(paste(letters[1:n.strata], "_", treat.lev[1], sep = ""), 
	         paste(letters[1:n.strata], "_", treat.lev[2], sep = ""))
	# final matrix contains all the final ES values as well as stratum-specific values for plotting.
	final = cbind(ord.uess, effect.size.ji1) 
	######################################################## END D
	
	####################### OUTPUT ################################
	sd.ESs = apply(effect.size.ji2, 1, sd) 
	final2 = round(cbind(ord.uess.2, effect.size.ji2, sd.ESs), 2) 
	out <- list(shom, som, round(mean.diff.adj,2), round(mean.diff.unadj,2),
	       final2, treat.lev, round(cbind(sd.un,var.cov.by.strattreat), 2)) 
	names(out)<-c("original.strata", "strata.used", "mean.diff.strata.wtd", 
	              "mean.diff.unadj", "effect.sizes", "treatment.levels", "effects.strata.treatment")     
	if(verbose){return(out)}else{return(invisible(out))}
}

#' Estimates propensity scores for three groups
#' 
#' The propensity score is \deqn{e(X)=P({ W }=1|X)}
#' This function will estimate the propensity scores for each pair of groups
#' (e.g. two treatments and one control).
#' 
#' \deqn{{ PS }_{ 1 }=e({ X }_{ { T }_{ 1 }C })=Pr(z=1|{ X }_{ { T }_{ 1 }C })}
#' 
#' \deqn{{ PS }_{ 2 }=e({ X }_{ { T }_{ 2 }C })=Pr(z=1|{ X }_{ { T }_{ 2 }C })}
#' 
#' \deqn{{ PS }_{ 3 }=e({ X }_{ { T }_{ 2 }{ T }_{ 1 } })=Pr(z=1|{ X }_{ { T }_{ 2 }{ T }_{ 1 } })}
#' 
#' @param thedata the data frame.
#' @param treat vector or factor indicating the treatment/control assignment for
#'        \code{thedata}. Length must be equal to \code{nrow(thedata)}.
#' @param formu the logistic regression formula. Note that the dependent variable
#'        should not be specified and will be modified.
#' @param groups a vector of exactly length three corresponding the values in
#'        \code{treat} for each control/treatment.
#' @param nstrata the number of strata marks to plot on the edge.
#' @param method the method to use to estimate the propensity scores. Current options are logistic
#'        or randomForest.
#' @param ... other parameters passed to [stats::glm()].
#' @export
#' @examples
#' \dontrun{
#' data(tutoring)
#' formu <- ~ Gender + Ethnicity + Military + ESL + EdMother + EdFather + Age +
#'      Employment + Income + Transfer + GPA
#' tpsa <- trips(tutoring, tutoring$treat, formu)
#' head(tpsa)
#' }
trips <- function(thedata, treat, formu = ~ ., groups=unique(treat), nstrata=5, 
				  method = 'logistic', ...) {
	if(length(groups) != 3) stop('Sorry, exactly three groups are required.')
	if(nrow(thedata) != length(treat)) stop('length(treat) does not equal nrow(thedata)')
	
	treats <- data.frame(id=1:length(treat), treat=treat, 
						 model1=rep(as.logical(NA), length(treat)),
						 model2=rep(as.logical(NA), length(treat)),
						 model3=rep(as.logical(NA), length(treat)),
						 ps1=rep(as.numeric(NA), length(treat)),
						 ps2=rep(as.numeric(NA), length(treat)),
						 ps3=rep(as.numeric(NA), length(treat)) )
	treats[which(treats$treat == groups[1]), c('model1','model2')] <- FALSE
	treats[which(treats$treat == groups[2]), c('model1','model3')] <- TRUE
	treats[which(treats$treat == groups[3]), c('model2')] <- TRUE
	treats[which(treats$treat == groups[3]), c('model3')] <- FALSE
	
	rows1 <- which(!is.na(treats$model1))
	rows2 <- which(!is.na(treats$model2))
	rows3 <- which(!is.na(treats$model3))
	if(method == 'logistic') {
		message('Using logistic regression to estimate propensity scores...')
		
		m1 <- glm(update(formu, treat ~ .),
				  data=cbind(thedata[rows1,], treat=treats[rows1,]$model1),
				  family='binomial', ...)
		treats[rows1,]$ps1 <- fitted(m1)
		
		m2 <- glm(update(formu, treat ~ .),
				  data=cbind(thedata[rows2,], treat=treats[rows2,]$model2),
				  family='binomial', ...)
		treats[rows2,]$ps2 <- fitted(m2)

		m3 <- glm(update(formu, treat ~ .),
				  data=cbind(thedata[rows3,], treat=treats[rows3,]$model3),
				  family='binomial', ...)
		treats[rows3,]$ps3 <- fitted(m3)
	} else if(method == 'randomForest') {
		message('Using random forests to estimate propensity scores...')

		m1 <- randomForest(update(formu, treat ~ .), 
						   data=cbind(thedata[rows1,], treat=factor(treats[rows1,]$model1)), ...)
		treats[rows1,]$ps1 <- predict(m1, type='prob')[,2]

		m2 <- randomForest(update(formu, treat ~ .), 
						   data=cbind(thedata[rows2,], treat=factor(treats[rows2,]$model2)), ...)
		treats[rows2,]$ps2 <- predict(m2, type='prob')[,2]
		
		m3 <- randomForest(update(formu, treat ~ .), 
						   data=cbind(thedata[rows3,], treat=factor(treats[rows3,]$model3)), ...)
		treats[rows3,]$ps3 <- predict(m3, type='prob')[,2]
	} else {
		stop(paste0('Unsupported method: ', method))
	}
	
	breaks1 <- quantile(treats$ps1, probs=seq(0,1,1/nstrata), na.rm=TRUE)
	breaks2 <- quantile(treats$ps2, probs=seq(0,1,1/nstrata), na.rm=TRUE)
	breaks3 <- quantile(treats$ps3, probs=seq(0,1,1/nstrata), na.rm=TRUE)
	
	treats$strata1 <- cut(treats$ps1, breaks=breaks1, labels=1:nstrata, include.lowest=TRUE)
	treats$strata2 <- cut(treats$ps2, breaks=breaks2, labels=1:nstrata, include.lowest=TRUE)
	treats$strata3 <- cut(treats$ps3, breaks=breaks3, labels=1:nstrata, include.lowest=TRUE)
	
	class(treats) <- c('triangle.psa', 'data.frame')
	attr(treats, 'data') <- thedata
	attr(treats, 'nstrata') <- nstrata
	attr(treats, 'model1') <- m1
	attr(treats, 'model2') <- m2
	attr(treats, 'model3') <- m3
	attr(treats, 'breaks1') <- breaks1
	attr(treats, 'breaks2') <- breaks2
	attr(treats, 'breaks3') <- breaks3
	attr(treats, 'groups') <- groups
	
	return(treats)
}

#' Returns significance level.
#' 
#' Returns the significance level as stars, or NA if a non-numeric value is
#' passed in.
#' 
#' @param x p-value.
star <- function(x) {
	if(is.numeric(x)) {
		return(cut(x, 
				   breaks=c(-Inf,.001,.01,.05,.1,Inf), 
				   labels=c('***','**','*','.',''))
		)
	} else {
		return(NA)
	}
}

#' Merges outcomes with the matched set.
#' 
#' The \code{y} parameter should be a subset of the original data used.
#' 
#' @param x the result of [trimatch()]
#' @param y another data frame or vector to merge with.
#' @param ... unused
#' @return \code{x} with the additional column(s) added.
#' @export
#' @method merge triangle.matches
merge.triangle.matches <- function(x, y, ...) {
	stopifnot('triangle.matches' %in% class(x))
	tpsa <- attr(x, 'triangle.psa')
	
	if(is.vector(y)) {
		if(length(y) != nrow(tpsa)) {
			stop('The length of the vector does not match that of the original data')
		}
		x <- cbind(x, y[as.integer(x[,1])])
		x <- cbind(x, y[as.integer(x[,2])])
		x <- cbind(x, y[as.integer(x[,3])])
		names(x)[(ncol(x)-2):ncol(x)] <- paste(names(x)[1:3], '.out', sep='')
	} else {
		if(nrow(y) != nrow(tpsa)) {
			stop('The length of the vector does not match that of the original data')
		}
		x <- cbind(x, y[as.integer(x[,1]),])
		x <- cbind(x, y[as.integer(x[,2]),])
		x <- cbind(x, y[as.integer(x[,3]),])
		names(x)[(ncol(x)-(3*ncol(y))+1):ncol(x)] <- paste(rep(names(x)[1:3], each=ncol(y)), 
						names(y), sep='.')
	}
	return(x)
}

utils::globalVariables(c('value','covariate','variable','model','group'))

#' Multiple covariate balance assessment plot.
#' 
#' A graphic based upon [cv.bal.psa()] function in the `PSAgraphics`
#' package. This graphic plots the effect sizes for multiple covariates before and
#' after propensity score adjustment.
#' 
#' @param tpsa results of [trips()].
#' @param tmatch results of [trimatch()].
#' @param grid if TRUE, then a grid of three plots for each model will be displayed.
#' @param cols character vector of covariates (i.e. column names) from the original 
#'        data to include in the plot. By default all covariates used in the
#'        logistic regression model are used.
#' @return a \code{ggplot2} figure.
#' @export
multibalance.plot <- function(tpsa, tmatch, grid=TRUE, cols) {
	if(!missing(tmatch)) {
		tpsa <- attr(results, 'triangle.psa', exact=TRUE)
	}
	
	covs <- attr(tpsa, 'data')
	m1 <- attr(tpsa, 'model1')
	if(missing(cols)) {
		cols <- attr(m1$terms, 'term.labels')
	}
	covs <- covs[,cols]
	
	#Recode factors. First we'll covert logicals and factors with two levels to integers
	for(i in 1:ncol(covs)) {
		if(is.logical(covs[,i])) {
			covs[,i] <- as.integer(covs[,i])
		} else if(is.factor(covs[,i]) & length(levels(covs[,i])) == 2) {
			covs[,i] <- as.integer(covs[,i])
		}
	}
	if('factor' %in% sapply(covs, class)) {
		#Convert remaining factors using cv.trans.psa from PSAgraphics
		#covs <- as.data.frame(cv.trans.psa(covs))
		covs <- cv.trans.psa(covs)[[1]]
		#names(covs) <- gsub('covariates.transformed.', '', names(covs))
	}
	
	tpsa2 <- cbind(tpsa, (covs))
	
	if(!missing(tmatch)) {
		rows <- c(tmatch$Control, tmatch$Treat1, tmatch$Treat2)
		tpsa2 <- tpsa2[rows,]
	} else {
		warning('Balance estimates may include observations not in the matched dataset. It is 
				recommended that the tmatch parameter is specified instead.')
	}
	
	results <- data.frame(covariate=character(), model=integer(), unadjusted=numeric(),
						  adjusted=numeric(), stringsAsFactors=FALSE)
	for(i in 1:3) {
		m <- tpsa2[!is.na(tpsa2[,paste('model', i, sep='')]),]
		
		bal <- covariateBalance(m[,names(m) %in% names(covs)], 
						  m[,paste('model', i, sep='')], 
						  m[,paste('ps', i, sep='')],
						  m[,paste('strata', i, sep='')])
		results <- rbind(results, data.frame(
			covariate = row.names(bal$effect.sizes),
			model = rep(i, ncol(covs)),
			unadjusted = bal$effect.sizes[,'stES_unadj'],
			adjusted = bal$effect.sizes[,'stES_adj'],
			stringsAsFactors = FALSE
		))
	}
	
	row.names(results) <- 1:nrow(results)
	results <- melt(results, id.vars=c('covariate','model'))
	results$group <- paste(results$variable, results$model, sep='-')
	
	results <- results[rev(order(results$model, results$covariate)),]
	
	results$covariate <- factor(results$covariate, ordered=TRUE)
	
	p <- ggplot(results, 
		aes(x=value, y=covariate, color=variable, shape=factor(model), linetype=factor(model))) + 
		geom_point() + geom_path(alpha=.5, aes(group=group)) +
		ylab('Covariate') + xlab('Effect Size') +
		scale_color_hue('Adjustment') + scale_linetype('Model') + scale_shape('Model')
	if(grid) {
		p <- p + facet_grid(~ model)
	}
	
	return(p)
}

utils::globalVariables(c('y','id'))

#' Triangle plot drawing matched triplets.
#' 
#' This plot function adds a layer to [plot.triangle.psa()] drawing 
#' matched triplets. If `p` is supplied, this function will simply draw on
#' top of the pre-existing plot, otherwise [plot.triangle.psa()] will
#' be called first.
#' 
#' If this function calls [plot.triangle.psa()], it will only draw
#' line segments and points for those data rows that were used in the matching
#' procedure. That is, data elements not matched will be excluded from the
#' figure. To plot all segments and points regardless if used in matching, set
#' `p = plot(tpsa)`.
#' 
#' @param x matched triplets from [triangle.match()].
#' @param sample an number between 0 and 1 representing the percentage of matched
#'        triplets to draw.
#' @param rows an integer vector corresponding to the rows in `tmatch` to draw.
#' @param line.color the line color.
#' @param line.alpha the alpha for the lines.
#' @param point.color color of matched triplet points.
#' @param point.size point size for matched triplets.
#' @param p a ggplot to add the match lines. If NULL, then [plot.triangle.psa()].
#' @param ... other parameters passed to [plot.triangle.psa()].
#' @return a `ggplot2` graphic.
#' @seealso plot.triangle.psa
#' @seealso triangle.match
#' @method plot triangle.matches
#' @export
plot.triangle.matches <- function(x, 
								  sample=.05, 
								  rows=sample(nrow(tmatch), nrow(tmatch) * sample), 
								  line.color='black',
								  line.alpha=.5,
								  point.color='black',
								  point.size=3,
								  p, ...) {
	tmatch <- x
	tpsa <- attr(tmatch, 'triangle.psa')
	pts.overlay <- data.frame(x=numeric(), y=numeric(), id=integer())
	for(i in rows) {
		tmp <- tpsa[as.integer(tmatch[i,1:3]),]
		tmp$id <- i
		tmp <- melt(tmp[,c('id','ps1','ps2','ps3')], id.vars='id')
		tmp <- tmp[!is.na(tmp$value),]
		tmp <- rbind(
			as.data.frame(matrix(unlist(lapply(tmp[which(tmp$variable == 'ps1'),]$value, segment1)), 
								 ncol=2, byrow=TRUE)),
			cbind(tmp[which(tmp$variable == 'ps2'),]$value, rep(0, 2)),
			as.data.frame(matrix(unlist(lapply(tmp[which(tmp$variable == 'ps3'),]$value, segment2)), 
								 ncol=2, byrow=TRUE))
		)
		tmp <- tmp[c(1:4,6,5),]
		tmp <- rbind(tmp, tmp[1,])
		tmp$id <- i
		pts.overlay <- rbind(pts.overlay, tmp)
	}
	names(pts.overlay) <- c('x','y','id')
	if(missing(p)) {
		if(length(which(names(as.list(match.call(expand.dots=TRUE))) == 'draw.segments')) == 0) {
			p <- plot(tpsa, draw.segments=FALSE, ...)
		} else {
			tpsa2 <- tpsa[which(tpsa$id %in% unlist(c(tmatch[,1:3]))),]
			p <- plot(tpsa2, ...)
		}
	}
	p <- p + geom_path(data=pts.overlay, aes(x=x, y=y, group=id), 
					   alpha=line.alpha, colour=line.color) +
		geom_point(data=pts.overlay, aes(x=x, y=y), colour=point.color, size=point.size)
	return(p)
}	

#' This packages provides functions to estimate and visualize propensity
#' score analyses including matching for non-binary treatments.
#' 
#' @name TriMatch-package
#' @aliases TriMatch
#' @title Propensity Score Analysis for Non-Binary Treatments
#' @author Jason Bryer \email{jason@@bryer.org}
#' @keywords propensity score analysis psa matching
#' @seealso \code{PSAgraphics} \code{multilevelPSA}
#' @import PSAgraphics
#' @importFrom psych describe
#' @importFrom psych describeBy
#' @import compiler
#' @import stats
#' @import gridExtra
#' @import ggplot2
#' @import reshape2
#' @importFrom graphics plot
#' @importFrom scales percent
#' @importFrom grid grob editGrob vpPath viewport vpTree grid.layout getGrob gTree
#'             grobWidth grobHeight pushViewport grid.draw upViewport grid.newpage
#' @importFrom randomForest randomForest
"_PACKAGE"

#' Results from a study examining the effects of tutoring services on course grades.
#' 
#' \itemize{
#' \item \code{treat} Treatment indicator.
#' \item \code{Course} The course id the student was enrolled in.
#' \item \code{Grade} The course grade the student earned (4=A, 3=B, 2=C, 1=D, 0=F or W).
#' \item \code{Gender} Gender of the student.
#' \item \code{Ethnicity} Ethnicity of the student, either White, Black, or Other.
#' \item \code{Military} Is the student an active military student.
#' \item \code{ESL} English second language student.
#' \item \code{EdMother} Education level of the mother (1 = did not finish high school;
#' 2 = high school grad; 3 = some college; 4 = earned associate degree;
#' 5 = earned baccalaureate degree; 6 = Earned Master's degree; 7 = earned doctorate).
#' \item \code{EdFather} Education level of the father (levels same as EdMother).
#' \item \code{Age} Age at the start of the course.
#' \item \code{Employment} Employment level at college enrollment (1 = No; 2 = part-time;
#' 3 = full-time).
#' \item \code{Income} Household income level at college enrollment (1 = <25K; 2 = <35K; 3 = <45K;
#' 4 = <55K; 5 = <70K; 6 = <85K; 7 = <100K; 8 = <120K; 9 = >120K).
#' \item \code{Transfer} Number of transfer credits at the start of the course.
#' \item \code{GPA} GPA as of the start of the course.
#' \item \code{GradeCode} Letter grade.
#' \item \code{Level} Level of the course, either Lower or Upper.
#' \item \code{ID} Randomly assigned student ID.
#' }
#'
#' @name tutoring
#' @docType data
#' @format a data frame with 17 variables.
#' @keywords datasets
NULL

#' Results from the 1987 National Medical Expenditure Study
#' 
#' This file was originally prepared by Anders Corr (corr@@fas.harvard.edu) who
#' reports on December 8, 2007 that the resulting numbers closely
#' match with those reported in the published article. It was later modified by
#' Jason Bryer (jason@@bryer.org) to an R data object to be included in this 
#' package. See \url{http://imai.princeton.edu/research/pscore.html} for more
#' information
#'
#' @name nmes
#' @docType data
#' @format a data frame with 9,708 observations of 12 variables.
#' @keywords datasets
#' @source http://imai.princeton.edu/research/pscore.html
#' @author United States Department of Health and Human Services. 
#'         Agency for Health Care Policy and Research
#' @references National Center For Health Services Research, 1987. National 
#'         Medical Expenditure Survey. Methods II. Questionnaires and data 
#'         collection methods for the household survey and the Survey of 
#'         American Indians and Alaska Natives. National Center for Health 
#'         Services Research and Health Technology Assessment.
#'         
#'         Imai, K., & van Dyk, D.A. (2004). Causal Inference With General 
#'         Treatment Regimes: Generalizing the Propensity Score, Journal of the 
#'         American Statistical Association, 99(467), pp. 854-866.
#'         
#'         Elizabeth Johnson, E., Dominici, F., Griswold, M., & Zeger, S.L. (2003).
#'         Disease cases and their medical costs attributable to smoking: An 
#'         analysis of the national medical expenditure survey. Journal of 
#'         Econometrics, 112.
NULL

#' @inherit as.data.frame.list
#' @export
data.frame.to.list <- function(...) {
	as.data.frame.list(...)
}

#' Convert a list of vectors to a data frame.
#' 
#' This function will convert a list of vectors to a data frame. This function
#' will handle three different types of lists of vectors. First, if all the elements
#' in the list are named vectors, the resulting data frame will have have a number
#' of columns equal to the number of unique names across all vectors. In cases
#' where some vectors do not have names in other vectors, those values will be
#' filled with \code{NA}.
#' 
#' The second case is when all the vectors are of the same length. In this case,
#' the resulting data frame is equivalent to applying \code{rbind} across all elements.
#' 
#' The third case handled is when there are varying vector lengths and not all the
#' vectors are named. This condition should be avoided. However, the function will
#' attempt to convert this list to a data frame. The resulting data frame will have
#' a number of columns equal to the length of the longest vector. For vectors with
#' length less than this will fill the row with \code{NA}s. Note that this function
#' will print a warning if this condition occurs.
#' 
#' @author Jason Bryer \href{mailto:jason@@bryer.org}{jason@@bryer.org}
#' @references \url{http://stackoverflow.com/questions/4227223/r-list-to-data-frame}
#' @param x a list to convert to a data frame.
#' @param row.names a vector equal to \code{length(x)} corresponding to the row names.
#'        If \code{NULL}, the row names will be set to \code{names(x)}.
#' @param optional not used.
#' @param ... other parameters passed to [base::data.frame()].
#' @return a data frame.
#' @method as.data.frame list
#' @examples
#'     test1 <- list( c(a='a',b='b',c='c'), c(a='d',b='e',c='f'))
#'     as.data.frame(test1)
#'     
#'     test2 <- list( c('a','b','c'), c(a='d',b='e',c='f'))
#'     as.data.frame(test2)
#'     
#'     test3 <- list('Row1'=c(a='a',b='b',c='c'), 'Row2'=c(var1='d',var2='e',var3='f'))
#'     as.data.frame(test3)
#'     
#'     \dontrun{
#'     #This will print a warning.
#'     test4 <- list('Row1'=letters[1:5], 'Row2'=letters[1:7], 'Row3'=letters[8:14])
#'     as.data.frame(test4)
#'     }
#'     
#'     test5 <- list(letters[1:10], letters[11:20])
#'     as.data.frame(test5)
#'     
#'     \dontrun{
#'     #This will throw an error.
#'     test6 <- list(list(letters), letters)
#'     as.data.frame(test6)
#'     }
as.data.frame.list <- function(x, row.names=NULL, optional=FALSE, ...) {
	if(!all(unlist(lapply(x, class)) %in% 
				c('raw','character','complex','numeric','integer','logical'))) {
		warning('All elements of the list must be a vector.')
		NextMethod(x, row.names=row.names, optional=optional, ...)
	}
	allequal <- all(unlist(lapply(x, length)) == length(x[[1]]))
	havenames <- all(unlist(lapply(x, FUN=function(x) !is.null(names(x)))))
	if(havenames) { #All the vectors in the list have names we can use
		colnames <- unique(unlist(lapply(x, names)))
		df <- data.frame(matrix(
			unlist(lapply(x, FUN=function(x) { x[colnames] })),
			nrow=length(x), byrow=TRUE))
		names(df) <- colnames
	} else if(allequal) { #No names, but are of the same length
		df <- data.frame(matrix(unlist(x), nrow=length(x), byrow=TRUE), ...)
		hasnames <- which(unlist(lapply(x, FUN=function(x) !is.null(names(x)))))
		if(length(hasnames) > 0) { #We'll use the first element that has names
			names(df) <- names(x[[ hasnames[1] ]])
		}
	} else { #No names and different lengths, we'll make our best guess here!
		warning(paste("The length of vectors are not the same and do not ",
					"are not named, the results may not be correct.", sep=''))
		#Find the largest
		lsizes <- unlist(lapply(x, length))
		start <- which(lsizes == max(lsizes))[1]
		df <- x[[start]]
		for(i in (1:length(x))[-start]) {
			y <- x[[i]]
			if(length(y) < length(x[[start]])) {
				y <- c(y, rep(NA, length(x[[start]]) - length(y)))
			}
			if(i < start) {
				df <- rbind(y, df)
			} else {
				df <- rbind(df, y)
			}
		}
		df <- as.data.frame(df, row.names=1:length(x))
		names(df) <- paste('Col', 1:ncol(df), sep='')
	}
	if(missing(row.names)) {
		row.names(df) <- names(x)
	} else {
		row.names(df) <- row.names
	}
	return(df)
}

# *Note that this adapted from the `ez` R package available here: https://github.com/mike-lawrence/ez
# It appears the `ez` package is no longer maintained so the function was copied here.*
ezANOVA_main <-	function(
		data,
		dv,
		wid,
		within,
		within_full,
		within_covariates,
		between,
		between_covariates,
		observed,
		diff,
		reverse_diff,
		type = 1,
		return_aov,
		white.adjust) {
	if (inherits(data, "tbl_df")) {
		data <- as.data.frame(data)
	}
	vars = as.character(c(dv, wid, between, within, diff, within_full))
	for (var in vars) {
		if (!(var %in% names(data))) {
			stop(paste(
				'"',
				var,
				'" is not a variable in the data frame provided.',
				sep = ''
			))
		}
	}
	if (is.null(within) & is.null(between)) {
		stop(
			'is.null(within) & is.null(between)\nYou must specify at least one independent variable.'
		)
	}
	if (!is.data.frame(data)) {
		stop('"data" must be a data frame.')
	}
	if (!is.numeric(data[, names(data) == dv])) {
		stop('"dv" must be numeric.')
	}
	if (!is.factor(data[, names(data) == wid])) {
		warning(
			paste('Converting "', wid, '" to factor for ANOVA.', sep = ''),
			immediate. = TRUE,
			call. = FALSE
		)
		data[, names(data) == wid] = factor(data[, names(data) == wid])
	} else{
		if (length(unique(data[, names(data) == wid])) != length(levels(data[, names(data) ==
																			 wid]))) {
			warning(
				paste(
					'You have removed one or more Ss from the analysis. Refactoring "',
					wid,
					'" for ANOVA.',
					sep = ''
				),
				immediate. = TRUE,
				call. = FALSE
			)
			data[, names(data) == wid] = factor(data[, names(data) == wid])
		}
	}
	within_numeric = rep(FALSE, length(within))
	if (!is.null(within)) {
		for (i in 1:length(within)) {
			if (is.numeric(data[, names(data) == within[i]])) {
				warning(
					paste(
						'"',
						within[i],
						'" will be treated as numeric.',
						sep = ''
					),
					immediate. = TRUE,
					call. = FALSE
				)
				within_numeric[i] = TRUE
			} else{
				if (!is.factor(data[, names(data) == within[i]])) {
					warning(
						paste(
							'Converting "',
							within[i],
							'" to factor for ANOVA.',
							sep = ''
						),
						immediate. = TRUE,
						call. = FALSE
					)
					data[, names(data) == within[i]] = factor(data[, names(data) == within[i]])
				}
				if (length(unique(data[, names(data) == within[i]])) != length(levels(data[, names(data) ==
																						   within[i]]))) {
					warning(
						paste(
							'You have removed one or more levels from variable "',
							within[i],
							'". Refactoring for ANOVA.',
							sep = ''
						),
						immediate. = TRUE,
						call. = FALSE
					)
					data[, names(data) == within[i]] = factor(data[, names(data) == within[i]])
				}
				if (length(levels(data[, names(data) == within[i]])) == 1) {
					stop(paste('"', within[i], '" has only one level."', sep = ''))
				}
			}
		}
	}
	between_numeric = rep(FALSE, length(between))
	if (!is.null(between)) {
		for (i in 1:length(between)) {
			if (is.numeric(data[, names(data) == between[i]])) {
				warning(
					paste(
						'"',
						between[i],
						'" will be treated as numeric.',
						sep = ''
					),
					immediate. = TRUE,
					call. = FALSE
				)
				between_numeric[i] = TRUE
			} else{
				if (!is.factor(data[, names(data) == between[i]])) {
					warning(
						paste(
							'Converting "',
							between[i],
							'" to factor for ANOVA.',
							sep = ''
						),
						immediate. = TRUE,
						call. = FALSE
					)
					data[, names(data) == between[i]] = factor(data[, names(data) == between[i]])
				}
				if (length(unique(data[, names(data) == between[i]])) != length(levels(data[, names(data) ==
																							between[i]]))) {
					warning(
						paste(
							'You have removed one or more levels from variable "',
							between[i],
							'". Refactoring for ANOVA.',
							sep = ''
						),
						immediate. = TRUE,
						call. = FALSE
					)
					data[, names(data) == between[i]] = factor(data[, names(data) == between[i]])
				}
				if (length(levels(data[, names(data) == between[i]])) == 1) {
					stop(paste('"', between[i], '" has only one level."', sep = ''))
				}
			}
		}
		temp = ddply(idata.frame(data)
					 , structure(as.list(c(
					 	wid, between
					 )), class = 'quoted')
					 , function(x) {
					 	to_return = 0
					 	return(to_return)
					 })
		wid_temp = data.frame(table(temp[, names(temp) == wid]))
		if (any(wid_temp$Freq > 1)) {
			warning(
				paste(
					'The column supplied as the wid variable contains non-unique values across levels of the supplied between-Ss variables. Automatically fixing this by generating unique wid labels.',
					sep = ''
				),
				immediate. = TRUE,
				call. = FALSE
			)
			data[, names(data) == wid] = as.character(data[, names(data) == wid])
			for (i in unique(as.character(between))) {
				data[, names(data) == wid] = paste(data[, names(data) == wid], data[, names(data) ==
																						i])
			}
			data[, names(data) == wid] = factor(data[, names(data) == wid])
			temp = ddply(idata.frame(data)
						 , structure(as.list(c(
						 	wid, between
						 )), class = 'quoted')
						 , function(x) {
						 	to_return = 0
						 	return(to_return)
						 })
		}
		temp = ddply(temp
					 , structure(as.list(c(between)), class = 'quoted')
					 , function(x) {
					 	to_return = data.frame(N = nrow(x))
					 	return(to_return)
					 })
		balanced = ifelse(length(unique(temp$N)) > 1
						  , FALSE
						  , TRUE)
		if (!balanced) {
			warning(
				'Data is unbalanced (unequal N per group). Make sure you specified a well-considered value for the type argument to ezANOVA().',
				immediate. = TRUE,
				call. = FALSE
			)
		}
	}
	if (!is.null(diff)) {
		if (length(diff) > 1) {
			stop(paste('Provide only one value to "diff".', sep = ''))
		}
		if (!is.factor(data[, names(data) == diff])) {
			warning(
				paste(
					'Converting "',
					diff,
					'" to factor for computing difference score.',
					sep = ''
				),
				immediate. = TRUE,
				call. = FALSE
			)
			data[, names(data) == diff] = factor(data[, names(data) == diff])
		}
		temp <- ddply(idata.frame(data)
					  , structure(as.list(c(wid, diff)), class = 'quoted')
					  , function(x) {
					  	to_return = 0
					  	return(to_return)
					  })
		if (!all(as.data.frame(table(temp[, names(temp) %in% c(wid, within)]))$Freq ==
				 2)) {
			stop(
				paste(
					'Variable supplied to "diff" ("',
					as.character(diff),
					'") does not appear to be a within variable.',
					sep = ''
				)
			)
		}
		data[, names(data) == as.character(diff)] = factor(data[, names(data) ==
																	as.character(diff)])
		if (length(unique(data[, names(data) == as.character(diff)])) != 2) {
			stop('The column passed as argument "diff" must have precisely 2 levels.')
		}
		if (reverse_diff) {
			data[, names(data) == as.character(diff)] = factor(data[, names(data) ==
																		as.character(diff)], levels = rev(levels(data[, names(data) == as.character(diff)])))
		}
	}
	########
	# computing residuals from covariates
	########
	if ((!is.null(between_covariates)) | (!is.null(within_covariates))) {
		warning(
			"Implementation of ANCOVA in this version of ez is experimental and not yet fully validated. Also, note that ANCOVA is intended purely as a tool to increase statistical power; ANCOVA can not eliminate confounds in the data. Specifically, covariates should: (1) be uncorrelated with other predictors and (2) should have effects on the DV that are independent of other predictors. Failure to meet these conditions may dramatically increase the rate of false-positives.",
			immediate. = TRUE,
			call. = FALSE
		)
	}
	if (!is.null(between_covariates)) {
		temp = idata.frame(cbind(data, ezDV = data[, names(data) == as.character(dv)]))
		temp <- ddply(temp
					  , structure(as.list(
					  	c(wid, between_covariates, within, within_full, diff)
					  ), class = 'quoted')
					  , function(x) {
					  	to_return = mean(x$ezDV)
					  	names(to_return) = as.character(dv)
					  	return(to_return)
					  })
		temp = idata.frame(cbind(temp, ezDV = temp[, names(temp) == as.character(dv)]))
		temp <- ddply(temp
					  , structure(as.list(c(
					  	wid, between_covariates
					  )), class = 'quoted')
					  , function(x) {
					  	to_return = mean(x$ezDV)
					  	names(to_return) = as.character(dv)
					  	return(to_return)
					  })
		for (cov in between_covariates) {
			temp$ezCov = temp[, names(temp) == cov]
			if (is.factor(temp$ezCov)) {
				contrasts(temp$ezCov) = 'contr.helmert'
			} else{
				temp$ezCov = temp$ezCov - mean(temp$ezCov)
				warning(
					'Covariate"',
					cov,
					'" is numeric and will therefore be fit to a linear effect.',
					immediate. = TRUE,
					call. = FALSE
				)
			}
			fit = eval(parse(text = paste(
				'lm(formula=', dv, '~ezCov,data=temp)'
			)))
			temp$fitted = fitted(fit)
			for (cov_lev in as.character(unique(temp[, names(temp) == cov]))) {
				data[as.character(data[, names(data) == cov]) == cov_lev, names(data) ==
					 	dv] = data[as.character(data[, names(data) == cov]) == cov_lev, names(data) ==
					 			   	dv] - temp$fitted[as.character(temp[, names(temp) == cov]) == cov_lev][1] + as.numeric(coef(fit)[1])
			}
		}
	}
	if (!is.null(within_covariates)) {
		temp = idata.frame(cbind(data, ezDV = data[, names(data) == as.character(dv)]))
		temp <- ddply(temp
					  , structure(as.list(
					  	c(wid, within_covariates, within, within_full, diff)
					  ), class = 'quoted')
					  , function(x) {
					  	to_return = mean(x$ezDV)
					  	names(to_return) = as.character(dv)
					  	return(to_return)
					  })
		for (cov in as.character(within_covariates)) {
			temp2 <- ddply(
				.data = temp
				,
				.variables = eval(parse(
					text = paste('.(', wid, ',', cov, ')')
				))#structure(as.list(c(wid,as.symbol(cov)))),class = 'quoted')
				,
				.fun = function(x) {
					to_return = data.frame(value = mean(x[, names(x) == dv]))
					names(to_return) = as.character(dv)
					return(to_return)
				}
			)
			if (is.numeric(temp2[, names(temp2) == cov])) {
				warning(
					'Covariate"',
					cov,
					'" is numeric and will therefore be fit to a linear effect.',
					immediate. = TRUE,
					call. = FALSE
				)
			}
			for (this_wid in unique(as.character(data[, names(data) == wid]))) {
				temp3 = temp2[temp2[, names(temp2) == wid] == this_wid, ]
				temp3$ezCov = temp3[, names(temp3) == cov]
				if (is.factor(temp3$ezCov)) {
					contrasts(temp3$ezCov) = 'contr.helmert'
				} else{
					temp3$ezCov = temp3$ezCov - mean(temp3$ezCov)
				}
				fit = eval(parse(
					text = paste('lm(formula=', dv, '~ezCov,data=temp3)')
				))
				temp3$fitted = fitted(fit)
				for (cov_lev in unique(as.character(temp3[, names(temp3) == cov]))) {
					data[(as.character(data[, names(data) == cov]) == cov_lev) &
						 	(data[, names(data) == wid] == this_wid), names(data) == dv] = data[(as.character(data[, names(data) ==
						 																						   	cov]) == cov_lev) &
						 																			(data[, names(data) == wid] == this_wid), names(data) == dv] - temp3$fitted[as.character(temp3[, names(temp3) ==
						 																																										   	cov]) == cov_lev][1] + as.numeric(coef(fit)[1])
				}
			}
		}
	}
	########
	# Collapsing the data to cell means
	########
	if (!is.null(within_full)) {
		warning(
			paste(
				'Collapsing data to cell means first using variables supplied to "within_full", then collapsing the resulting means to means for the cells supplied to "within".',
				sep = ''
			),
			immediate. = TRUE,
			call. = FALSE
		)
		temp = idata.frame(cbind(data, ezDV = data[, names(data) == as.character(dv)]))
		data <- ddply(temp
					  , structure(as.list(
					  	c(wid, between, within, within_full, diff)
					  ), class = 'quoted')
					  , function(x) {
					  	to_return = mean(x$ezDV)
					  	names(to_return) = as.character(dv)
					  	return(to_return)
					  })
		temp = idata.frame(cbind(data, ezDV = data[, names(data) == as.character(dv)]))
		data <- ddply(temp
					  , structure(as.list(c(
					  	wid, between, within, diff
					  )), class = 'quoted')
					  , function(x) {
					  	to_return = mean(x$ezDV)
					  	names(to_return) = as.character(dv)
					  	return(to_return)
					  })
	} else{
		data <- ddply(data
					  , structure(as.list(c(
					  	wid, between, within, diff
					  )), class = 'quoted')
					  , function(x) {
					  	to_return = data.frame(temp = mean(x[, names(x) == as.character(dv)])
					  						   , ezNum = nrow(x))
					  	names(to_return)[1] = as.character(dv)
					  	return(to_return)
					  })
		if (any(data$ezNum > 1)) {
			warning(
				paste(
					'Collapsing data to cell means. *IF* the requested effects are a subset of the full design, you must use the "within_full" argument, else results may be inaccurate.',
					sep = ''
				),
				immediate. = TRUE,
				call. = FALSE
			)
		}
	}
	if (any(is.na(data[, names(data) == as.character(dv)]))) {
		stop('One or more cells returned NA when aggregated to a mean. Check your data.')
	}
	if (is.null(diff)) {
		if (!all(as.data.frame(table(data[, names(data) %in% c(wid, within)]))$Freq ==
				 1)) {
			stop('One or more cells is missing data. Try using ezDesign() to check your data.')
		}
	} else{
		if (!all(as.data.frame(table(data[, names(data) %in% c(wid, within, diff)]))$Freq ==
				 1)) {
			stop('One or more cells is missing data. Try using ezDesign() to check your data.')
		}
	}
	if (!is.null(between)) {
		if (any(as.data.frame(table(data[, names(data) %in% c(between)]))$Freq ==
				0)) {
			stop('One or more cells is missing data. Try using ezDesign() to check your data.')
		}
	}
	if (!is.null(diff)) {
		warning(
			paste(
				'Collapsing "',
				as.character(diff),
				'" to a difference score ("',
				levels(data[, names(data) == as.character(diff)])[2],
				'"-"',
				levels(data[, names(data) == as.character(diff)])[1],
				'") prior to computing statistics.',
				sep = ''
			),
			immediate. = TRUE,
			call. = FALSE
		)
		temp = idata.frame(cbind(data, ezDV = data[, names(data) == as.character(dv)]))
		data <- ddply(temp
					  , structure(as.list(c(
					  	wid, within, between
					  )), class = 'quoted')
					  , function(x) {
					  	to_return = diff(x$ezDV)
					  	names(to_return) = as.character(dv)
					  	return(to_return)
					  })
		temp = names(within)
		temp = temp[!(within %in% diff)]
		within = within[!(within %in% diff)]
		names(within) = temp
	}
	to_return = list()
	if (is.null(between)) {
		if (any(within_numeric)) {
			warning(
				'There is at least one numeric within variable, therefore aov() will be used for computation and no assumption checks will be obtained.',
				immediate. = TRUE,
				call. = FALSE
			)
			from_aov = ezANOVA_aov(data, dv, wid, within, between)
			to_return$ANOVA = from_aov$ANOVA
			if (return_aov) {
				to_return$aov = from_aov$aov
			}
		} else{
			#there are no numeric within-Ss variables
			if (type != 1) {
				wide_lm = ezANOVA_get_wide_lm(data, dv, wid, within, between)
				to_return = NULL
				try(to_return <- ezANOVA_summary(Anova(
					wide_lm$lm
					,
					idata = wide_lm$idata
					,
					type = 3 #hardcoded to 3, else car::Anova() prints a note
					,
					idesign = eval(parse(text = wide_lm$idesign_formula))
				)))
				if (is.null(to_return)) {
					stop(
						'The car::Anova() function used to compute results and assumption tests seems to have failed. Most commonly this is because you have too few subjects relative to the number of cells in the within-Ss design. It is possible that trying the ANOVA again with "type=1" may yield results (but definitely no assumption tests).'
					)
				}
				if (return_aov) {
					from_aov = ezANOVA_aov(data, dv, wid, within, between)
					to_return$aov = from_aov$aov
				}
			} else{
				from_aov = ezANOVA_aov(data, dv, wid, within, between)
				to_return$ANOVA = from_aov$ANOVA
				if (return_aov) {
					to_return$aov = from_aov$aov
				}
			}
		}
	} else{
		#there are between-Ss variables
		if (balanced) {
			if (is.null(within)) {
				wide_lm = ezANOVA_get_wide_lm(data, dv, wid, within, between)
				from_Anova = Anova(wide_lm$lm
								   ,
								   type = type
								   ,
								   white.adjust = white.adjust)
				to_return$ANOVA = ezANOVA_between_summary(from_Anova, white.adjust, between_numeric)
				if (!any(between_numeric)) {
					to_return$'Levene\'s Test for Homogeneity of Variance' = ezANOVA_levene(wide_lm$lm)
				} else{
					warning(
						'At least one numeric between-Ss variable detected, therefore no assumption test will be returned.',
						immediate. = TRUE,
						call. = FALSE
					)
				}
				if (return_aov) {
					from_aov = ezANOVA_aov(data, dv, wid, within, between)
					to_return$aov = from_aov$aov
				}
			} else{
				#there are within-Ss variables
				if (any(within_numeric)) {
					warning(
						'There is at least one numeric within variable, therefore aov() will be used for computation and no assumption checks will be obtained.',
						immediate. = TRUE,
						call. = FALSE
					)
					from_aov = ezANOVA_aov(data, dv, wid, within, between)
					to_return$ANOVA = from_aov$ANOVA
					if (return_aov) {
						to_return$aov = from_aov$aov
					}
				} else{
					#there are no numeric within-Ss variables
					if (type != 1) {
						wide_lm = ezANOVA_get_wide_lm(data, dv, wid, within, between)
						to_return = NULL
						try(to_return <- ezANOVA_summary(
							Anova(
								wide_lm$lm
								,
								idata = wide_lm$idata
								,
								type = type
								,
								idesign = eval(
									parse(text = wide_lm$idesign_formula)
								)
							)
						))
						if (is.null(to_return)) {
							stop(
								'The car::Anova() function used to compute results and assumption tests seems to have failed. Most commonly this is because you have too few subjects relative to the number of cells in the within-Ss design. It is possible that trying the ANOVA again with "type=1" may yield results (but definitely no assumption tests).'
							)
						}
						if (return_aov) {
							from_aov = ezANOVA_aov(data, dv, wid, within, between)
							to_return$aov = from_aov$aov
						}
					} else{
						#type 1 was requested
						from_aov = ezANOVA_aov(data, dv, wid, within, between)
						to_return$ANOVA = from_aov$ANOVA
						if (return_aov) {
							to_return$aov = from_aov$aov
						}
					}
				}
			}
		} else{
			#data is imbalanced
			if (type == 1) {
				if ((length(between) > 1) | (!is.null(within))) {
					warning(
						'Using "type==1" is highly questionable when data are unbalanced and there is more than one variable. Hopefully you are doing this for demonstration purposes only!',
						immediate. = TRUE,
						call. = FALSE
					)
				}
				from_aov = ezANOVA_aov(data, dv, wid, within, between)
				to_return$ANOVA = from_aov$ANOVA
				if (return_aov) {
					to_return$aov = from_aov$aov
				}
			} else{
				if (is.null(within)) {
					wide_lm = ezANOVA_get_wide_lm(data, dv, wid, within, between)
					from_Anova = Anova(wide_lm$lm
									   ,
									   type = type
									   ,
									   white.adjust = white.adjust)
					to_return$ANOVA = ezANOVA_between_summary(from_Anova,
															  white.adjust,
															  between_numeric)
					if (!any(between_numeric)) {
						to_return$'Levene\'s Test for Homogeneity of Variance' = ezANOVA_levene(wide_lm$lm)
					} else{
						warning(
							'At least one numeric between-Ss variable detected, therefore no assumption test will be returned.',
							immediate. = TRUE,
							call. = FALSE
						)
					}
					if (return_aov) {
						from_aov = ezANOVA_aov(data, dv, wid, within, between)
						to_return$aov = from_aov$aov
					}
				} else{
					if (any(within_numeric)) {
						stop(
							'Cannot perform ANOVA when data are imbalanced and when one or more within-Ss variables are numeric. Try ezMixed() instead.'
						)
					}
					wide_lm = ezANOVA_get_wide_lm(data, dv, wid, within, between)
					to_return = NULL
					try(to_return <- ezANOVA_summary(
						Anova(
							wide_lm$lm
							,
							idata = wide_lm$idata
							,
							type = type
							,
							idesign = eval(parse(
								text = wide_lm$idesign_formula
							))
						)
					))
					if (is.null(to_return)) {
						stop(
							'The car::Anova() function used to compute results and assumption tests seems to have failed. Most commonly this is because you have too few subjects relative to the number of cells in the within-Ss design. It is possible that trying the ANOVA again with "type=1" may yield results (but definitely no assumption tests).'
						)
					}
					if (return_aov) {
						from_aov = ezANOVA_aov(data, dv, wid, within, between)
						to_return$aov = from_aov$aov
					}
				}
			}
		}
	}
	to_return$ANOVA = to_return$ANOVA[order(stringr::str_count(to_return$ANOVA$Effect, ':')), ]
	to_return$data = data
	return(to_return)
}

ezANOVA_get_wide_lm <- function(data, dv, wid, within, between) {
	to_return = list()
	if(!is.null(within)) {
		for(this_within in within){
			old_levs = levels(data[,names(data)==this_within])
			new_levs = rep(NA,length=length(old_levs))
			temp = strsplit(old_levs,'_')
			for(i in 1:length(old_levs)){
				new_levs[i] = paste(temp[[i]],collapse='.')
			}
			levels(data[,names(data)==this_within]) = new_levs
		}
		wide_formula = paste(paste(wid,paste(between,collapse='+'),sep='+'),paste(within,collapse='+'),sep='~')
		wide_formula = sub('+~','~',wide_formula,fixed=T)
		wide = dcast(data, wide_formula, value.var = as.character(dv))
		to_return$idata=ldply(strsplit(names(wide)[!(names(wide) %in% c(between,wid))],'_'))
		names(to_return$idata)=within
		for(this_within in within){
			to_return$idata[,names(to_return$idata)==this_within] = factor(to_return$idata[,names(to_return$idata)==this_within])
		}
		wide_dv=data.matrix(wide[,!(names(wide) %in% c(wid,between))])
		to_return$idesign_formula = paste('~',paste(within,collapse='*'),sep='')
	} else {
		wide=data
	}
	if(is.null(between)) {
		lm_formula=paste('wide_dv~1',sep='')
	} else if(is.null(within)) {
		lm_formula=paste(dv,'~',paste(between,collapse='*'),sep='')
	} else {
		lm_formula=paste('wide_dv~',paste(between,collapse='*'),sep='')
	}
	op <- options( "contrasts" = c( "contr.sum", "contr.poly" ) )
	to_return$lm = lm(eval(parse(text=lm_formula)),wide)
	options(op)
	return(to_return)
}

ezANOVA_aov <- function(data, dv, wid, within, between){
	aov_formula = paste(
		as.character(dv)
		,'~'
		,paste(as.character(between),collapse = '*')
		,ifelse(is.null(between),'',ifelse(is.null(within),'','*'))
		,paste(as.character(within),collapse = '*')
		,ifelse(
			is.null(within)
			,''
			,paste(
				'+Error('
				,as.character(wid)
				,'/('
				,paste(as.character(within),collapse = '*')
				,'))'
				,sep = ''
			)
		)
		,sep = ''
	)
	this_aov = aov(
		formula(aov_formula)
		,data = data
	)
	ANOVA = NULL
	for(x in summary(this_aov)){
		if(length(x)==1){
			x=x[[1]]
		}
		for(row in 1:length(x[,1])){
			if(!is.na(x$P[row])){
				ANOVA = rbind(
					ANOVA
					, data.frame(
						Effect=strsplit(row.names(x)[row],' ')[[1]][1]
						, DFn=x$D[row]
						, DFd=x$D[length(x$D)]
						, SSn=x$S[row]
						, SSd=x$S[length(x$S)]
						, F=x$F[row]
						, p=x$P[row]
					)
				)
			}
		}
	}
	ANOVA$'p<.05'=ifelse(ANOVA$p<.05,'*','')
	to_return = list(
		ANOVA = ANOVA
		, aov = this_aov
	)
	return(to_return)
}

####################################################################################################
# Copied from https://github.com/mike-lawrence/ez/blob/master/R/ez-internal.R
ezANOVA_levene <-
	function (y) {
		form <- y
		mf <- model.frame(form)
		if (any(sapply(2:dim(mf)[2], function(j) is.numeric(mf[[j]])))) stop("Levene's test is not appropriate with quantitative explanatory variables.")
		y <- mf[,1]
		if(dim(mf)[2]==2) {
			group <- mf[,2]
		}else {
			if (length(grep("\\+ | \\| | \\^ | \\:",form))>0) stop("Model must be completely crossed formula only.")
			group <- interaction(mf[,2:dim(mf)[2]])
		}
		if (!is.numeric(y))
			stop(deparse(substitute(y)), " is not a numeric variable")
		if (!is.factor(group)) {
			warning(deparse(substitute(group)), " coerced to factor.")
			group <- as.factor(group)
		}
		meds <- tapply(y, group, median, na.rm = TRUE)
		resp <- abs(y - meds[group])
		table <- as.data.frame(anova(lm(resp ~ group))[, c(1,2, 4, 5)])
		to_return = data.frame(table$D[1],table$D[2],table$S[1],table$S[2],table$F[1],table$P[1])
		names(to_return)=c("DFn", "DFd", "SSn", "SSd", "F", "p")
		to_return$"p<.05"=ifelse(to_return$p<.05,'*','')
		return(to_return)
	}

ezANOVA_summary <-
	function(object){
		to_return=list()
		GG <- function(SSPE, P){
			p <- nrow(SSPE)
			if (p < 2) return(NA)
			lambda <- eigen(SSPE %*% solve(t(P) %*% P))$values
			lambda <- lambda[lambda > 0]
			((sum(lambda)/p)^2)/(sum(lambda^2)/p)
		}
		HF <- function(gg, error.df, p){
			((error.df + 1)*p*gg - 2)/(p*(error.df - p*gg))
		}
		mauchly <- function (SSD, P, df) {
			# most of this function borrowed from stats:::mauchly.test.SSD
			if (nrow(SSD) < 2) return(c(NA, NA))
			Tr <- function (X) sum(diag(X))
			p <- nrow(P)
			I <- diag(p)
			Psi <- t(P) %*% I %*% P
			B <- SSD
			pp <- nrow(SSD)
			U <- solve(Psi, B)
			n <- df
			logW <- log(det(U)) - pp * log(Tr(U/pp))
			rho <- 1 - (2 * pp^2 + pp + 2)/(6 * pp * n)
			w2 <- (pp + 2) * (pp - 1) * (pp - 2) * (2 * pp^3 + 6 * pp^2 +
														3 * p + 2)/(288 * (n * pp * rho)^2)
			z <- -n * rho * logW
			f <- pp * (pp + 1)/2 - 1
			Pr1 <- pchisq(z, f, lower.tail = FALSE)
			Pr2 <- pchisq(z, f + 4, lower.tail = FALSE)
			pval <- Pr1 + w2 * (Pr2 - Pr1)
			c(statistic = c(W = exp(logW)), p.value = pval)
		}
		test.statistic <- 1:4
		nterms <- length(object$terms)
		error.df <- object$error.df
		table <- data.frame(matrix(0, nterms, 8))
		table2 <- data.frame(matrix(0, nterms, 7))
		table3 <- data.frame(matrix(0, nterms, 4))
		table3[,1] <- table2[,1] <- table[,1] <- object$terms
		colnames(table) <- c("Effect","DFn", "DFd", "SSn", "SSd", "F", "p", "p<.05")
		colnames(table2) <- c("Effect","GGe", "p[GG]", "p[GG]<.05", "HFe", "p[HF]","p[HF]<.05")
		colnames(table3) <- c("Effect","W", "p", "p<.05")
		for (term in 1:nterms){
			SSP <- object$SSP[[term]]
			SSPE <- object$SSPE[[term]]
			P <- object$P[[term]]
			p <- ncol(P)
			PtPinv <- solve(t(P) %*% P)
			gg <- GG(SSPE, P)
			table[term, "SSn"] <- sum(diag(SSP %*% PtPinv))
			table[term, "SSd"] <- sum(diag(SSPE %*% PtPinv))
			table[term, "DFn"] <- object$df[term] * p
			table[term, "DFd"] <- error.df * p
			table[term, "F"] <-  (table[term, "SSn"]/table[term, "DFn"])/
				(table[term, "SSd"]/table[term, "DFd"])
			table[term, "p"] <- pf(table[term, "F"], table[term, "DFn"],
								   table[term, "DFd"], lower.tail=FALSE)
			table[term, "p<.05"] = ifelse(table[term, "p"]<.05,'*','')
			table2[term, "GGe"] <- gg
			table2[term, "HFe"] <- HF(gg, error.df, p)
			table3[term,2:3] <- mauchly(SSPE, P, object$error.df)
			table3[term, "p<.05"] = ifelse(table3[term, "p"]<.05,'*','')
		}
		ANOVA = as.data.frame(table)
		to_return$ANOVA=ANOVA
		table3=table3[!is.na(table3$W),]
		if (nrow(table3) > 0){
			to_return$'Mauchly\'s Test for Sphericity'=table3
			table2[,"p[GG]"] <- pf(table[,"F"], table2[,"GGe"]*table[,"DFn"],table2[,"GGe"]*table[,"DFd"], lower.tail=FALSE)
			table2[, "p[GG]<.05"] = ifelse(table2[, "p[GG]"]<.05,'*','')
			table2[,"p[HF]"] <- pf(table[,"F"], pmin(1, table2[,"HFe"])*table[,"DFn"],	pmin(1, table2[,"HFe"])*table[,"DFd"], lower.tail=FALSE)
			table2[, "p[HF]<.05"] = ifelse(table2[, "p[HF]"]<.05,'*','')
			table2=table2[!is.na(table2$GG),]
			to_return$'Sphericity Corrections'=table2
		}
		return(to_return)
	}

ezANOVA_between_summary <-
	function(from_Anova,white.adjust,between_numeric){
		temp = as.data.frame(from_Anova)
		if(!white.adjust){
			names(temp) = c('SSn','DFn','F','p')
			temp$DFd = temp$D[length(temp$D)]
			temp$SSd = temp$S[length(temp$S)]
			temp$Effect = row.names(temp)
			row.names(temp) = 1:length(temp[,1])
			temp = temp[1:(length(temp[,1])-1),c(7,2,5,1,6,3,4)]
		}else{
			names(temp) = c('DFn','F','p')
			temp$DFd = temp$D[length(temp$D)]
			temp$SSd = temp$S[length(temp$S)]
			temp$Effect = row.names(temp)
			row.names(temp) = 1:length(temp[,1])
			temp = temp[1:(length(temp[,1])-1),c(5,1,4,2,3)]
		}
		temp$'p<.05'=ifelse(temp$p<.05,'*','')
		return(temp)
	}


ezANOVA_get_wide_lm<-
	function(data, dv, wid, within, between){
		to_return = list()
		if(!is.null(within)){
			for(this_within in within){
				old_levs = levels(data[,names(data)==this_within])
				new_levs = rep(NA,length=length(old_levs))
				temp = strsplit(old_levs,'_')
				for(i in 1:length(old_levs)){
					new_levs[i] = paste(temp[[i]],collapse='.')
				}
				levels(data[,names(data)==this_within]) = new_levs
			}
			wide_formula = paste(paste(wid,paste(between,collapse='+'),sep='+'),paste(within,collapse='+'),sep='~')
			wide_formula = sub('+~','~',wide_formula,fixed=T)
			wide = dcast(data, wide_formula, value.var = as.character(dv))
			to_return$idata=ldply(strsplit(names(wide)[!(names(wide) %in% c(between,wid))],'_'))
			names(to_return$idata)=within
			for(this_within in within){
				to_return$idata[,names(to_return$idata)==this_within] = factor(to_return$idata[,names(to_return$idata)==this_within])
			}
			wide_dv=data.matrix(wide[,!(names(wide) %in% c(wid,between))])
			to_return$idesign_formula = paste('~',paste(within,collapse='*'),sep='')
		}else{
			wide=data
		}
		if(is.null(between)){
			lm_formula=paste('wide_dv~1',sep='')
		}else if(is.null(within)){
			lm_formula=paste(dv,'~',paste(between,collapse='*'),sep='')
		}else{
			lm_formula=paste('wide_dv~',paste(between,collapse='*'),sep='')
		}
		op <- options( "contrasts" = c( "contr.sum", "contr.poly" ) )
		to_return$lm = lm(eval(parse(text=lm_formula)),wide)
		options(op)
		return(to_return)
	}

ezANOVA_main <-
	function(data, dv, wid, within, within_full, within_covariates, between, between_covariates, observed, diff, reverse_diff, type, return_aov, white.adjust){
		if(inherits(data, "tbl_df")) {
			data <- as.data.frame(data)
		}
		vars = as.character(c(dv,wid,between,within,diff,within_full))
		for(var in vars){
			if(!(var %in% names(data))){
				stop(paste('"',var,'" is not a variable in the data frame provided.',sep=''))
			}
		}
		if(is.null(within) & is.null(between)){
			stop('is.null(within) & is.null(between)\nYou must specify at least one independent variable.')
		}
		if(!is.data.frame(data)){
			stop('"data" must be a data frame.')
		}
		if(!is.numeric(data[,names(data)==dv])){
			stop('"dv" must be numeric.')
		}
		if(!is.factor(data[,names(data)==wid])){
			warning(paste('Converting "',wid,'" to factor for ANOVA.',sep=''),immediate.=TRUE,call.=FALSE)
			data[,names(data)==wid]=factor(data[,names(data)==wid])
		}else{
			if(length(unique(data[,names(data)==wid]))!=length(levels(data[,names(data)==wid]))){
				warning(paste('You have removed one or more Ss from the analysis. Refactoring "',wid,'" for ANOVA.',sep=''),immediate.=TRUE,call.=FALSE)
				data[,names(data)==wid]=factor(data[,names(data)==wid])
			}
		}
		within_numeric = rep(FALSE,length(within))
		if(!is.null(within)){
			for(i in 1:length(within)){
				if(is.numeric(data[,names(data)==within[i]])){
					warning(paste('"',within[i],'" will be treated as numeric.',sep=''),immediate.=TRUE,call.=FALSE)
					within_numeric[i] = TRUE
				}else{
					if(!is.factor(data[,names(data)==within[i]])){
						warning(paste('Converting "',within[i],'" to factor for ANOVA.',sep=''),immediate.=TRUE,call.=FALSE)
						data[,names(data)==within[i]]=factor(data[,names(data)==within[i]])
					}
					if(length(unique(data[,names(data)==within[i]]))!=length(levels(data[,names(data)==within[i]]))){
						warning(paste('You have removed one or more levels from variable "',within[i],'". Refactoring for ANOVA.',sep=''),immediate.=TRUE,call.=FALSE)
						data[,names(data)==within[i]]=factor(data[,names(data)==within[i]])
					}
					if(length(levels(data[,names(data)==within[i]]))==1){
						stop(paste('"',within[i],'" has only one level."',sep=''))
					}
				}
			}
		}
		between_numeric = rep(FALSE,length(between))
		if(!is.null(between)){
			for(i in 1:length(between)){
				if(is.numeric(data[,names(data)==between[i]])){
					warning(paste('"',between[i],'" will be treated as numeric.',sep=''),immediate.=TRUE,call.=FALSE)
					between_numeric[i] = TRUE
				}else{
					if(!is.factor(data[,names(data)==between[i]])){
						warning(paste('Converting "',between[i],'" to factor for ANOVA.',sep=''),immediate.=TRUE,call.=FALSE)
						data[,names(data)==between[i]]=factor(data[,names(data)==between[i]])
					}
					if(length(unique(data[,names(data)==between[i]]))!=length(levels(data[,names(data)==between[i]]))){
						warning(paste('You have removed one or more levels from variable "',between[i],'". Refactoring for ANOVA.',sep=''),immediate.=TRUE,call.=FALSE)
						data[,names(data)==between[i]]=factor(data[,names(data)==between[i]])
					}
					if(length(levels(data[,names(data)==between[i]]))==1){
						stop(paste('"',between[i],'" has only one level."',sep=''))
					}
				}
			}
			temp = ddply(
				idata.frame(data)
				,structure(as.list(c(wid,between)),class = 'quoted')
				,function(x){
					to_return = 0
					return(to_return)
				}
			)
			wid_temp = data.frame(table(temp[,names(temp)==wid]))
			if(any(wid_temp$Freq>1)){
				warning(paste('The column supplied as the wid variable contains non-unique values across levels of the supplied between-Ss variables. Automatically fixing this by generating unique wid labels.',sep=''),immediate.=TRUE,call.=FALSE)
				data[,names(data)==wid] = as.character(data[,names(data)==wid])
				for(i in unique(as.character(between))){
					data[,names(data)==wid] = paste(data[,names(data)==wid],data[,names(data)==i])
				}
				data[,names(data)==wid] = factor(data[,names(data)==wid])
				temp = ddply(
					idata.frame(data)
					,structure(as.list(c(wid,between)),class = 'quoted')
					,function(x){
						to_return = 0
						return(to_return)
					}
				)
			}
			temp = ddply(
				temp
				,structure(as.list(c(between)),class = 'quoted')
				,function(x){
					to_return = data.frame(
						N = nrow(x)
					)
					return(to_return)
				}
			)
			balanced = ifelse(
				length(unique(temp$N))>1
				, FALSE
				, TRUE
			)
			if(!balanced){
				warning('Data is unbalanced (unequal N per group). Make sure you specified a well-considered value for the type argument to ezANOVA().',immediate.=TRUE,call.=FALSE)
			}
		}
		if(!is.null(diff)){
			if(length(diff)>1){
				stop(paste('Provide only one value to "diff".',sep=''))
			}
			if(!is.factor(data[,names(data)==diff])){
				warning(paste('Converting "',diff,'" to factor for computing difference score.',sep=''),immediate.=TRUE,call.=FALSE)
				data[,names(data)==diff]=factor(data[,names(data)==diff])
			}
			temp <- ddply(
				idata.frame(data)
				,structure(as.list(c(wid,diff)),class = 'quoted')
				,function(x){
					to_return = 0
					return(to_return)
				}
			)
			if(!all(as.data.frame(table(temp[,names(temp) %in% c(wid,within)]))$Freq==2)){
				stop(paste('Variable supplied to "diff" ("',as.character(diff),'") does not appear to be a within variable.',sep=''))
			}
			data[,names(data)==as.character(diff)] = factor(data[,names(data)==as.character(diff)])
			if(length(unique(data[,names(data)==as.character(diff)]))!=2){
				stop('The column passed as argument "diff" must have precisely 2 levels.')
			}
			if(reverse_diff){
				data[,names(data)==as.character(diff)] = factor(data[,names(data)==as.character(diff)],levels=rev(levels(data[,names(data)==as.character(diff)])))
			}
		}
		########
		# computing residuals from covariates
		########
		if((!is.null(between_covariates))|(!is.null(within_covariates))){
			warning("Implementation of ANCOVA in this version of ez is experimental and not yet fully validated. Also, note that ANCOVA is intended purely as a tool to increase statistical power; ANCOVA can not eliminate confounds in the data. Specifically, covariates should: (1) be uncorrelated with other predictors and (2) should have effects on the DV that are independent of other predictors. Failure to meet these conditions may dramatically increase the rate of false-positives.",immediate.=TRUE,call.=FALSE)
		}
		if(!is.null(between_covariates)){
			temp = idata.frame(cbind(data,ezDV=data[,names(data) == as.character(dv)]))
			temp <- ddply(
				temp
				,structure(as.list(c(wid,between_covariates,within,within_full,diff)),class = 'quoted')
				,function(x){
					to_return = mean(x$ezDV)
					names(to_return) = as.character(dv)
					return(to_return)
				}
			)
			temp = idata.frame(cbind(temp,ezDV=temp[,names(temp) == as.character(dv)]))
			temp <- ddply(
				temp
				,structure(as.list(c(wid,between_covariates)),class = 'quoted')
				,function(x){
					to_return = mean(x$ezDV)
					names(to_return) = as.character(dv)
					return(to_return)
				}
			)
			for(cov in between_covariates){
				temp$ezCov = temp[,names(temp)==cov]
				if(is.factor(temp$ezCov)){
					contrasts(temp$ezCov) = 'contr.helmert'
				}else{
					temp$ezCov = temp$ezCov - mean(temp$ezCov)
					warning('Covariate"',cov,'" is numeric and will therefore be fit to a linear effect.',immediate.=TRUE,call.=FALSE)
				}
				fit = eval(parse(text=paste('lm(formula=',dv,'~ezCov,data=temp)')))
				temp$fitted = fitted(fit)
				for(cov_lev in as.character(unique(temp[,names(temp)==cov]))){
					data[as.character(data[,names(data)==cov])==cov_lev,names(data)==dv] = data[as.character(data[,names(data)==cov])==cov_lev,names(data)==dv] - temp$fitted[as.character(temp[,names(temp)==cov])==cov_lev][1] + as.numeric(coef(fit)[1])
				}
			}
		}
		if(!is.null(within_covariates)){
			temp = idata.frame(cbind(data,ezDV=data[,names(data) == as.character(dv)]))
			temp <- ddply(
				temp
				,structure(as.list(c(wid,within_covariates,within,within_full,diff)),class = 'quoted')
				,function(x){
					to_return = mean(x$ezDV)
					names(to_return) = as.character(dv)
					return(to_return)
				}
			)
			for(cov in as.character(within_covariates)){
				temp2 <- ddply(
					.data = temp
					, .variables = eval(parse(text=paste('.(',wid,',',cov,')')))#structure(as.list(c(wid,as.symbol(cov)))),class = 'quoted')
					, .fun = function(x){
						to_return = data.frame(
							value = mean(x[,names(x)==dv])
						)
						names(to_return) = as.character(dv)
						return(to_return)
					}
				)
				if(is.numeric(temp2[,names(temp2)==cov])){
					warning('Covariate"',cov,'" is numeric and will therefore be fit to a linear effect.',immediate.=TRUE,call.=FALSE)
				}
				for(this_wid in unique(as.character(data[,names(data)==wid]))){
					temp3 = temp2[temp2[,names(temp2)==wid]==this_wid,]
					temp3$ezCov = temp3[,names(temp3)==cov]
					if(is.factor(temp3$ezCov)){
						contrasts(temp3$ezCov) = 'contr.helmert'
					}else{
						temp3$ezCov = temp3$ezCov - mean(temp3$ezCov)
					}
					fit = eval(parse(text=paste('lm(formula=',dv,'~ezCov,data=temp3)')))
					temp3$fitted = fitted(fit)
					for(cov_lev in unique(as.character(temp3[,names(temp3)==cov]))){
						data[(as.character(data[,names(data)==cov])==cov_lev)&(data[,names(data)==wid]==this_wid),names(data)==dv] = data[(as.character(data[,names(data)==cov])==cov_lev)&(data[,names(data)==wid]==this_wid),names(data)==dv] - temp3$fitted[as.character(temp3[,names(temp3)==cov])==cov_lev][1] + as.numeric(coef(fit)[1])
					}
				}
			}
		}
		########
		# Collapsing the data to cell means
		########
		if(!is.null(within_full)){
			warning(paste('Collapsing data to cell means first using variables supplied to "within_full", then collapsing the resulting means to means for the cells supplied to "within".',sep=''),immediate.=TRUE,call.=FALSE)
			temp = idata.frame(cbind(data,ezDV=data[,names(data) == as.character(dv)]))
			data <- ddply(
				temp
				,structure(as.list(c(wid,between,within,within_full,diff)),class = 'quoted')
				,function(x){
					to_return = mean(x$ezDV)
					names(to_return) = as.character(dv)
					return(to_return)
				}
			)
			temp = idata.frame(cbind(data,ezDV=data[,names(data) == as.character(dv)]))
			data <- ddply(
				temp
				,structure(as.list(c(wid,between,within,diff)),class = 'quoted')
				,function(x){
					to_return = mean(x$ezDV)
					names(to_return) = as.character(dv)
					return(to_return)
				}
			)
		}else{
			data <- ddply(
				data
				,structure(as.list(c(wid,between,within,diff)),class = 'quoted')
				,function(x){
					to_return = data.frame(
						temp = mean(x[,names(x)==as.character(dv)])
						, ezNum = nrow(x)
					)
					names(to_return)[1] = as.character(dv)
					return(to_return)
				}
			)
			if(any(data$ezNum>1)){
				warning(paste('Collapsing data to cell means. *IF* the requested effects are a subset of the full design, you must use the "within_full" argument, else results may be inaccurate.',sep=''),immediate.=TRUE,call.=FALSE)
			}
		}
		if(any(is.na(data[,names(data)==as.character(dv)]))){
			stop('One or more cells returned NA when aggregated to a mean. Check your data.')
		}
		if(is.null(diff)){
			if(!all(as.data.frame(table(data[,names(data) %in% c(wid,within)]))$Freq==1)){
				stop('One or more cells is missing data. Try using ezDesign() to check your data.')
			}
		}else{
			if(!all(as.data.frame(table(data[,names(data) %in% c(wid,within,diff)]))$Freq==1)){
				stop('One or more cells is missing data. Try using ezDesign() to check your data.')
			}
		}
		if(!is.null(between)){
			if(any(as.data.frame(table(data[,names(data) %in% c(between)]))$Freq==0)){
				stop('One or more cells is missing data. Try using ezDesign() to check your data.')
			}
		}
		if(!is.null(diff)){
			warning(paste('Collapsing "',as.character(diff),'" to a difference score ("',levels(data[,names(data)==as.character(diff)])[2],'"-"',levels(data[,names(data)==as.character(diff)])[1],'") prior to computing statistics.',sep=''),immediate.=TRUE,call.=FALSE)
			temp = idata.frame(cbind(data,ezDV=data[,names(data) == as.character(dv)]))
			data <- ddply(
				temp
				,structure(as.list(c(wid,within,between)),class = 'quoted')
				,function(x){
					to_return = diff(x$ezDV)
					names(to_return) = as.character(dv)
					return(to_return)
				}
			)
			temp = names(within)
			temp = temp[!(within %in% diff)]
			within = within[!(within %in% diff)]
			names(within) = temp
		}
		to_return = list()
		if(is.null(between)){
			if(any(within_numeric)){
				warning('There is at least one numeric within variable, therefore aov() will be used for computation and no assumption checks will be obtained.',immediate.=TRUE,call.=FALSE)
				from_aov = ezANOVA_aov(data, dv, wid, within, between)
				to_return$ANOVA = from_aov$ANOVA
				if(return_aov){
					to_return$aov = from_aov$aov
				}
			}else{ #there are no numeric within-Ss variables
				if(type!=1){
					wide_lm = ezANOVA_get_wide_lm(data, dv, wid, within, between)
					to_return = NULL
					try(
						to_return <- ezANOVA_summary(
							Anova(
								wide_lm$lm
								, idata = wide_lm$idata
								, type = 3 #hardcoded to 3, else car::Anova() prints a note
								, idesign = eval(parse(text=wide_lm$idesign_formula))
							)
						)
					)
					if(is.null(to_return)){
						stop('The car::Anova() function used to compute results and assumption tests seems to have failed. Most commonly this is because you have too few subjects relative to the number of cells in the within-Ss design. It is possible that trying the ANOVA again with "type=1" may yield results (but definitely no assumption tests).')
					}
					if(return_aov){
						from_aov = ezANOVA_aov(data, dv, wid, within, between)
						to_return$aov = from_aov$aov
					}
				}else{
					from_aov = ezANOVA_aov(data, dv, wid, within, between)
					to_return$ANOVA = from_aov$ANOVA
					if(return_aov){
						to_return$aov = from_aov$aov
					}
				}
			}
		}else{ #there are between-Ss variables
			if(balanced){
				if(is.null(within)){
					wide_lm = ezANOVA_get_wide_lm(data, dv, wid, within, between)
					from_Anova = Anova(
						wide_lm$lm
						, type = type
						, white.adjust = white.adjust
					)
					to_return$ANOVA = ezANOVA_between_summary(from_Anova,white.adjust,between_numeric)
					if(!any(between_numeric)){
						to_return$'Levene\'s Test for Homogeneity of Variance' = ezANOVA_levene(wide_lm$lm)
					}else{
						warning('At least one numeric between-Ss variable detected, therefore no assumption test will be returned.',immediate.=TRUE,call.=FALSE)
					}
					if(return_aov){
						from_aov = ezANOVA_aov(data, dv, wid, within, between)
						to_return$aov = from_aov$aov
					}
				}else{ #there are within-Ss variables
					if(any(within_numeric)){
						warning('There is at least one numeric within variable, therefore aov() will be used for computation and no assumption checks will be obtained.',immediate.=TRUE,call.=FALSE)
						from_aov = ezANOVA_aov(data, dv, wid, within, between)
						to_return$ANOVA = from_aov$ANOVA
						if(return_aov){
							to_return$aov = from_aov$aov
						}
					}else{ #there are no numeric within-Ss variables
						if(type!=1){
							wide_lm = ezANOVA_get_wide_lm(data, dv, wid, within, between)
							to_return = NULL
							try(
								to_return <- ezANOVA_summary(
									Anova(
										wide_lm$lm
										, idata = wide_lm$idata
										, type = type
										, idesign = eval(parse(text=wide_lm$idesign_formula))
									)
								)
							)
							if(is.null(to_return)){
								stop('The car::Anova() function used to compute results and assumption tests seems to have failed. Most commonly this is because you have too few subjects relative to the number of cells in the within-Ss design. It is possible that trying the ANOVA again with "type=1" may yield results (but definitely no assumption tests).')
							}
							if(return_aov){
								from_aov = ezANOVA_aov(data, dv, wid, within, between)
								to_return$aov = from_aov$aov
							}
						}else{ #type 1 was requested
							from_aov = ezANOVA_aov(data, dv, wid, within, between)
							to_return$ANOVA = from_aov$ANOVA
							if(return_aov){
								to_return$aov = from_aov$aov
							}
						}
					}
				}
			}else{ #data is imbalanced
				if(type==1){
					if((length(between)>1)|(!is.null(within))){
						warning('Using "type==1" is highly questionable when data are unbalanced and there is more than one variable. Hopefully you are doing this for demonstration purposes only!',immediate.=TRUE,call.=FALSE)
					}
					from_aov = ezANOVA_aov(data, dv, wid, within, between)
					to_return$ANOVA = from_aov$ANOVA
					if(return_aov){
						to_return$aov = from_aov$aov
					}
				}else{
					if(is.null(within)){
						wide_lm = ezANOVA_get_wide_lm(data, dv, wid, within, between)
						from_Anova = Anova(
							wide_lm$lm
							, type = type
							, white.adjust = white.adjust
						)
						to_return$ANOVA = ezANOVA_between_summary(from_Anova,white.adjust,between_numeric)
						if(!any(between_numeric)){
							to_return$'Levene\'s Test for Homogeneity of Variance' = ezANOVA_levene(wide_lm$lm)
						}else{
							warning('At least one numeric between-Ss variable detected, therefore no assumption test will be returned.',immediate.=TRUE,call.=FALSE)
						}
						if(return_aov){
							from_aov = ezANOVA_aov(data, dv, wid, within, between)
							to_return$aov = from_aov$aov
						}
					}else{
						if(any(within_numeric)){
							stop('Cannot perform ANOVA when data are imbalanced and when one or more within-Ss variables are numeric. Try ezMixed() instead.')
						}
						wide_lm = ezANOVA_get_wide_lm(data, dv, wid, within, between)
						to_return = NULL
						try(
							to_return <- ezANOVA_summary(
								Anova(
									wide_lm$lm
									, idata = wide_lm$idata
									, type = type
									, idesign = eval(parse(text=wide_lm$idesign_formula))
								)
							)
						)
						if(is.null(to_return)){
							stop('The car::Anova() function used to compute results and assumption tests seems to have failed. Most commonly this is because you have too few subjects relative to the number of cells in the within-Ss design. It is possible that trying the ANOVA again with "type=1" may yield results (but definitely no assumption tests).')
						}
						if(return_aov){
							from_aov = ezANOVA_aov(data, dv, wid, within, between)
							to_return$aov = from_aov$aov
						}
					}
				}
			}
		}
		to_return$ANOVA = to_return$ANOVA[order(str_count(to_return$ANOVA$Effect,':')),]
		to_return$data = data
		return(to_return)
	}

ezANOVA_aov <-
	function(data, dv, wid, within, between){
		aov_formula = paste(
			as.character(dv)
			,'~'
			,paste(as.character(between),collapse = '*')
			,ifelse(is.null(between),'',ifelse(is.null(within),'','*'))
			,paste(as.character(within),collapse = '*')
			,ifelse(
				is.null(within)
				,''
				,paste(
					'+Error('
					,as.character(wid)
					,'/('
					,paste(as.character(within),collapse = '*')
					,'))'
					,sep = ''
				)
			)
			,sep = ''
		)
		this_aov = aov(
			formula(aov_formula)
			,data = data
		)
		ANOVA = NULL
		for(x in summary(this_aov)){
			if(length(x)==1){
				x=x[[1]]
			}
			for(row in 1:length(x[,1])){
				if(!is.na(x$P[row])){
					ANOVA = rbind(
						ANOVA
						, data.frame(
							Effect=strsplit(row.names(x)[row],' ')[[1]][1]
							, DFn=x$D[row]
							, DFd=x$D[length(x$D)]
							, SSn=x$S[row]
							, SSd=x$S[length(x$S)]
							, F=x$F[row]
							, p=x$P[row]
						)
					)
				}
			}
		}
		ANOVA$'p<.05'=ifelse(ANOVA$p<.05,'*','')
		to_return = list(
			ANOVA = ANOVA
			, aov = this_aov
		)
		return(to_return)
	}

ezPerm_aov <-
	function(data, aov_formula){
		this_aov = aov(
			formula(aov_formula)
			,data = data
		)
		f_list=llply(this_aov,function(x){summary(x)[[1]]$F})
		f = NULL
		for(i in f_list){
			f=c(f,i)
		}
		f=f[!is.na(f)]
		return(f)
	}

########
# Framework for ezANOVA branching:
########

#if no between
#	if any within numeric
#		use aov
#	else if there are within numeric
#		if type != 1
#			use Anova
#		else if type==1
#			use aov
#else if there are between
#	if between is balanced
#		if there are no within
#			use Anova
#			if there are no numeric between
#				report levene
#		else if there are within
#			if any within numeric
#				use aov
#			else if there are within numeric
#				if type != 1
#					use Anova
#				else if type==1
#					use aov
#	else if between is unbalanced
#		if type==1
#			if more than one variable
#				print big warning
#			use aov with big warning
#		else if type!= 1
#			if there are no within
#				use Anova
#				if there are no numeric between
#					report levene
#			else if there are within
#				if there are any within numeric
#					stop with error
#				else if there are no within numeric
#					use Anova

utils::globalVariables(c('Treatment','Covariate','Mean','Strata','ymin','ymax',
						 '..count..','ID','describeBy','cstrata.psa','cv.trans.psa'))

#' Balance plot for the given covariate.
#'
#' If the covariate is numeric, boxplots will be drawn with red points for the mean
#' and green error bars for the standard error. For non-numeric covariates a barplot
#' will be drawn.
#' 
#' A Friedman rank sum test will be performed for all covariate types, printed,
#' and stored as an attribute to the returned object named \code{friedman}. If
#' a continuous covariate a repeated measures ANOVA will also be performed, printed,
#' and returned as an attribute named \code{rmanova}.
#' 
#' @param x results from [trimatch()].
#' @param covar vector of the covariate to check balance of.
#' @param model an integer between 1 and 3 indicating from which model the 
#'        propensity scores will be used.
#' @param nstrata number of strata to use.
#' @param ylab label of the y-axis.
#' @param xlab label of the x-axis.
#' @param se.ratio a multiplier for how large standard error bars will be.
#' @param label label for the legend.
#' @param print print the output if the Freidman Rank Sum Test and repeated
#'        measures ANOVA (for continuous variables).
#' @param legend.position the position of the legend. 
#' @param x.axis.labels labels for the x-axis.
#' @param x.axis.angle angle for x-axis labels.
#' @param ... parameters passed to [plot.balance.plots()].
#' @return a \code{ggplot2} figure or a list of \code{ggplot2} figures if \code{covar}
#'        is a data frame.
#' @export
balance.plot <- function(x, covar, model,
					     nstrata=attr(attr(tmatch, 'triangle.psa'), 'nstrata'),
					     label='Covariate',
					     ylab='',
						 xlab=NULL,
						 se.ratio = 2,
						 print=TRUE,
						 legend.position='top',
						 x.axis.labels,
						 x.axis.angle = -45,
						 ...) {
	tmatch <- x
	if(is.data.frame(covar)) {
		#If covar is a data frame, create a grid of figures.
		plots <- list()
		for(i in seq_along(covar)) {
			psub <- balance.plot(tmatch,
								 covar[,i], 
								 model = model,
								 nstrata = nstrata,
								 label = names(covar)[i],
								 ylab = names(covar)[i],
								 se.ratio = se.ratio,
								 legend.position = legend.position,
								 x.axis.labels = x.axis.labels,
								 x.axis.angle = x.axis.angle,
								 print = FALSE, ...)
			plots[[names(covar)[i]]] <- psub
		}
		class(plots) <- 'balance.plots'
		return(plots)
	} else {	
		if(!is.numeric(covar)) {
			covar <- as.character(covar)
		}
		
		#TODO: Much of this code is shared with plot.loess3
		tpsa <- attr(tmatch, 'triangle.psa')
		tmatch2 <- merge(x=tmatch, y=covar)
		groups <- names(tmatch2)[1:3]
		
		if(missing(model)) {
			for(i in 1:3) {
				if(length(which(is.na(tpsa[tpsa$treat %in% 
								groups[1:2],paste('model', i, sep='')]))) == 0) {
					model <- i
					break;
				}
			}
			if(model == 0) {
				stop('Could not find model. There are missing propensity scores in all models.')
			}
			if(print) {
				message(paste('Using propensity scores from model ', model, 
							  ' for evaluating balance.', sep=''))
			}
		} else {
			#Need to determine which groups these models represent
			if(length(which(is.na(tpsa[tpsa$treat %in% groups[1:2],
									paste('model', model, sep='')]))) == 0) {
				groups <- groups[c(1,2,3)]
			} else if(length(which(is.na(tpsa[tpsa$treat %in% groups[2:3],
									paste('model', model, sep='')]))) == 0) {
				groups <- groups[c(2,3,1)]
				
			} else if(length(which(is.na(tpsa[tpsa$treat %in% groups[c(1,3)],
									paste('model', model, sep='')]))) == 0) {
				groups <- groups[c(1,3,2)]			
			} else {
				stop(paste0('Could not use model ', model, 
						   '. There is are unexpected missing propensity scores'))
			}
		}
	
		tmatch2 <- merge(tmatch2, tpsa[which(tpsa$treat == groups[1]), 
									   c('id',paste('ps', model, sep=''))], 
						 by.x=groups[1], by.y='id', all.x=TRUE)
		names(tmatch2)[ncol(tmatch2)] <- paste(groups[1], '.ps', sep='')
		tmatch2 <- merge(tmatch2, tpsa[which(tpsa$treat == groups[2]), 
									   c('id',paste('ps', model, sep=''))], 
						 by.x=groups[2], by.y='id', all.x=TRUE)
		names(tmatch2)[ncol(tmatch2)] <- paste(groups[2], '.ps', sep='')
		tmatch2[,'mean.ps'] <- apply(tmatch2[,(ncol(tmatch2)-1):ncol(tmatch2)], 1, mean)
	
		breaks <- quantile(tmatch2$mean.ps, probs=seq(0,1,1/nstrata), na.rm=TRUE)
		tmatch2$strata <- cut(tmatch2$mean.ps, 
							  breaks=breaks, 
							  labels=1:nstrata, 
							  include.lowest=TRUE)
		
		tmatch2$id <- 1:nrow(tmatch2)
		
		badrows <- which(is.na(tmatch2$strata))
		if(length(badrows) > 0) {
			warning(paste('Could not determine strata for the following rows: ', 
						  paste(badrows, collapse=', '), sep=''))
			tmatch2 <- tmatch2[-badrows,] 
		}
		
		out <- melt(tmatch2[,c(paste(groups, '.out', sep=''), 'id', 'strata')], 
					id.vars=c('id','strata'))
		names(out) <- c('ID','Strata','Treatment','Covariate')	
		out$Treatment <- as.character(out$Treatment)
		#Strip .out from value
		out$Treatment <- substr(out$Treatment, 1, (sapply(out$Treatment, nchar)-4))
		
		p <- ggplot(out, aes(x=Treatment)) + facet_grid(~ Strata) 
		if(is.numeric(out$Covariate)) {
			df <- describeBy(out$Covariate, group=list(out$Treatment, out$Strata), 
							 mat=TRUE, skew=FALSE)[,c('group1','group2','mean','sd','se')]
			names(df) <- c('Treatment','Strata','Mean','SD','SE')
			df$ymin <- (df$Mean - se.ratio * df$SE)
			df$ymax <- (df$Mean + se.ratio * df$SE)
			p <- p + geom_boxplot(aes(y=Covariate)) +
				geom_point(data=df, aes(y=Mean), color='red', size=3) +
				geom_line(data=df, aes(y=Mean, group=Strata)) +
				geom_errorbar(data=df, aes(ymin=ymin, ymax=ymax), color='green', width=.5)
		} else { #Categorical varaible
			p <- p + geom_bar(aes(fill=factor(Covariate), y=(after_stat(count))/sum(after_stat(count))), 
							  position='fill')  +
				scale_fill_hue(label) + ylab('Percent') +
				scale_y_continuous(labels = percent) +
				theme(legend.position=legend.position)
		}
		
		p <- p + theme(axis.text.x=element_text(angle=x.axis.angle, vjust=.5),
					   panel.background=element_rect(color='black', fill='#F9F3FD'))
		p <- p + xlab(xlab) + ylab(ylab)
		
		if(!missing(x.axis.labels)) {
			p <- p + scale_x_discrete(labels=c('C','T1','T2'))
		}
		
		ft <- friedman.test(Covariate ~ Treatment | ID, out)
		if(print) { print(ft) }
		attr(p, 'friedman') <- ft
		
		if(is.numeric(out$Covariate)) {
			#We can also use repeated measure ANOVA for continuous covariates
			out$Treatment <- as.factor(out$Treatment)
			out$ID <- as.factor(out$ID)
			rmanova <- ezANOVA(data=out, dv=Covariate, wid=ID, within=Treatment)
			if(print) {
				cat(' Repeated measures ANOVA\n\n')
				print(rmanova$ANOVA)
			}
			attr(p, 'rmanova') <- rmanova
		}
		
		return(p)
	}
}

#' Print the results of [balance.plot()] for a data frame of covariates.
#' 
#' @param x the results of [balance.plot()] when a data frame is specified.
#' @param ... parameters passed to [plot.balance.plots()] and
#'        [summary.balance.plots()].
#' @method print balance.plots
#' @export
print.balance.plots <- function(x, ...) {
	print(summary(x, ...))
	plot(x, ...)
}

#' Prints a summary table of the test statistics of each balance plot.
#' 
#' The [balance.plot()] function will create a grid of balance plots
#' if a data frame is provided. The returned object is a list of \code{ggplot2}
#' figures with the statistical tests (i.e. Friedmen Rank Sum tests and if a
#' continuous variable, repeated measures ANOVA as well) saved as attributes.
#' This function will return a data frame combining all of those results.
#' 
#' @param object the results of [balance.plot()] when a data frame is specified.
#' @param ... currently unused.
#' @return a data frame
#' @export
#' @method summary balance.plots
summary.balance.plots <- function(object, ...) {
	friedman.results <- data.frame(Covariate=character(), 
								   Friedman=numeric(), Friedman.p=numeric(),
								   rmANOVA=numeric(), rmANOVA.p=numeric(),
								   stringsAsFactors=FALSE)
	for(i in seq_along(object)) {
		f <- attr(object[[i]], 'friedman')
		a <- attr(object[[i]], 'rmanova')
		friedman.results <- rbind(friedman.results, data.frame(
			Covariate = names(object)[i],
			Friedman = f$statistic,
			Friedman.p = f$p.value,
			Friedman.sig = star(f$p.value),
			rmANOVA = ifelse(is.null(a), NA, a[[1]]$F),
			rmANOVA.p = ifelse(is.null(a), NA, a[[1]]$p),
			rmANOVA.sig = star(ifelse(is.null(a), NA, a[[1]]$p)),
			stringsAsFactors=FALSE
		))
	}
	row.names(friedman.results) <- 1:nrow(friedman.results)
	return(friedman.results)
}

#' Prints a grid of balance plots.
#' 
#' @param x the results of [balance.plot()] when a data frame is specified.
#' @param rows if \code{covar} is a data frame of covariates, the number of
#'        rows in the grid of figures.
#' @param cols if \code{covar} is a data frame of covariates, the number of
#'        columns in the grid of figures.
#' @param byrow if TRUE (default), plots will be drawn by rows, otherwise by columns.
#' @param plot.sequence the sequence (or subset) of plots to draw.
#' @param ... currently unused.
#' @method plot balance.plots
#' @export
plot.balance.plots <- function(x, rows, cols, byrow = TRUE, 
							   plot.sequence=seq_along(bplots), ...) {
	bplots <- x
	grid.newpage()
	if(missing(rows) & missing(cols)) {
		rows <- ceiling(sqrt(length(bplots)))
		cols <- ceiling(length(bplots) / rows)
	} else if(missing(rows)) {
		rows <- ceiling(length(bplots) / cols)
	} else if(missing(cols)) {
		cols <- ceiling(length(bplots) / rows)
	}
	pushViewport(viewport(layout = grid.layout(rows, cols)))
	vplayout <- function(x, y) viewport(layout.pos.row = x, layout.pos.col = y)
	for(i in seq_along(plot.sequence)) {
		psub <- bplots[[plot.sequence[i]]]
		if(byrow) {
			print(psub, vp=vplayout( ceiling(i / cols) , (i-1) %% cols + 1 ))
		} else {
			print(psub, vp=vplayout( (i-1) %% rows + 1 , ceiling(i / rows) ))
		}
	}
}

utils::globalVariables(c('Treatments','Difference','V1','V2','estimate'))

#' Returns a \code{ggplot2} box plot of the differences.
#' 
#' A boxplot of differences between each pair of treatments.
#' 
#' @param tmatch the results from [trimatch()].
#' @param out a vector of the outcome measure of interest.
#' @param plot.mean logical indicating whether the means should be plotted.
#' @param ordering specify the order for doing the paired analysis, that is
#'        analysis will be conducted as:
#'        \code{ordering[1] - ordering[2]}, \code{ordering[1] - ordering[3]},
#'        and \code{ordering[2] - ordering[3]}.
#' @param ci.width the width for the confidence intervals.
#' @return a \code{ggplot2} boxplot of the differences.
#' @export
boxdiff.plot <- function(tmatch, out, plot.mean=TRUE, 
						 ordering = attr(tmatch, 'match.order'),
						 ci.width=.5) {
	tmatch.out <- merge(tmatch, out)
	outcomes <- sapply(ordering, function(x) { which(names(tmatch.out) == paste0(x, '.out')) })
	tmatch.out$id <- 1:nrow(tmatch.out)
	
	diffcols <- c(
		paste(names(tmatch.out)[outcomes[c(1,2)]], collapse='-'),
		paste(names(tmatch.out)[outcomes[c(1,3)]], collapse='-'),
		paste(names(tmatch.out)[outcomes[c(2,3)]], collapse='-')
	)
	
	tmatch.out[, diffcols[1] ] <- tmatch.out[,outcomes[1]] - tmatch.out[,outcomes[2]]
	tmatch.out[, diffcols[2] ] <- tmatch.out[,outcomes[1]] - tmatch.out[,outcomes[3]]
	tmatch.out[, diffcols[3] ] <- tmatch.out[,outcomes[2]] - tmatch.out[,outcomes[3]]
	
	#Individual t-tests
	t1 <- t.test(x=tmatch.out[,outcomes[1]], y=tmatch.out[,outcomes[2]], paired=TRUE)
	t2 <- t.test(x=tmatch.out[,outcomes[1]], y=tmatch.out[,outcomes[3]], paired=TRUE)
	t3 <- t.test(x=tmatch.out[,outcomes[2]], y=tmatch.out[,outcomes[3]], paired=TRUE)
	
	ci <- as.data.frame(rbind(t1$conf.int, t2$conf.int, t3$conf.int))
	ci$Treatments <- diffcols
	ci$estimate <- c(t1$estimate, t2$estimate, t3$estimate)

	out.box <- melt(tmatch.out[,c('id',diffcols)], id.vars='id')
	names(out.box) <- c('ID','Treatments','Difference')
	
	#Create new labels for each box
	labels <- gsub('-', ' - ', gsub('.out', '', diffcols))
	names(labels) <- diffcols
	
	p <- ggplot(out.box, aes(x=Treatments, y=Difference)) + 
		geom_boxplot() + 
		geom_crossbar(data=ci, aes(x=Treatments, ymin=V1, ymax=V2, y=estimate), 
					  color='green', fill='green', width=ci.width, alpha=.6) +
		geom_hline(color='blue', yintercept=0)
	if(plot.mean) {
		p <- p + 
			geom_point(data=ci, aes(x=Treatments, y=estimate), color='red', size=3) +
			geom_text(data=ci, aes(x=Treatments, y=estimate, 
					  label=prettyNum(estimate, digits=2)), vjust=-1)
	}
	
	p <- p + scale_x_discrete(NULL, labels=labels) + xlab(NULL)
	
	return(p)
}

#' Recursive function to find possible matched triplets using the apply functions.
#' 
#' Internal method. This version does not use the exact matching. Instead, this
#' function should be called separately for each grouping.
#' 
#' @inheritParams trimatch
#' @param sd1 standard deviation for propensity scores from model 1.
#' @param sd2 standard deviation for propensity scores from model 2.
#' @param sd3 standard deviation for propensity scores from model 3.
trimatch.apply2 <- function(tpsa, caliper, nmatch, match.order, sd1, sd2, sd3) {	
	ps1 <- getPS(tpsa, match.order[1], match.order[2])
	ps2 <- getPS(tpsa, match.order[2], match.order[3])
	ps3 <- getPS(tpsa, match.order[3], match.order[1])
	
	#Calculate the distances
	ps1.2 <- ps1[tpsa$treat == match.order[2]]
	ps1.1 <- ps1[tpsa$treat == match.order[1]]
	d1 <- abs(sapply(ps1.2, FUN=function(x) { x - ps1.1 })) / sd1		
	dimnames(d1) <- list(
		tpsa[tpsa$treat == match.order[1],'id'],
		tpsa[tpsa$treat == match.order[2],'id']	)
	
	ps2.3 <- ps2[tpsa$treat == match.order[3]]
	ps2.2 <- ps2[tpsa$treat == match.order[2]]
	d2 <- abs(sapply(ps2.3,FUN=function(x) { x - ps2.2 })) / sd2
	dimnames(d2) <- list(
		tpsa[tpsa$treat == match.order[2],'id'],
		tpsa[tpsa$treat == match.order[3],'id']	)
	
	ps3.1 <- ps3[tpsa$treat == match.order[1]]
	ps3.3 <- ps3[tpsa$treat == match.order[3]]
	d3 <- abs(sapply(ps3.1, FUN=function(x) { x - ps3.3 })) / sd3
	dimnames(d3) <- list(
		tpsa[tpsa$treat == match.order[3],'id'],
		tpsa[tpsa$treat == match.order[1],'id']	)

	#Check the caliper
	d1[d1 > caliper[1]] <- NA
	d2[d2 > caliper[2]] <- NA
	d3[d3 > caliper[3]] <- NA
	
	match1 <- function(i1) {
		row1 <- d1[i1,]
		row1 <- row1[!is.na(row1)]
		row1 <- row1[order(row1)]
		nxt <- names(row1)[seq_len(min(nmatch[1], length(row1)))]
		if(length(nxt) > 0) {
			return(lapply(nxt, match2, i1=i1, row1=row1))
		} else {
			return(c())
		}
	}
	
	match2 <- function(i2, i1, row1) {
		row2 <- d2[i2,]
		row2 <- row2[!is.na(row2)]
		row2 <- row2[order(row2)]
		nxt <- names(row2)[seq_len(min(nmatch[2], length(row2)))]
		if(length(nxt) > 0) {
			return(lapply(nxt, match3, i1=i1, i2=i2, row1=row1, row2=row2))
		} else {
			return(c())
		}
	}
	
	match3 <- function(i3, i1, i2, row1, row2) {
		val <- d3[i3,i1]
		if(is.na(val)) {
			return(c())
		} else {
			return(c(i1, i2, i3, row1[i2], row2[i3], val))
		}
	}
	
	results <- lapply(dimnames(d1)[[1]], match1)
	results <- unlist(results)
	if(!is.null(results)) {
		results <- as.data.frame(matrix(results, ncol=6, byrow=TRUE), stringsAsFactors=FALSE)
		for(i in 4:6) { results[,i] <- as.numeric(results[,i]) }
		invisible(results)
	} else {
		invisible(c())
	}
}

utils::globalVariables(c('Outcome','ID','group1'))

#' Parallel coordinate plot for the three groups and dependent variable.
#' 
#' Creates a \code{ggplot2} figure of a parallel coordinate plot.
#' 
#' @param tmatch results from [trimatch()].
#' @param outcome vector of the outcome
#' @export
parallel.plot <- function(tmatch, outcome) {
	tmatch <- merge(tmatch, outcome)
	outcomes <- grep("*.out", names(tmatch))
	tmatch$id <- 1:nrow(tmatch)
	out <- melt(tmatch[,c(outcomes, ncol(tmatch))],id.vars='id')
	names(out) <- c('ID','Treatment','Outcome')
	out2 <- describeBy(out$Outcome, group=out$Treatment, mat=TRUE)
	p <- ggplot(out) + 
		geom_line(aes(x=Treatment, y=Outcome, group=ID, colour=ID), alpha=.1) +
		theme(legend.position='none') + scale_colour_gradient() +
		geom_point(data=out2, aes(x=group1, y=mean), color='red', shape=20, size=4)
	return(p)
}

utils::globalVariables(c('ps','treat','id'))

#' Loess plot for matched triplets.
#' 
#' This function will create a \code{ggplot2} figure with propensity scores on the
#' x-axis and the outcome on the y-axis. Three Loess regression lines will be plotted
#' based upon the propensity scores from \code{model}. Since each model produces
#' propensity scores for two of the three groups, the propensity score for the third
#' group in each matched triplet will be the mean of the other two. If \code{model}
#' is not specified, the default will be to use the model that estimates the propensity
#' scores for the first two groups in the matching order.
#' 
#' @param tmatch the results of [trimatch()].
#' @param outcome a vector representing the outcomes.
#' @param model an integer between 1 and 3 indicating from which model the propensity
#'        scores will be used.
#' @param ylab the label for the y-axis.
#' @param plot.connections boolean indicating whether lines will be drawn connecting
#'        each matched triplet.
#' @param connections.color the line color of connections.
#' @param connections.alpha number between 0 and 1 representing the alpha levels for 
#'        connection lines.
#' @param plot.points a \code{ggplot2} function for plotting points. Usually 
#'        [ggplot2::geom_point()] or [ggplot2::geom_jitter()]. If `NULL` no points
#'        will be drawn.
#' @param points.alpha number between 0 and 1 representing the alpha level for the points.
#' @param points.palette the color palette to use. See [ggplot2::scale_colour_brewer()]
#'        and \url{https://colorbrewer2.org} for more information.
#' @param ... other parameters passed to [ggplot2::geom_smooth()] and 
#'        [ggplot2::stat_smooth()].
#' @return a `ggplot2` figure.
#' @export
loess3.plot <- function(tmatch, outcome, model,
						ylab='Outcome', 
						plot.connections=FALSE,
						connections.color='black',
						connections.alpha=.2,
						plot.points=geom_point,
						points.alpha=.1,
						points.palette='Dark2', ...) {
	tpsa <- attr(tmatch, 'triangle.psa')
	tmatch2 <- merge(x=tmatch, y=outcome)
	groups <- names(tmatch2)[1:3]
	
	if(missing(model)) {
		for(i in 1:3) {
			if(length(which(is.na(tpsa[tpsa$treat %in% groups[1:2],
									   paste('model', i, sep='')]))) == 0) {
				model <- i
				break;
			}
		}
		if(model == 0) {
			stop('Could not find model. There are missing propensity scores in all models.')
		}
	} else {
		groups <- c(
			as.character(tpsa[which(!tpsa[,paste0('model',model)]),'treat'][1]),
			as.character(tpsa[which(tpsa[,paste0('model',model)]),'treat'][1]),
			as.character(tpsa[which(is.na(tpsa[,paste0('model',model)])),'treat'][1])
		)
	}

	xlab <- paste0('Propensity Score (0=', 
				   tpsa[which(!tpsa[,paste0('model',model)]),'treat'][1],
				   ', 1=',
				   tpsa[which(tpsa[,paste0('model',model)]),'treat'][1],
				   ')')
	
	tmatch2 <- merge(tmatch2, tpsa[which(tpsa$treat == groups[1]), 
								   c('id',paste('ps', model, sep=''))], 
					 by.x=groups[1], by.y='id', all.x=TRUE)
	names(tmatch2)[ncol(tmatch2)] <- paste(groups[1], '.ps', sep='')
	tmatch2 <- merge(tmatch2, tpsa[which(tpsa$treat == groups[2]), 
								   c('id',paste('ps', model, sep=''))], 
					 by.x=groups[2], by.y='id', all.x=TRUE)
	names(tmatch2)[ncol(tmatch2)] <- paste(groups[2], '.ps', sep='')
	tmatch2[,paste(groups[3], '.ps', sep='')] <- apply(
			tmatch2[,(ncol(tmatch2)-1):ncol(tmatch2)], 1, mean)
	
	tmatch2$id <- 1:nrow(tmatch2)
	
	out <- cbind(melt(tmatch2[,c(paste(groups, '.out', sep=''), 'id')], id.vars='id'),
	             melt(tmatch2[,c(paste(groups, '.ps', sep=''), 'id')], id.vars='id') )[,c(1,2,3,6)]
	names(out) <- c('id','treat','out','ps')
	out$treat <- as.character(out$treat)
	out$treat <- substr(out$treat, 1, (sapply(out$treat, nchar)-4)) #Strip .out from value
	
	p <- ggplot(out, aes(x=ps, y=out, group=treat, fill=treat, colour=treat))
	if(plot.connections) {
		p <- p + geom_path(aes(group=id), colour=connections.color, alpha=connections.alpha)
	}
	if(!is.null(plot.points)) {
		p <- p + plot.points(alpha=points.alpha)
	}
	p <- p + geom_smooth(...)
	p <- p + ylab(ylab) + xlab(xlab)
	p <- p + scale_color_brewer('Treatment', palette=points.palette)
	p <- p + scale_fill_brewer('Treatment', palette=points.palette)
	p
}

#' Creates matched triplets.
#' 
#' Create matched triplets by minimizing the total distance between matched triplets
#' within a specified caliper.
#' 
#' The [trips()] function will estimate the propensity scores
#' for three models. This method will then find the best matched triplets based
#' upon minimizing the summed differences between propensity scores across the
#' three models. That is, the algorithm works as follows:
#' 
#' \itemize{
#' \item The first subject from model 1 is selected.
#' \item The \code{nmatch[1]} smallest distances are selected using propensity scores from
#'  model 1.
#' \item For each of the matches identified, the subjects propensity score from model
#'  2 is retrieved.
#' \item The \code{nmatch[2]} smallest distances are selected using propensity score from
#'  model 3.
#' \item For each of those matches identified, the subjects propensity score from model
#'  2 is retrieved.
#' \item The distances is calculated from the first and last subjects propensity scores
#'  from model 2.
#' \item The three distances are summed.
#' \item The triplet with the smallest overall distance is selected and returned.
#' }
#' 
#' @param tpsa the results from [trips()]
#' @param caliper a vector of length one or three indicating the caliper to use 
#'        for matching within each step. This is expressed in standardized units such 
#'        that .25 means that matches must be within .25 of one standard deviation 
#'        to be kept, otherwise the match is dropped.
#' @param nmatch number of closest matches to retain before moving to next edge. This can
#'        be \code{Inf} in which case all matches within the caliper will be retained
#'        through to the next step. For large datasets, evaluating all possible
#'        matches within the caliper could be time consuming. 
#' @param match.order character vector of length three indicating the order in 
#'        which the matching algorithm will processes. The default is to use start
#'        with the group the middle number of subjects, followed by the smallest,
#'        and then the largest. 
#' @param exact a vector or data frame of representing covariates for exact matching.
#'        That is, matched triplets will first be matched exactly on these covariates
#'        before evaluating distances.
#' @param method This is a function that specifies which matched triplets will be
#'        retained. If `NULL`, all matched triplets within the specified
#'        caliper will be returned (equivalent to caliper matching in two group
#'        matching). The default is [maximumTreat()] that
#'        attempts include each treatment at least once. 
#'        Another option is [OneToN()] which mimicks the one-to-n 
#'        matching where treatments are matched to multiple control units.
#' @param ... other parameters passed to `method`.
#' @export
#' @examples
#' \dontrun{
#' data(turoing)
#' formu <- ~ Gender + Ethnicity + Military + ESL + EdMother + EdFather + Age +
#'      Employment + Income + Transfer + GPA
#' tpsa <- trips(tutoring, tutoring$treat, formu)
#' tmatch <- trimatch(tpsa, status=FALSE)
#' }
trimatch <- function(tpsa, 
					 caliper=.25, 
					 nmatch=c(15),
					 match.order,
					 exact,
					 method=maximumTreat, 
					 ...) {
	if(length(nmatch) == 1) {
		nmatch <- c(nmatch, nmatch)
	}
	if(length(caliper) == 1) {
		caliper <- rep(caliper, 3)
	}
	
	groups <- attr(tpsa, 'groups')
	if(missing(match.order)) {
		group.sizes <- table(tpsa$treat)
		ordering <- order(group.sizes, decreasing=FALSE)
		ordering <- ordering[c(2,1,3)]
		match.order <- names(group.sizes)[ordering]
	} else {
		ordering <- c(
			which(groups == match.order[1]),
			which(groups == match.order[2]),
			which(groups == match.order[3]))
	}

	ps1 <- getPS(tpsa, match.order[1], match.order[2])
	ps2 <- getPS(tpsa, match.order[2], match.order[3])
	ps3 <- getPS(tpsa, match.order[3], match.order[1])

	sd1 <- sd(ps1, na.rm=TRUE)
	sd2 <- sd(ps2, na.rm=TRUE)
	sd3 <- sd(ps3, na.rm=TRUE)
	
	if(!missing(exact)) {
		if(class(exact) %in% c('matrix','data.frame')) {
			exact <- apply(exact, 1, paste, collapse='.')
		}
	
		results <- data.frame()
		for(i in unique(exact)) {
			tmp <- tpsa[exact == i,]
			if(all(table(tmp$treat) > 1)) {
				results <- rbind(results, trimatch.apply2(
					tmp, caliper=caliper, nmatch=nmatch, match.order=match.order,
					sd1=sd1, sd2=sd2, sd3=sd3
				))
			} #Else there are not at least one unit from each group that exactly match
		}
	} else {
		results <- trimatch.apply2(tpsa, caliper, nmatch, match.order=match.order,
								   sd1=sd1, sd2=sd2, sd3=sd3)
	}
	
	names(results) <- c(match.order,
			paste0('D.', getModel(tpsa, match.order[1],match.order[2])),
			paste0('D.', getModel(tpsa, match.order[2],match.order[3])),
			paste0('D.', getModel(tpsa, match.order[3],match.order[1]))
	)
	
	results$Dtotal <- results$D.m1 + results$D.m2 + results$D.m3
	results <- results[order(results$Dtotal),]
	
	if(!is.null(method)) {
		results <- method(results, ...)
		row.names(results) <- 1:nrow(results)
	}
	
	unmatched <- tpsa[!(tpsa$id %in% c(results[,1], results[,2], results[,3])),]

	class(results) <- c('triangle.matches','data.frame')
	attr(results, 'triangle.psa') <- tpsa
	attr(results, 'match.order') <- match.order
	attr(results, 'unmatched') <- unmatched
	attr(results, getModel(tpsa, match.order[1],match.order[2])) <- c(match.order[1],match.order[2])
	attr(results, getModel(tpsa, match.order[2],match.order[3])) <- c(match.order[2],match.order[3])
	attr(results, getModel(tpsa, match.order[3],match.order[1])) <- c(match.order[3],match.order[1])
	
	return(results)
}

getModel <- function(tpsa, x, y) {
	test <- tpsa[which(tpsa$treat %in% c(x,y)),c('model1','model2','model3')]
	if(length(which(is.na(test$model1))) == 0) {
		return('m1')
	} else if(length(which(is.na(test$model2))) == 0) {
		return('m2')
	} else if(length(which(is.na(test$model3))) == 0) {
		return('m3')
	} else {
		stop('Could not find model.')
	}
}

getPS <- function(tpsa, x, y) {
	test <- tpsa[which(tpsa$treat %in% c(x,y)),c('model1','model2','model3')]
	if(length(which(is.na(test$model1))) == 0) {
		return(tpsa$ps1)
	} else if(length(which(is.na(test$model2))) == 0) {
		return(tpsa$ps2)
	} else if(length(which(is.na(test$model3))) == 0) {
		return(tpsa$ps3)
	} else {
		stop('Could not find model.')
	}
}

#' This method will return at least one treatment from groups one and two within
#' the caliper.
#' 
#' This method will attempt to return enough rows to use each treatment (the first
#' two groups in the matching order) at least once. Assuming treat1 is the first
#' group in the match order and treat2 the second, all duplicate treat1 rows
#' are removed. Next, all treat2 units not in present in after removing duplicate
#' treat1 units are identified. For each of those treat2 units, the matched
#' triplet with the smallest overall distances where treat2 is one of the mathched
#' units is retained.
#' 
#' @param tmatch initial results from [trimatch()] that contains all
#'        possible matches within the specified caliper.
#' @param ... currently unused.
#' @export
maximumTreat <- function(tmatch, ...) {
	keep <- which(!duplicated(tmatch[,1]))
	t2.ids <- unique(tmatch[keep,2])
	retain <- tmatch[keep,]
	keep.t2 <- tmatch[-which(tmatch[,2] %in% t2.ids),]
	keep.t2 <- keep.t2[!duplicated(keep.t2[,2]),]
	retain <- rbind(retain, keep.t2)
	return(retain)
}

#' This method will use a M1-to-M2-to-1 matching.
#' 
#' In this method, \code{M2} corresponds to the number of times a treat1 unit can be
#' matched with a treat2 unit. The \code{M1} parameter corresponds to the number of
#' times a treat1 unit can be used in total.
#' 
#' @param tmatch initial results from [trimatch()] that contains all
#'        possible matches within the specified caliper.
#' @param M1 a scaler indicating the number of unique subjects in group one to
#'        retain. This applies only to the first group in the matching order.
#' @param M2 a scaler indicating the number of unique matches to retain. This applies
#'        to the first two groups in the matching order.
#' @param ... currently unused.
#' @export
OneToN <- function(tmatch, M1=2, M2=1, ...) {
	results <- tmatch
	retain <- data.frame()
	for(i in 1:M2) {
		keep <- which(!duplicated(results[, 2]))
		if(length(keep) > 0) {
			retain <- rbind(retain, results[keep,])
			results <- results[-keep,]
		}
	}
	results <- retain
	row.names(results) <- 1:nrow(results)
	
	retain <- data.frame()
	for(i in 1:M1) {
		keep <- which(!duplicated(results[,1]))
		if(length(keep) > 0) {
			retain <- rbind(retain, results[keep,])
			results <- results[-keep,]
		}
	}
	return(retain)
}

#' Euclidean distance calculation.
#' 
#' This method uses a simple Euclidean distance calculation for determining the
#' distances between two matches. That is, |ps1 - ps2|.
#' 
#' @param x vector of propensity scores.
#' @param grouping vector or factor identifying group membership.
#' @param id vector corresponding to unique identifer for each element in 
#'        \code{x} and \code{grouping}.
#' @param groups vector of length two indicating the unique groups to calculate
#'        the distance between. The first element will be the rows, the second columns.
#' @param caliper a scaler indicating the caliper to use for matching within
#'        each step.
#' @param nmatch number of smallest distances to retain.
#' @return a list of length equal to \code{x}. Each element of the list is a
#'        named numeric vector where the values correspond to the distance and the
#'        name to the \code{id}.
distance.euclid <- function(x, grouping, id, groups, caliper, nmatch=Inf) {
	sd <- sd(x, na.rm=TRUE)
	tmp <- data.frame(ps=x, group=grouping, id=id, stringsAsFactors=FALSE)
	tmp <- tmp[!is.na(tmp$ps),]
	tmp.1 <- tmp[tmp$group == groups[1],]
	tmp.2 <- tmp[tmp$group == groups[2],]
	d.list <- list()
	for(i in 1:nrow(tmp.1)) {
		dist <- abs(tmp.1[i,]$ps - tmp.2$ps)
		names(dist) <- tmp.2$id
		dist <- dist[order(dist)]
		dist <- dist / sd #Convert to standized units
		dist <- dist[1:min(length(dist), nmatch)]
		dist <- dist[dist < caliper]
		d.list[[i]] <- dist
	}
	names(d.list) <- as.character(tmp.1$id)
	return(d.list)
}

utils::globalVariables(c('y','treat','name','angle','vjust','hjust','xend','yend','strata','label'))

#' Triangle plot.
#' 
#' Triangle plot showing the fitted values (propensity scores) for three different
#' models.
#' 
#' @param x the results from [trips()].
#' @param point.alpha alpha level for points.
#' @param point.size point size.
#' @param legend.title title for the legend.
#' @param text.size text size.
#' @param draw.edges draw edges of the triangle.
#' @param draw.segments draw segments connecting points across two models.
#' @param edge.alpha alpha level for edges if drawn.
#' @param edge.color the color for edges if drawn.
#' @param edge.labels the labels to use for each edge of the triangle.
#' @param sample a vector of length 1 or 3 representing the sample of points to plot.
#'        The position of each element corresponds to the groups as returned
#'        by \code{attr(tpsa,'groups')}. If equal to one, all points will be plotted.
#'        Values less than one will plot a percentage of points. Values greater
#'        than one exactly that number of points will be plotted.
#' @param ... currently unused.
#' @return ggplot2 figure
#' @seealso triangle.psa
#' @method plot triangle.psa
#' @export
plot.triangle.psa <- function(x, 	
							  point.alpha = .3,
							  point.size = 1.5,
							  legend.title = 'Treatment',
							  text.size = 4,
							  draw.edges = FALSE,
							  draw.segments = TRUE,
							  edge.alpha = .2,
							  edge.color = 'grey',
							  edge.labels = c('Model 1', 'Model 2', 'Model 3'),
							  #ordering = attr(tpsa, 'groups'),
							  sample = c(1),
							  ...) {
	tpsa <- x
	if(length(sample) == 1) {
		sample <- rep(sample,3)
	}
	
	expand <- .2
	axis.offset <- .1
	
	groups <- attr(tpsa, 'groups')
	#groups <- ordering
	nstrata <- attr(tpsa, 'nstrata')
	m <- sqrt(.75)
	
	sampleRows <- function(rows, size) {
		if(size <= 1) {
			rows <- sample(rows, size * length(rows))
		} else {
			rows <- sample(rows, min(size, length(rows)))
		}
		return(rows)
	}
	
	rows1 <- sampleRows(which(tpsa$treat == groups[1]), sample[1])
	rows2 <- sampleRows(which(tpsa$treat == groups[2]), sample[2])
	rows3 <- sampleRows(which(tpsa$treat == groups[3]), sample[3])
	tpsa <- tpsa[c(rows1, rows2, rows3),]
	
	pts1 <- as.data.frame(matrix(unlist(lapply(tpsa$ps1, segment1)), ncol=2, byrow=TRUE))
	pts2 <- data.frame(x=tpsa$ps2, y=ifelse(is.na(tpsa$ps2), NA, 0))
	pts3 <- as.data.frame(matrix(unlist(lapply(tpsa$ps3, segment2)), ncol=2, byrow=TRUE))
	names(pts1) <- names(pts2) <- names(pts3) <- c('x','y')
	pts1$treat <- pts2$treat <- pts3$treat <- tpsa$treat
	pts1$name <- pts2$name <- pts3$name <- 1:nrow(pts1)
	pts <- rbind(pts1, pts2, pts3)
	
	cols <- c('blue', 'red', 'green')
	names(cols) <- groups
	
	#Tick marks for strata
	q1 <- as.data.frame(attr(tpsa, 'breaks1'))
	q2 <- as.data.frame(attr(tpsa, 'breaks2'))
	q3 <- as.data.frame(attr(tpsa, 'breaks3'))
	names(q1) <- names(q2) <- names(q3) <- 'ps'
	
	tmp <- as.data.frame(matrix(unlist(lapply(q1$ps, segment1)), ncol=2, byrow=TRUE))
	names(tmp) <- c('x','y')
	q1 <- cbind(q1, tmp)
	q1 <- cbind(q1, t(as.data.frame(apply(q1, 1, FUN=function(x) { 
		pt <- perpPt(x=x['x'], y=x['y'])
		pt[1] <- x['x'] - pt[1]
		pt[2] <- x['y'] + pt[2]
		return(pt)
	}))))
	names(q1)[4:5] <- c('xend', 'yend')
	
	q1.labels <- data.frame(
		strata=1:nstrata,
		x=unlist(lapply(2:nrow(q1), function(x) mean(c(q1[x-1,'xend'], q1[x,'xend'])))),
		y=unlist(lapply(2:nrow(q1), function(x) mean(c(q1[x-1,'yend'], q1[x,'yend'])))) )
	
	q2$x <- q2$ps
	q2$y <- rep(0, nrow(q2))
	q2$xend <- q2$x
	q2$yend <- rep(-.05, nrow(q2))
	
	q2.labels <- data.frame(
		strata=1:nstrata,
		x=unlist(lapply(2:nrow(q2), function(x) mean(c(q2[x-1,'xend'], q2[x,'xend'])))),
		y=unlist(lapply(2:nrow(q2), function(x) mean(c(q2[x-1,'yend'], q2[x,'yend'])))) )
	
	tmp <- as.data.frame(matrix(unlist(lapply(q3$ps, segment2)), ncol=2, byrow=TRUE))
	names(tmp) <- c('x','y')
	q3 <- cbind(q3, tmp)
	q3 <- cbind(q3, t(as.data.frame(apply(q3, 1, FUN=function(x) { 
		pt <- perpPt(x=x['x'], y=x['y'])
		pt[1] <- x['x'] + pt[1]
		pt[2] <- x['y'] + pt[2]
		return(pt)
	}))))
	names(q3)[4:5] <- c('xend', 'yend')
	
	q3.labels <- data.frame(
		strata=1:nstrata,
		x=unlist(lapply(2:nrow(q3), function(x) mean(c(q3[x-1,'xend'], q3[x,'xend'])))),
		y=unlist(lapply(2:nrow(q3), function(x) mean(c(q3[x-1,'yend'], q3[x,'yend'])))) )
	
	labels <- data.frame(label=groups, x=c(0, .5, 1), y=c(0, m, 0), angle=c(-45, 0, 45),
						 vjust=c(3,-2,3), hjust=c(.7, .5, .3))
	
	p <- ggplot(data=tpsa)
	
	if(draw.edges) {
		p <- p +
			geom_segment(x=0, y=0, xend=.5, yend=m, alpha=edge.alpha, colour=edge.color) +
			geom_segment(x=.5, y=m, xend=1, yend=0, alpha=edge.alpha, colour=edge.color) +
			geom_segment(x=0, y=0, xend=1, yend=0, alpha=edge.alpha, colour=edge.color)
	}
	
	#TODO: draw grid
	if(draw.segments) {
		p <- p + geom_line(data=pts, aes(x=x, y=y, colour=treat, group=name), alpha=edge.alpha)
	}
	
	p <- p +
		geom_text(data=labels, aes(x=x, y=y, angle=angle, label=label, vjust=vjust, hjust=hjust)) +
		geom_point(data=pts, aes(x=x, y=y, colour=treat), alpha=point.alpha, size=point.size) +
		scale_color_manual(legend.title, values=cols)
	
	p <- p +
		xlim(c(-expand,1 + expand)) + ylim(c(-expand, (m + expand))) + 
		xlab(NULL) + ylab(NULL) +
		coord_equal() +
		theme(axis.text=element_blank(), axis.ticks=element_blank(), panel.grid=element_blank())
	
	#Plot strata
	p <- p + 
		geom_segment(data=q1, aes(x=x, y=y, xend=xend, yend=yend)) +
		geom_segment(data=q2, aes(x=x, y=0, xend=xend, yend=-.05)) +
		geom_segment(data=q3, aes(x=x, y=y, xend=xend, yend=yend)) +
		geom_text(data=q1.labels, aes(x=x, y=y, label=strata), angle=60, size=text.size) +
		geom_text(data=q2.labels, aes(x=x, y=y, label=strata), angle=0, size=text.size) +
		geom_text(data=q3.labels, aes(x=x, y=y, label=strata), angle=-60, size=text.size)
	
	#Label each side of the triangle
	a1 <- perpPt(segment1(.5)[1], segment1(.5)[2], d=axis.offset)
	a1[1] <- segment1(.5)[1] - a1[1]
	a1[2] <- segment1(.5)[2] + a1[2]
	a2 <- c(x=.5, y=-axis.offset)
	a3 <- perpPt(segment2(.5)[1], segment2(.5)[2], d=axis.offset)
	a3[1] <- segment2(.5)[1] + a3[1]
	a3[2] <- segment2(.5)[2] + a3[2]
	
	p <- p + 
		geom_text(label=edge.labels[1], x=a1[1], y=a1[2], angle=60, size=text.size) +
		geom_text(label=edge.labels[2], x=a2[1], y=a2[2], angle=0, size=text.size) +
		geom_text(label=edge.labels[3], x=a3[1], y=a3[2], angle=-60, size=text.size)
	
	p <- p + theme(legend.position=c(1,.9), legend.justification=c(1,1))
	
	class(p) <- c('triangle.plot', class(p))
	
	return(p)
}

#' Print method for [plot.triangle.psa()]. The primary purpose is to 
#' suppress the "Removed n rows containing missing values" warning printed
#' by `ggplot2`.
#' 
#' @param x a plot from [plot.triangle.psa()].
#' @param ... other parameters passed to ggplot2.
#' @export
#' @method print triangle.plot
print.triangle.plot <- function(x, ...) {
	suppressWarnings(NextMethod(x, ...))
}

#' Compute ANOVA
#' 
#' *Note that this adapted from the `ez` R package available here: https://github.com/mike-lawrence/ez
#' It appears the `ez` package is no longer maintained so the function was copied here.*
#' 
#' This function provides easy analysis of data from factorial experiments, including purely 
#' within-Ss designs (a.k.a. “repeated measures”), purely between-Ss designs, and mixed 
#' within-and-between-Ss designs, yielding ANOVA results, generalized effect sizes and 
#' assumption checks.
#' 
#' @details
#' 
#' ANCOVA is implemented by first regressing the DV against each covariate (after collapsing the 
#' data to the means of that covariate's levels per subject) and subtracting from the raw data the 
#' fitted values from this regression (then adding back the mean to maintain scale). These 
#' regressions are computed across Ss in the case of between-Ss covariates and computed within 
#' each Ss in the case of within-Ss covariates.
#' 
#' **Warning**
#' 
#' Prior to running (though after obtaining running ANCOVA regressions as described in the details 
#' section), dv is collapsed to a mean for each cell defined by the combination of wid and any 
#' variables supplied to within and/or between and/or diff. Users are warned that while convenient 
#' when used properly, this automatic collapsing can lead to inconsistencies if the pre-collapsed 
#' data are unbalanced (with respect to cells in the full design) and only the partial design is 
#' supplied to ezANOVA. When this is the case, use within_full to specify the full design to ensure 
#' proper automatic collapsing.
#' 
#' @param data Data frame containing the data to be analyzed.
#' @param dv Name of the column in data that contains the dependent variable. Values in this column 
#' must be numeric.
#' @param wid Name of the column in data that contains the variable specifying the case/Ss 
#' identifier. This should be a unique value per case/Ss.
#' @param within Names of columns in data that contain predictor variables that are manipulated 
#' (or observed) within-Ss. If a single value, may be specified by name alone; if multiple values, 
#' must be specified as a .() list.
#' @param within_full Same as within, but intended to specify the full within-Ss design in cases 
#' where the data have not already been collapsed to means per condition specified by within and 
#' when within only specifies a subset of the full design.
#' @param within_covariates Names of columns in data that contain predictor variables that are 
#' manipulated (or observed) within-Ss and are to serve as covariates in the analysis. If a single 
#' value, may be specified by name alone; if multiple values, must be specified as a .() list.
#' @param between Names of columns in data that contain predictor variables that are manipulated 
#' (or observed) between-Ss. If a single value, may be specified by name alone; if multiple values,
#'  must be specified as a .() list.
#' @param between_covariates Names of columns in data that contain predictor variables that are 
#' manipulated (or observed) between-Ss and are to serve as covariates in the analysis. If a single 
#' value, may be specified by name alone; if multiple values, must be specified as a .() list.
#' @param observed Names of columns in data that are already specified in either within or between 
#' that contain predictor variables that are observed variables (i.e. not manipulated). If a single 
#' value, may be specified by name alone; if multiple values, must be specified as a .() list. The 
#' presence of observed variables affects the computation of the generalized eta-squared measure of 
#' effect size reported by ezANOVA.
#' @param diff Names of any variables to collapse to a difference score. If a single value, may be 
#' specified by name alone; if multiple values, must be specified as a .() list. All supplied 
#' variables must be factors, ideally with only two levels (especially if setting the reverse_diff 
#' argument to TRUE).
#' @param reverse_diff Logical. If TRUE, triggers reversal of the difference collapse requested by 
#' diff. Take care with variables with more than 2 levels.
#' @param type Numeric value (either 1, 2 or 3) specifying the Sums of Squares "type" to employ 
#' when data are unbalanced (eg. when group sizes differ). type = 2 is the default because this will 
#' yield identical ANOVA results as type = 1 when data are balanced but type = 2 will additionally 
#' yield various assumption tests where appropriate. When data are unbalanced, users are warned that 
#' they should give special consideration to the value of type. type=3 will emulate the approach 
#' taken by popular commercial statistics packages like SAS and SPSS, but users are warned that this 
#' approach is not without criticism.
#' @param white.adjust Only affects behavior if the design contains only between-Ss predictor 
#' variables. If not FALSE, the value is passed as the white.adjust argument to Anova, which 
#' provides heteroscedasticity correction. See Anova for details on possible values.
#' @param detailed Logical. If TRUE, returns extra information (sums of squares columns, intercept 
#' row, etc.) in the ANOVA table.
#' @param return_aov Logical. If TRUE, computes and returns an aov object corresponding to the 
#' requested ANOVA (useful for computing post-hoc contrasts).
#' 
#' @return 
#' A list containing one or more of the following components:
#' 
#' \describe{
#' \item{ANOVA}{A data frame containing the ANOVA results.}
#' \item{Mauchly's Test for Sphericity}{If any within-Ss variables with >2 levels are present, a data frame containing the results of Mauchly's test for Sphericity. Only reported for effects >2 levels because sphericity necessarily holds for effects with only 2 levels.}
#' \item{Sphericity Corrections}{If any within-Ss variables are present, a data frame containing the Greenhouse-Geisser and Huynh-Feldt epsilon values, and corresponding corrected p-values.}
#' \item{Levene's Test for Homogeneity}{If the design is purely between-Ss, a data frame containing the results of Levene's test for Homogeneity of variance. Note that Huynh-Feldt corrected p-values where the Huynh-Feldt epsilon >1 will use 1 as the correction epsilon.}
#' \item{aov}{An aov object corresponding to the requested ANOVA.}
#' }
#' 
#' Some column names in the output data frames are abbreviated to conserve space:
#' 
#' \describe{
#' \item{DFn}{Degrees of Freedom in the numerator (a.k.a. DFeffect).}
#' \item{DFd}{Degrees of Freedom in the denominator (a.k.a. DFerror).}
#' \item{SSn}{Sum of Squares in the numerator (a.k.a. SSeffect).}
#' \item{SSd}{Sum of Squares in the denominator (a.k.a. SSerror).}
#' \item{F}{F-value.}
#' \item{p}{p-value (probability of the data given the null hypothesis).}
#' \item{p<.05}{Highlights p-values less than the traditional alpha level of .05.}
#' \item{ges}{Generalized Eta-Squared measure of effect size (see in references below: Bakeman, 2005).}
#' \item{GGe}{Greenhouse-Geisser epsilon.}
#' \item{p[GGe]}{p-value after correction using Greenhouse-Geisser epsilon.}
#' \item{p[GGe]<.05}{Highlights p-values (after correction using Greenhouse-Geisser epsilon) less than the traditional alpha level of .05.}
#' \item{HFe}{Huynh-Feldt epsilon.}
#' \item{p[HFe]}{p-value after correction using Huynh-Feldt epsilon.}
#' \item{p[HFe]<.05}{Highlights p-values (after correction using Huynh-Feldt epsilon) less than the traditional alpha level of .05.}
#' \item{W}{Mauchly's W statistic}
#' }
#' 
#' @author Mike Lawrence
#' @references Bakeman, R. (2005). Recommended effect size statistics for repeated measures designs. 
#' Behavior Research Methods, 37 (3), 379-384.
#' 
#' @import plyr
#' @importFrom car Anova
#' @importFrom stringr str_count str_detect
ezANOVA <-	function(
		data,
		dv,
		wid,
		within = NULL,
		within_full = NULL,
		within_covariates = NULL,
		between = NULL,
		between_covariates = NULL,
		observed = NULL,
		diff = NULL,
		reverse_diff = FALSE,
		type = 1,
		white.adjust = FALSE,
		detailed = FALSE,
		return_aov = FALSE) {
	if (inherits(data, "tbl_df")) {
		data <- as.data.frame(data)
	}
	
	args_to_check = c(
		'dv',
		'wid',
		'within',
		'between',
		'observed',
		'diff',
		'within_full',
		'within_covariates',
		'between_covariates'
	)
	
	args <- as.list(match.call()[-1])
	for(i in 1:length(args)) {
		arg_name <- names(args)[i]
		if(arg_name %in% args_to_check) {
			if(is.symbol(args[[i]])) {
				code <- paste(arg_name, '=.(', as.character(args[[i]]), ')', sep = '')
				eval(parse(text = code))
			} else {
				if(is.language(args[[i]])) {
					arg_vals <- as.character(args[[i]])
					arg_vals <- arg_vals[2:length(arg_vals)]
					arg_vals <- paste(arg_vals, collapse = ',')
					code <- paste(arg_name, '=.(', arg_vals, ')', sep = '')
					eval(parse(text = code))
				}
			}
		}
	}
	
	to_return = ezANOVA_main(
		data = data,
		dv = dv,
		wid = wid,
		within = within,
		within_full = within_full,
		within_covariates = within_covariates,
		between = between,
		between_covariates = between_covariates,
		diff = diff,
		reverse_diff = reverse_diff,
		type = type,
		return_aov = return_aov,
		white.adjust = white.adjust
	)
	
	########
	# Compute effect size
	########
	if(!white.adjust) {
		if (!is.null(observed)) {
			obs <- rep(F, nrow(to_return$ANOVA))
			for (i in as.character(observed)) {
				obs <- obs | stringr::str_detect(to_return$ANOVA$Effect, i)
			}
			obs_SSn1 <- sum(to_return$ANOVA$SSn * obs)
			obs_SSn2 <- to_return$ANOVA$SSn * obs
		} else{
			obs_SSn1 <- 0
			obs_SSn2 <- 0
		}
		to_return$ANOVA$ges <- to_return$ANOVA$SSn / (to_return$ANOVA$SSn + sum(unique(to_return$ANOVA$SSd)) +
													 	obs_SSn1 - obs_SSn2)
	}
	
	########
	# Final clean-up
	########
	
	#remove the data & type label from to_return
	to_return <- to_return[!(names(to_return) %in% c('data', 'type'))]
	
	#if necessary, remove extra columns and the Intercept row from the anova
	if (!detailed) {
		to_return$ANOVA <- to_return$ANOVA[, names(to_return$ANOVA) %in% c('Effect', 'DFn', 'DFd', 'F', 'p', 'p<.05', 'ges')]
		row_keep <- !stringr::str_detect(to_return$ANOVA$Effect, '(Intercept)')
		to_return$ANOVA <- to_return$ANOVA[row_keep, ]
	}
	
	#all done!
	return(to_return)
}

#' Merges covariate(s) with the results of [trips()].
#' 
#' The \code{y} parameter should be a subset of the original data used.
#' 
#' @param x the result of [trips()]
#' @param y another data frame or vector to merge with.
#' @param ... unused
#' @return \code{x} with the additional column(s) added.
#' @export
#' @method merge triangle.psa
merge.triangle.psa <- function(x, y, ...) {
	stopifnot('triangle.psa' %in% class(x))
	if(is.vector(y) & length(y) != nrow(x)) {
		stop('The length of the vector does not match that of the original data')
	} else if(nrow(y) != nrow(x)) {
		stop('The length of the vector does not match that of the original data')
	}
	x <- cbind(x, y)
	return(x)
}

utils::globalVariables(c('ID','Distance','Model','id'))

#' Barplot for the sum of distances.
#' 
#' @param tmatch the results of [trimatch()].
#' @param caliper a vector indicating where vertical lines should be drawn as a
#'        factor of the standard deviation. Rosenbaum and Rubin (1985) suggested
#'        one quarter of one standard deviation.
#' @param label label the bars that exceed the minimum caliper.
#' @seealso triangle.match
#' @export
distances.plot <- function(tmatch, caliper=.25, label=FALSE) {
	tpsa <- attr(tmatch, 'triangle.psa')
	tmatch$id <- row.names(tmatch)
	tmp <- melt(tmatch[,c('id','D.m1','D.m2','D.m3')], id.vars=1)
	names(tmp) <- c('ID','Model','Distance')
	l <- (sd(tpsa$ps1, na.rm=TRUE) + sd(tpsa$ps2, na.rm=TRUE) + sd(tpsa$ps3, na.rm=TRUE))
	stdev <- sapply(tpsa[,c('ps1','ps2','ps3')], sd, na.rm=TRUE)
	cat(paste("Standard deviations of propensity scores: ",
			  paste(prettyNum(stdev, digits=2), collapse=' + '),
			  " = ", prettyNum(l, digits=2), '\n', sep='' ))
	#sddf <- data.frame(Model=c('1','2','3'), stdev=stdev)
	#sddf$cumsum <- cumsum(sddf$stdev)
	sddf <- data.frame(Model=c('1','2','3'), cumsum=caliper * 1:3)
	levels(tmp$Model) <- levels(sddf$Model)
	p <- ggplot(tmp, aes(x=ID, y=Distance)) +
		geom_bar(aes(fill=Model), stat='identity', width=1) + coord_flip() + 
		#geom_hline(yintercept=3 * caliper, color='black') +
		geom_hline(data=sddf, aes(yintercept=cumsum, color=Model), linetype=2) +
		theme(axis.text.y=element_blank()) +
		xlab(NULL) + xlim(tmatch[order(tmatch$Dtotal),'id'])
	if(label & length(which(tmatch$Dtotal > 3 * (min(caliper)))) > 0) {
		p <- p + geom_text(data=tmatch[tmatch$Dtotal > 3 * (min(caliper)),], 
						   aes(x=id, y=0, label=id),
						   size=3, hjust=1)	
	}
	for(i in seq_along(caliper)) {
		cat(paste("Percentage of matches exceding a distance of ", 
				  prettyNum(caliper[i], digits=2), 
				  " (caliper = ", caliper[i], "): ", 
				  prettyNum(length(which(tmatch$Dtotal > 3 * caliper[i])) / 
				  		  	length(tmatch$Dtotal) * 100, digits=2), '%\n',
				  sep=''))
	}
	return(p)
}

utils::globalVariables(c('Outcome','ID','Treatment'))

#' Provides a summary of the matched triplets including analysis of outcome
#' measure if provided.
#' 
#' If an outcome measure is provided this function will perform a Freidman
#' Rank Sum Test and repeated measures ANOVA. If either test has a statistically
#' significant difference (as determined by the value of the \code{p} parameter),
#' a Pairwise Wilcoxon Rank Sum Test will also be provided.
#' 
#' @param object result of [trimatch()].
#' @param outcome vector representing the outcome measure.
#' @param p threshold of the p value to perform a 
#' @param ... parameters passed to other statistical tests.
#' @param ordering specify the order for doing the paired analysis, that is
#'        analysis will be conducted as:
#'        \code{ordering[1] - ordering[2]}, \code{ordering[1] - ordering[3]},
#'        and \code{ordering[2] - ordering[3]}.
#' @seealso [stats::friedman.test()], ez::ezANOVA(), [stats::pairwise.wilcox.test()]
#' @return a trimatch.summary object.
#' @method summary triangle.matches
#' @export
summary.triangle.matches <- function(object, outcome, p=.05, 
									 ordering = attr(object, 'match.order'), ...) {
	results <- list()
	
	tpsa <- attr(object, 'triangle.psa')
	
	tab <- table(tpsa$treat)
	tab2 <- c(length(unique(object[,names(tab)[1]])),
		length(unique(object[,names(tab)[2]])),
		length(unique(object[,names(tab)[3]])) )
	
	results$PercentMatched <- as.numeric(tab2 / tab)
	names(results$PercentMatched) <- names(tab)
		
	if(missing(outcome)) {
		stop('Outcome measure not specified.')
	} else {
		tmatch.out <- merge(object, outcome)
		#outcomes <- grep(".out$", names(tmatch.out), perl=TRUE)
		outcomes <- sapply(ordering, function(x) { 
				which(names(tmatch.out) == paste0(x, '.out')) })
		tmatch.out$id <- 1:nrow(tmatch.out)
		out <- melt(tmatch.out[,c(outcomes, which(names(tmatch.out) == 'id'))],id.vars='id')
		names(out) <- c('ID','Treatment','Outcome')
		out$ID <- as.factor(out$ID)
		results$friedman.test <- friedman.test(Outcome ~ Treatment | ID, out, ...)
		
		#Repeated measures ANOVA
		results$rmanova <- ezANOVA(data=out, dv=Outcome, wid=ID, 
								   within=Treatment, ...)
		
		if(results$rmanova$ANOVA$p <= p || results$friedman.test$p.value <= p) {
			#Possible approach for post-hoc test
			results$pairwise.wilcox.test <- pairwise.wilcox.test(
						x=out$Outcome, g=out$Treatment, paired=TRUE, 
						p.adjust.method='bonferroni', ...)
			
			# Paired t-tests
			diffcols <- c(
				paste(names(tmatch.out)[outcomes[c(1,2)]], collapse='-'),
				paste(names(tmatch.out)[outcomes[c(1,3)]], collapse='-'),
				paste(names(tmatch.out)[outcomes[c(2,3)]], collapse='-')
			)
			
			tmatch.out[, diffcols[1] ] <- tmatch.out[,outcomes[1]] - tmatch.out[,outcomes[2]]
			tmatch.out[, diffcols[2] ] <- tmatch.out[,outcomes[1]] - tmatch.out[,outcomes[3]]
			tmatch.out[, diffcols[3] ] <- tmatch.out[,outcomes[2]] - tmatch.out[,outcomes[3]]
			
			#Individual t-tests
			t1 <- t.test(x=tmatch.out[,outcomes[1]], y=tmatch.out[,outcomes[2]], paired=TRUE)
			t2 <- t.test(x=tmatch.out[,outcomes[1]], y=tmatch.out[,outcomes[3]], paired=TRUE)
			t3 <- t.test(x=tmatch.out[,outcomes[2]], y=tmatch.out[,outcomes[3]], paired=TRUE)
			
			ci <- data.frame(row.names=1:3)
			ci$Treatments <- diffcols
			ci$t <- c(t1$statistic, t2$statistic, t3$statistic)
			ci$df <- c(t1$parameter, t2$parameter, t3$parameter)
			ci$p.value <- c(t1$p.value, t2$p.value, t3$p.value)
			ci$sig <- star(ci$p.value)
			ci$mean.diff <- c(t1$estimate, t2$estimate, t3$estimate)
			ci$ci.min <- c(t1$conf.int[1], t2$conf.int[1], t3$conf.int[1])
			ci$ci.max <- c(t1$conf.int[2], t2$conf.int[2], t3$conf.int[2])
			
			results$t.tests <- ci
		}		
	}
	
	class(results) <- c("trimatch.summary", "list")
	return(results)
}

#' Prints the results of [summary.triangle.matches()].
#' 
#' This is an S3 generic function to print the results of 
#' [summary.triangle.matches()].
#' 
#' @param x results of [summary.triangle.matches()].
#' @param ... multiple results of [summary.triangle.matches()]. These
#'        must be named. For example, \code{"Method 1" = summary(tmath, outcome)}.
#' @method print trimatch.summary
#' @export
print.trimatch.summary <- function(x, ...) {
	parms <- list(...)
	if(!missing(x)) {
		NextMethod(x) #TODO: Provide a better print method.
	} else {
		df <- data.frame(
			Method=character(),
			Friedman.chi2=numeric(), 
			Friedman.p=numeric(), 
			Friedman.sig=character(),
			rmANOVA.F=numeric(), 
			rmANOVA.p=numeric(), 
			rmANOVA.sig=character(),
			stringsAsFactors=FALSE)
		for(i in seq_along(parms)) {
			df <- rbind(df, data.frame(
				Method = names(parms)[i],
				Friedman.chi2 = parms[[i]]$friedman.test$statistic,
				Friedman.p = parms[[i]]$friedman.test$p.value,
				Friedman.sig = star(parms[[i]]$friedman.test$p.value),
				rmANOVA.F = parms[[i]]$rmanova[[1]]$F,
				rmANOVA.p = parms[[i]]$rmanova[[1]]$p,
				rmANOVA.sig = star(parms[[i]]$rmanova[[1]]$p),
				stringsAsFactors=FALSE))
		}
		names(df)[7] <- ' '
		names(df)[4] <- ' '
		row.names(df) <- 1:nrow(df)
		class(df) <- c('tritable','data.frame')
		return(df)
	}
}

#' Prints the summary results of the logistic regression models.
#' 
#' The [trips()] function estimates three separate logistic regression
#' models for each pair of groups. This function will print a combined table
#' of the three summaries.
#' 
#' @param object the results of [trips()].
#' @param ... currently unused.
#' @method summary triangle.psa
#' @export
summary.triangle.psa <- function(object, ...) {
	m1 <- attr(object, 'model1')
	m2 <- attr(object, 'model2')
	m3 <- attr(object, 'model3')
	
	m1sum <- summary(m1)
	m2sum <- summary(m2)
	m3sum <- summary(m3)
	
	#ls(m1sum)
	return(cbind(as.data.frame(m1sum$coefficients),
		  'm1'=star(m1sum$coefficients[,4]),
		  as.data.frame(m2sum$coefficients),
		  'm2'=star(m2sum$coefficients[,4]),
		  as.data.frame(m3sum$coefficients), 
		  'm3'=star(m3sum$coefficients[,4])))
}

# star <- function(pvals) {
# 	ifelse(pvals < .001, '***', 
# 		   ifelse(pvals < .01, '**',
# 		   	   ifelse(pvals < .05, '*',
# 		   	   	   ifelse(pvals < .1, '.', ''))))
# }

#' Internal method for plotting.
#' Position along the left side segment
#' 
#' @param d the distance
#' @export
segment1 <- function(d) {
	x <- cos((60 * pi) / 180) * d
	y <- x * (sqrt(.75) / .5)
	return(c(x=x, y=y))
}

#' Internal method for plotting.
#' Position along the right side segment
#' 
#' @param d the distance
#' @export
segment2 <- function(d) {
	x <- 1 - (cos((60 * pi) / 180) * d)
	y <- -(sqrt(.75) / .5) * x + sqrt(.75) / .5
	return(c(x=x, y=y))
}

#' Internal method for plotting.
#' Finds a point d distance from x, y
#' 
#' @param x x coordinate
#' @param y y coordinate
#' @param d the distance
#' @export
perpPt <- function(x, y, d=.05) {
	x2 <- cos( (30*pi) / 180) * d
	y2 <- sin( (30*pi) / 180) * d
	return(c(x=x2, y=y2))
}

---
title: "Propensity Score Matching with Three Groups"
shorttitle: "TriMatch"
author: 
  - name          : "Jason Bryer"
    corresponding : yes    # Define only one corresponding author
    address       : "119 W 31st St, New York, NY 10001"
    email         : "jason.bryer@cuny.edu"
date: "`r Sys.Date()`"
abstract: "The use of propensity score methods [@RosenbaumRubin1983] have become popular for estimating causal inferences in observational studies in medical research [@Austin2008] and in the social sciences [@ThoemmesKim2011]. In most cases however, the use of propensity score methods have been confined to a single treatment. Several researchers have suggested using propensity score methods with multiple control groups, or to simply perform two separate analyses, one between treatment one and the control and another between treatment two and control. This paper introduces the TriMatch package for R that provides a method for determining matched triplets. Examples from educational and medical contexts will be discussed."
# keywords: "propensity score analysis, causality, matching"
bibliography: "references.bib"
# output:
#  bookdown::html_document2: default
output:
 bookdown::pdf_document2: default
pkgdown:
 as_is: false
fontsize: 11pt
geometry: = 2in
line-numbers: true
list-tables: true
list-figures: true
vignette: >
  %\VignetteIndexEntry{Propensity Score Matching with Three Groups}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::rmarkdown}
editor_options: 
  chunk_output_type: console
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  message = FALSE,
  warning = FALSE,
  error = FALSE
)
options(digits = 2)
```

```{r setup}
library(TriMatch)
data("tutoring")
data("nmes")
```

# Introduction

Consider two treatments, $Tr_1$ and $Tr_2$, and a control, $C$. We estimate propensity scores with three separate logistic regression models where model one predicts $Tr_1$ with $C$, model two predicts $Tr_2$ with $C$, and model three predicts $Tr_1$ with $Tr_2$. The triangle plot in Figure \@ref(fig:triangleplot) depicts the fitted values (i.e. propensity scores) from the three models on each edge of the triangle. Since each unit has a propensity score in two of the three models, their scores are connected. We can then calculate three distances between propensity scores for each possible matched triplet using the three models. Given those distances, the matched triplets with the smallest standardized distance (i.e. $D_{x,y} = \frac{| PS_{x} - PS_{y} |}{sd(PS)}$) are retained. Several methods for determining which matched triplets to retain are provided with the possibility of the researcher to implement their own. The black lines in Figure \@ref(fig:triangleplot) represent one matched triplet (i.e. one row in the returned data frame).

Propensity score analysis of two groups typically use dependent sample *t*-tests [@Austin2010]. The analogue for matched triplets include repeated measures ANOVA and the Freidman Rank Sum Test. The `TriMatch` package provides utility functions for conducting and visualizing these statistical tests. Moreover, a set of functions extending `PSAgraphics` [@HelmreichPruzek2008] for matched triplets to check covariate balance are provided.

## The `TriMatch` Algorithm

The `trips` and `trimatch` functions are used to estimate the propensity scores and find the best matched triplets, respectively. 


A. Propensity scores are estimated for three models using logistic regression.
$${ PS }_{ 1 }=e({ x }_{ { T }_{ 1 }C })=Pr(z=1|{ X }_{ { T }_{ 1 }C })$$
$${ PS }_{ 2 }=e({ x }_{ { T }_{ 2 }C })=Pr(z=1|{ X}_{ { T }_{ 2 }C })$$
$${ PS }_{ 3 }=e({ x }_{ { T }_{ 2 }{ T }_{ 1 } })=Pr(z=1|{ X }_{ { T }_{ 2 }{ T }_{ 1 } })$$

B. Match order is determined. The default is to start with the larger of the two treatments, followed the second treatment, and lastly the control group. However, the match order is configurable vis-a-vis the `match.order` parameter.

C. Three distance matrices are calculated, $D_1$, $D_2$, and $D_3$ corresponding to the propensity scores estimated in step \@ref(item:ps). That is, $D_1$ is a $n_{Tr_1}$ x $n_{Tr_2}$  matrix where $D_1[x,y]$ is the standardized distance between $PS_1[x]$ and $PS_1[y]$.

D. Distances greater than the caliper, 0.25 by default as recommended by [@RosenbaumRubin1985], are eliminated. The caliper is specified in standard units so 0.25 corresponds to one-quarter of one standard deviation.

E. If partial exact matching is desired, three logical matrices are created with the same dimensions as the distance matrices calculated in step \@ref(item:distance). That is, position $x,y$ in the matrix is true if the covariate(s) to match exactly on between unit $x$ and $y$ match exactly. Distances where exact there are not exact matches are eliminated.

F. For the remaining units, all possible combinations of matched triplets are formed and a total standardized distance is calculated.


The result of the above procedure is the equivalent of caliper matching in the two group case. That is, all possible matches within a specified caliper are retained. This can be achieved by specifying `method = NULL` parameter to the `trimatch` function. Two additional methods are provided to reduce the number of matched triplets. The `maximumTreat` method attempts to reduce the number of duplicate treatment units. This is analogous to matching without replacement in the two group case. However, treatment 1 units may be matched to two different treatment 2 units if that treatment 2 unit would otherwise not be matched. The `OneToN` method will allow the user to specify exactly how many times each treatment 1 and treatment 2 may be reused. 

# Effects of Tutoring on Course Grades

In the first example we will utilize observational data obtained to evaluate the effectiveness of tutoring services on course grades. Treatment students consisted of those students who used tutoring services while enrolled in a online writing course between 2008 and 2011. A comparison group was identified as students enrolled in a course section with a student who used tutoring services. The treatment group was then divided into two based upon the number of times they utilized tutoring services. "Novice" users are those who used the services once and "regular" users are those who used services two or more times. Covariates available for estimating propensity scores are gender, ethnicity, military status, English second language learner, educational level for mother and father, age at the beginning of the course, employment level at college enrollment, income level at college enrollment, number of transfer credits, and GPA at the start of the course.

```{r}
names(tutoring)
```

The courses represented here are structured such that the variation from section-to-section is minimal. However, the differences between courses is substantial and therefore we will utilize partial exact matching so that all matched students will have taken the same course.

```{r}
table(tutoring$treat, tutoring$Course, useNA="ifany")
```

The first step of analysis is to estimate the propensity scores. The `trips` function will estimate three propensity score models, $PS_1$, $PS_2$, and $PS_3$ as described above. Note that when specifying the formula the dependent variable, or treatment indicator, is not included. The `trips` function will replace the dependent variable as it estimates the three logistic regression models.

```{r}
formu <- ~ Gender + Ethnicity + Military + ESL + EdMother + EdFather + 
Age + Employment + Income + Transfer + GPA
tutoring.tpsa <- trips(tutoring, tutoring$treat, formu)
```

Figure \@ref(fig:triangleplot) is a triangle plot that depicts the propensity scores from the three models. Since each student has two propensity scores, their scores are connected with a line. The black line in Figure \@ref(fig:triangleplot) represents one matched triplet estimated below.

```{r}
plot(tutoring.tpsa)
```

The default for `trimatch` is to use the `maximumTreat` method retaining each treatment unit once with treatment one units matched more than once only if the corresponding treatment two unit would not be matched otherwise.

```{r}
tutoring.matched <- trimatch(tutoring.tpsa, exact=tutoring[,c("Course")])
```

Setting the `method` parameter to `NULL` will result in caliper matching. All matched triplets within the specified caliper are retained. This will result in the largest number of matched triplets.

```{r}
tutoring.matched.caliper <- trimatch(tutoring.tpsa, 
exact=tutoring[,c("Course")], method=NULL)
```

Lastly, we will use the `OneToN` method to retain a 2-to-1-to-n and 3-to-2-n matches.

```{r}
tutoring.matched.2to1 <- trimatch(tutoring.tpsa, 
exact=tutoring[,c("Course")], method=OneToN, M1=2, M2=1)
tutoring.matched.3to2 <- trimatch(tutoring.tpsa, 
exact=tutoring[,c("Course")], 
method=OneToN, M1=3, M2=2)
```

```{r triangleplot, fig.cap = 'Traingle Plot'}
print(plot(tutoring.matched, rows=c(50), line.alpha=1, draw.segments=TRUE))
```

## Examining Unmatched Students

The different methods for retaining matched triplets address the issue of overrepresentation of treatment units. In this example there four times as many control units as treatment units (the ratio is larger when considering the treatments separately). These methods fall on a spectrum where each treatment unit is used minimally (`maximumTreat` method) or all units are used (caliper matching). [@Rosenbaum2012] suggests testing hypothesis more than once and it is our general recommendation to utilize multiple methods. Functions to help present and compare the results from multiple methods are provided and discussed below.

The `unmatched` function will return the rows of students who were not matched. The `summary` function will provide information about how many students within each group were not matched. As shown below, the caliper matching will match the most students. In this particular example, in fact, the only substantial difference in the unmatched students is with the control group. All methods fail to match 37 treatment one students. This is due to the fact that there is not another student within the specified caliper that match exactly on the course.

```{r}
summary(unmatched(tutoring.matched))
```

```{r}
summary(unmatched(tutoring.matched.caliper))
```

```{r}
summary(unmatched(tutoring.matched.2to1))
```

```{r}
summary(unmatched(tutoring.matched.3to2))
```


## Checking Balance

The eventual strength of propensity score methods is dependent on how well balance is achieved. [@HelmreichPruzek2008] introduced graphical approaches to evaluating balance. We provide functions that extend that framework to matching of three groups. Figure \@ref(fig:multibalance) is a multiple covariate balance plot that plots the absolute effect size of each covariate before and after adjustment. In this example, the figure suggests that reasonable balance has been achieved across all covariates and across all three models since effect sizes are smaller than the unadjusted in most cases and relatively small.

```{r multibalance, fig.cap = 'Multiple Covariate Balance Plot of Absolute Standardized Effect Sizes Before and After Propensity Score Adjustment'}
print(multibalance.plot(tutoring.tpsa) + ggtitle("Covariate Balance Plot"))
```

Figure \@ref(fig:balance) is the results of the `balance.plot` function. This function will provide a bar chart for categorical covariates and box plots for quantitative covariates, individually or in a grid.

```{r balance, fig.cap = 'Covariate Balance Plots', cache = TRUE}
bplots <- balance.plot(tutoring.matched, tutoring[,all.vars(formu)], 
		legend.position="none", x.axis.labels=c("C","T1","T1"), x.axis.angle=0)
print(plot(bplots, cols=3, byrow=FALSE))
```


## Phase II: Estimating Effects of Tutoring on Course Grades

In phase two of propensity score analysis we wish to compare our outcome of interest, course grade in this example, across the matches. A custom `merge` function is provided to merge an outcome from the original data frame to the results of `trimatch`. This merge function will add three columns with the outcome for each of the three groups.

```{r}
matched.out <- merge(tutoring.matched, tutoring$Grade)
names(matched.out)
head(matched.out)
```

Although the `merge` function is convenient for conducting your own analysis, the `summary` function will perform the most common analyses including Friedman Rank Sum test and repeated measures ANOVA. If either of those tests produce a *p* value less than the specified threshold (0.05 by default), then the `summary` function will also perform and return Wilcoxon signed rank test and three separate dependent sample *t*-tests see [@Austin2010] for discussion of dependent versus independent *t*-tests.

```{r}
s1 <- summary(tutoring.matched, tutoring$Grade)
names(s1)
s1$friedman.test
s1$t.tests
```

The `print` method will accept multiple object returned by `summary` so to combine them into a single table output. Note that each parameter must be named and that name will be used to identify the row containing those results.

```{r}
s2 <- summary(tutoring.matched.caliper, tutoring$Grade)
s3 <- summary(tutoring.matched.2to1, tutoring$Grade)
s4 <- summary(tutoring.matched.3to2, tutoring$Grade)

print("Max Treat"=s1, "Caliper"=s2, "2-to-1"=s3, "3-to-2"=s4)
```

```{r boxplots, fig.cap = 'Boxplot of Differences', fig.show = 'hold', out.width = '30%'}
boxdiff.plot(tutoring.matched, tutoring$Grade, 
			 ordering=c("Treat2","Treat1","Control")) + 
	ggtitle("Maximum Treatment Matching")
boxdiff.plot(tutoring.matched.caliper, tutoring$Grade, 
			 ordering=c("Treat2","Treat1","Control")) +
	ggtitle("Caliper Matching")
boxdiff.plot(tutoring.matched.2to1, tutoring$Grade, 
			 ordering=c("Treat2","Treat1","Control")) +
	ggtitle("2-to-1-to-n Matching")
```


Another useful visualization for presenting the results is the Loess plot. In Figure \@ref(fig:loess) we plot the propensity scores on the *x*-axis and the outcome (grade in this example) on the *y*-axis. A Loess regression line is then overlaid.^[We utilize the `geom_smooth` geometry in the `ggplot2` package that provides other smoothing functions including linear modeling (`lm`), generalized linear modeling (`glm`), and robust generalized additive models (`gam`). See the documentation for the `stat_smooth` function in `ggplot2`.] Since there are three propensity score scales, the `plot.loess3` function will use the propensity scores from the model predicting treatment one from treatment two. Propensity scores for the control group are then imputed by taking the mean of the propensity scores of the two treatment units that control was matched to. It should be noted that if a control unit is matched to two different sets of treatment units, then that control unit will have two propensity scores. Which propensity score scale is utilized can be explicitly specified using the `model` parameter.


```{r loess, fig.cap = 'Loess Plot for Caliper Matching'}
loess3.plot(tutoring.matched.caliper, tutoring$Grade, ylab="Grade", 
			points.alpha=.1, method="loess")
```

# Effects of Smoking on Medical Expenditures

In this example^[This example is included as a demo in the package. Type `demo(nmes)` in R to start the demo.]  we will utilize the National Medical Expenditure Study [@nmes] to estimate the effects of smoking on medical expenditures. This dataset was first used by [@Johnson2003] to estimate the effects of smoking on diseases, and then the effect of diseases on medical expenditures. [@ImaiVanDyk2004] developed an a method to generalize the propensity score, called a p-score, to directly estimate the effects of smoking on medical expenditures. More specifically, they defined a quantitative treatment variable, pack year, defined as:

$$packyear = \frac{\text{number of cigarettes per day}}{20} \times \text{number of years smoked}$$

Our approach is designed to match three separate groups and not a continuous treatment. We will address two research questions: (1) What are the effects of smoking status (i.e. never smoked, former smoker, and current smoker) on medical expenditures? and (2) What are the effects of lifetime smoking on medical expenditures? Figure \@ref(fig:packyearsAndTotalExp) represent the relationship between these two different treatments^[Note that the control group in both instances are people who never smoked and is omitted from this figure.]. This figure reveals several, perhaps counterintuitive, facts. First, the unadjusted total medical expenditures for former smokers is higher than current smokers. Secondly, the distribution of $log(packyear)$ overlap substantial between former and current smokers. To dichotomize the pack year smoking variable, we will split on the median of pack year, labeled moderate smokers (i.e. $packyear \le median(packyear)$) and heavy smokers (i.e. $packyear > median(packyear)$).

```{r}
data(nmes)
nmes <- subset(nmes, select = c(packyears, smoke, LASTAGE, MALE, RACE3, beltuse, educate, marital, SREGION, POVSTALB, HSQACCWT, TOTALEXP))
```

Both [@Johnson2003] and [@ImaiVanDyk2004] conducted a complete-case analysis and Johnson et al. reported that multiple imputation did not substantially affect their results.

```{r}
nmes <- na.omit(nmes)
```

Since many participants had zero medical expenditures, we will add one to the total expenditures before log transforming the variable. We will then calculate the median of pack year and create a new treatment variable, `smoke2`, for moderate and heavy smokers with non-smokers.

```{r}
nmes$smoke <- factor(nmes$smoke, levels=c(0,1,2), labels=c("Never","Smoker","Former"))
nmes$LogTotalExp <- log(nmes$TOTALEXP + 1)
(medPY <- median(nmes[nmes$smoke != "Never",]$packyears))
table(nmes$smoke, nmes$packyears > medPY)
nmes$smoke2 <- ifelse(nmes$smoke == "Never", "Never", 
ifelse(nmes$packyears > 17, "Heavy", "Moderate"))
table(nmes$smoke, nmes$smoke2, useNA="ifany")
```



```{r packyearsAndTotalExp, fig.cap = 'Relationship Between Pack Year and Total Expenditures by Current Smoking Status', fig.show = 'hold', out.width = '50%'}
ggplot(nmes[nmes$smoke != "Never",], aes(x=log(packyears+1), color=smoke, fill=smoke)) +
	  	geom_density(alpha=.1) + 
	  	theme(legend.position="none", plot.margin=rep(unit(0, "cm"), 4)) +
	  	xlab("") + ylab("Density")
ggplot(nmes[nmes$smoke != "Never",], aes(x=log(packyears+1), y=LogTotalExp, color=smoke, fill=smoke)) + 
	  	geom_point(alpha=.2) + 
	  	geom_smooth(method="loess", formula = y ~ x) +
	  	scale_color_hue("") + scale_fill_hue("") +
	  	theme(legend.position=c(.9,1), plot.margin=rep(unit(0, "cm"), 4)) + 
	  	xlab("log(Pack Year)") + ylab("log(Total Expenditures)")
```

Imai and van Dyk observed that there appeared to be a relationship between age and medical expenditures. We will create a new categorical age variable using quintiles to use for partial exact matching. This serves two purposes, first it ensures balance on this critical covariate (note that we will also exactly match on gender and ethnicity) and two, decrease the search space for matched triplets therefore increasing the efficiency of the matching algorithm. The possible disadvantage of exact matching is that too many treated units will not be matched. We will examine unmatched treatment units below.

```{r}
nmes$LastAge5 <- cut(nmes$LASTAGE, 
breaks=quantile(nmes$LASTAGE, probs=seq(0,1,1/5)),
include.lowest=TRUE, orderd_result=TRUE)
```

Define our model to estimate the propensity scores.

```{r}
formu <- ~ LASTAGE + MALE + RACE3 + beltuse + educate + marital + 
SREGION + POVSTALB
```

Estimate propensity scores for our two different treatments. Figure \@ref(fig:nmestriangleplot) provides triangle plots for both models.

```{r}
tpsa.smoke <- trips(nmes, nmes$smoke, formu)
tpsa.packyears <- trips(nmes, nmes$smoke2, formu)
```

```{r nmestriangleplot, fig.cap = 'Triangle Plots for NMES', fig.show = 'hold', out.width = '50%'}
p.smoke <- plot(tpsa.smoke, sample=c(.05), edge.alpha=.1) + ggtitle("Treatment Variable: Current Smoking Status")
p.packyears <- plot(tpsa.packyears, sample=c(.05), edge.alpha=.1) + ggtitle("Treatment Variable: Lifetime Smoking Frequency")
p.smoke
p.packyears
```

Create two sets of matched triplets for our two treatments.

```{r, cache = TRUE}
tmatch.smoke <- trimatch(tpsa.smoke, 
exact=nmes[,c("LastAge5","MALE","RACE3")])
tmatch.packyears <- trimatch(tpsa.packyears, 
exact=nmes[,c("LastAge5","MALE","RACE3")])
```

The following summary of the unmatched rows show that more than 96% of the treatment units were matched in both models.

```{r}
summary(unmatched(tmatch.smoke))
summary(unmatched(tmatch.packyears))
```

Figure \@ref(fig:nmesbalance) is a multiple covariate balance plot for the two treatments. It shows that the absolute effect sizes after adjustment is better for all covariates. The demo included in the `TriMatch` package provides functions to create individual balance plots for each covaraite.

```{r nmesbalance, fig.cap = 'Multiple Covariate Balance Plots for NMES', fig.show = 'hold', out.width = '50%'}
p.smoke <- multibalance.plot(tpsa.smoke) + ggtitle("Treatment Variable: Current Smoking Status")
p.packyears <- multibalance.plot(tpsa.packyears) + ggtitle("Treatment Variable: Lifetime Smoking Frequency")
p.smoke
p.packyears
```

## Phase II: Estimating Effects of Smoking on Medical Expenditures

For both treatment regimes we used the `maximumTreat` method for finding matched triplets that will retain each treatment unit once with the possibility of using treatment units twice in cases where a treatment unit would not otherwise be matched. The Friedman Rank Sum Test and repeated measures ANOVA indicate there a statistically significant difference in both treatment regimes. Figure \@ref(fig:nmesboxplots) provides box plots of the differences for the two treatment regimes. For the current smoking status treatment, the results indicate that smoker's actually spend less than former and non-smokers. However, as [@ImaiVanDyk2004] explain, the sample of smokers includes only survivors and should be considered when interpreting these results. 

Imai and van Dyk's analysis used pack year as treatment indicator. Our dichotomizing of pack year into moderate and heavy smokers more closely adheres to their approach. The results with this treatment regime indicate that smokers, both moderate and heavy, have higher medical expenditures than non-smokers. However, there is no statistically significant difference between heavy and moderate smokers in medical expenditures. 

```{r nmesboxplots, fig.cap = 'Boxplot of Differences for NMES', fig.show = 'hold', out.width = '50%'}
boxdiff.plot(tmatch.smoke, nmes$LogTotalExp, ordering=c("Smoker","Former","Never")) + 
	ggtitle("Treatment Variable: Current Smoking Status")
boxdiff.plot(tmatch.packyears, nmes$LogTotalExp, ordering=c("Heavy","Moderate","Never")) +
	ggtitle("Treatment Variable: Lifetime Smoking Frequency")
```

```{r, eval=FALSE}
sum.smoke <- summary(tmatch.smoke, nmes$LogTotalExp, ordering=c("Smoker","Former","Never"))
sum.packyears <- summary(tmatch.packyears, nmes$LogTotalExp, ordering=c("Heavy","Moderate","Never"))
print("Current Smoking Status" = sum.smoke, "Smoking Frequency" = sum.packyears)
```

```{r, eval=FALSE}
sum.smoke$t.tests
sum.packyears$t.test
```

# References

<div id="refs" custom-style="Bibliography"></div>

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.balance.R
\name{balance.plot}
\alias{balance.plot}
\title{Balance plot for the given covariate.}
\usage{
balance.plot(
  x,
  covar,
  model,
  nstrata = attr(attr(tmatch, "triangle.psa"), "nstrata"),
  label = "Covariate",
  ylab = "",
  xlab = NULL,
  se.ratio = 2,
  print = TRUE,
  legend.position = "top",
  x.axis.labels,
  x.axis.angle = -45,
  ...
)
}
\arguments{
\item{x}{results from [trimatch()].}

\item{covar}{vector of the covariate to check balance of.}

\item{model}{an integer between 1 and 3 indicating from which model the 
propensity scores will be used.}

\item{nstrata}{number of strata to use.}

\item{label}{label for the legend.}

\item{ylab}{label of the y-axis.}

\item{xlab}{label of the x-axis.}

\item{se.ratio}{a multiplier for how large standard error bars will be.}

\item{print}{print the output if the Freidman Rank Sum Test and repeated
measures ANOVA (for continuous variables).}

\item{legend.position}{the position of the legend.}

\item{x.axis.labels}{labels for the x-axis.}

\item{x.axis.angle}{angle for x-axis labels.}

\item{...}{parameters passed to [plot.balance.plots()].}
}
\value{
a \code{ggplot2} figure or a list of \code{ggplot2} figures if \code{covar}
       is a data frame.
}
\description{
If the covariate is numeric, boxplots will be drawn with red points for the mean
and green error bars for the standard error. For non-numeric covariates a barplot
will be drawn.
}
\details{
A Friedman rank sum test will be performed for all covariate types, printed,
and stored as an attribute to the returned object named \code{friedman}. If
a continuous covariate a repeated measures ANOVA will also be performed, printed,
and returned as an attribute named \code{rmanova}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/triangle.match.R
\name{distance.euclid}
\alias{distance.euclid}
\title{Euclidean distance calculation.}
\usage{
distance.euclid(x, grouping, id, groups, caliper, nmatch = Inf)
}
\arguments{
\item{x}{vector of propensity scores.}

\item{grouping}{vector or factor identifying group membership.}

\item{id}{vector corresponding to unique identifer for each element in 
\code{x} and \code{grouping}.}

\item{groups}{vector of length two indicating the unique groups to calculate
the distance between. The first element will be the rows, the second columns.}

\item{caliper}{a scaler indicating the caliper to use for matching within
each step.}

\item{nmatch}{number of smallest distances to retain.}
}
\value{
a list of length equal to \code{x}. Each element of the list is a
       named numeric vector where the values correspond to the distance and the
       name to the \code{id}.
}
\description{
This method uses a simple Euclidean distance calculation for determining the
distances between two matches. That is, |ps1 - ps2|.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.triangle.matches.R
\name{print.trimatch.summary}
\alias{print.trimatch.summary}
\title{Prints the results of [summary.triangle.matches()].}
\usage{
\method{print}{trimatch.summary}(x, ...)
}
\arguments{
\item{x}{results of [summary.triangle.matches()].}

\item{...}{multiple results of [summary.triangle.matches()]. These
must be named. For example, \code{"Method 1" = summary(tmath, outcome)}.}
}
\description{
This is an S3 generic function to print the results of 
[summary.triangle.matches()].
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TriMatch-package.R
\docType{package}
\name{TriMatch-package}
\alias{TriMatch-package}
\alias{TriMatch}
\title{Propensity Score Analysis for Non-Binary Treatments}
\description{
This packages provides functions to estimate and visualize propensity
score analyses including matching for non-binary treatments.
}
\seealso{
\code{PSAgraphics} \code{multilevelPSA}
}
\author{
Jason Bryer \email{jason@bryer.org}
}
\keyword{analysis}
\keyword{matching}
\keyword{propensity}
\keyword{psa}
\keyword{score}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/triangle.match.R
\name{maximumTreat}
\alias{maximumTreat}
\title{This method will return at least one treatment from groups one and two within
the caliper.}
\usage{
maximumTreat(tmatch, ...)
}
\arguments{
\item{tmatch}{initial results from [trimatch()] that contains all
possible matches within the specified caliper.}

\item{...}{currently unused.}
}
\description{
This method will attempt to return enough rows to use each treatment (the first
two groups in the matching order) at least once. Assuming treat1 is the first
group in the match order and treat2 the second, all duplicate treat1 rows
are removed. Next, all treat2 units not in present in after removing duplicate
treat1 units are identified. For each of those treat2 units, the matched
triplet with the smallest overall distances where treat2 is one of the mathched
units is retained.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.utils.R
\name{segment2}
\alias{segment2}
\title{Internal method for plotting.
Position along the right side segment}
\usage{
segment2(d)
}
\arguments{
\item{d}{the distance}
}
\description{
Internal method for plotting.
Position along the right side segment
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ezANOVA.R
\name{ezANOVA}
\alias{ezANOVA}
\title{Compute ANOVA}
\usage{
ezANOVA(
  data,
  dv,
  wid,
  within = NULL,
  within_full = NULL,
  within_covariates = NULL,
  between = NULL,
  between_covariates = NULL,
  observed = NULL,
  diff = NULL,
  reverse_diff = FALSE,
  type = 1,
  white.adjust = FALSE,
  detailed = FALSE,
  return_aov = FALSE
)
}
\arguments{
\item{data}{Data frame containing the data to be analyzed.}

\item{dv}{Name of the column in data that contains the dependent variable. Values in this column 
must be numeric.}

\item{wid}{Name of the column in data that contains the variable specifying the case/Ss 
identifier. This should be a unique value per case/Ss.}

\item{within}{Names of columns in data that contain predictor variables that are manipulated 
(or observed) within-Ss. If a single value, may be specified by name alone; if multiple values, 
must be specified as a .() list.}

\item{within_full}{Same as within, but intended to specify the full within-Ss design in cases 
where the data have not already been collapsed to means per condition specified by within and 
when within only specifies a subset of the full design.}

\item{within_covariates}{Names of columns in data that contain predictor variables that are 
manipulated (or observed) within-Ss and are to serve as covariates in the analysis. If a single 
value, may be specified by name alone; if multiple values, must be specified as a .() list.}

\item{between}{Names of columns in data that contain predictor variables that are manipulated 
(or observed) between-Ss. If a single value, may be specified by name alone; if multiple values,
 must be specified as a .() list.}

\item{between_covariates}{Names of columns in data that contain predictor variables that are 
manipulated (or observed) between-Ss and are to serve as covariates in the analysis. If a single 
value, may be specified by name alone; if multiple values, must be specified as a .() list.}

\item{observed}{Names of columns in data that are already specified in either within or between 
that contain predictor variables that are observed variables (i.e. not manipulated). If a single 
value, may be specified by name alone; if multiple values, must be specified as a .() list. The 
presence of observed variables affects the computation of the generalized eta-squared measure of 
effect size reported by ezANOVA.}

\item{diff}{Names of any variables to collapse to a difference score. If a single value, may be 
specified by name alone; if multiple values, must be specified as a .() list. All supplied 
variables must be factors, ideally with only two levels (especially if setting the reverse_diff 
argument to TRUE).}

\item{reverse_diff}{Logical. If TRUE, triggers reversal of the difference collapse requested by 
diff. Take care with variables with more than 2 levels.}

\item{type}{Numeric value (either 1, 2 or 3) specifying the Sums of Squares "type" to employ 
when data are unbalanced (eg. when group sizes differ). type = 2 is the default because this will 
yield identical ANOVA results as type = 1 when data are balanced but type = 2 will additionally 
yield various assumption tests where appropriate. When data are unbalanced, users are warned that 
they should give special consideration to the value of type. type=3 will emulate the approach 
taken by popular commercial statistics packages like SAS and SPSS, but users are warned that this 
approach is not without criticism.}

\item{white.adjust}{Only affects behavior if the design contains only between-Ss predictor 
variables. If not FALSE, the value is passed as the white.adjust argument to Anova, which 
provides heteroscedasticity correction. See Anova for details on possible values.}

\item{detailed}{Logical. If TRUE, returns extra information (sums of squares columns, intercept 
row, etc.) in the ANOVA table.}

\item{return_aov}{Logical. If TRUE, computes and returns an aov object corresponding to the 
requested ANOVA (useful for computing post-hoc contrasts).}
}
\value{
A list containing one or more of the following components:

\describe{
\item{ANOVA}{A data frame containing the ANOVA results.}
\item{Mauchly's Test for Sphericity}{If any within-Ss variables with >2 levels are present, a data frame containing the results of Mauchly's test for Sphericity. Only reported for effects >2 levels because sphericity necessarily holds for effects with only 2 levels.}
\item{Sphericity Corrections}{If any within-Ss variables are present, a data frame containing the Greenhouse-Geisser and Huynh-Feldt epsilon values, and corresponding corrected p-values.}
\item{Levene's Test for Homogeneity}{If the design is purely between-Ss, a data frame containing the results of Levene's test for Homogeneity of variance. Note that Huynh-Feldt corrected p-values where the Huynh-Feldt epsilon >1 will use 1 as the correction epsilon.}
\item{aov}{An aov object corresponding to the requested ANOVA.}
}

Some column names in the output data frames are abbreviated to conserve space:

\describe{
\item{DFn}{Degrees of Freedom in the numerator (a.k.a. DFeffect).}
\item{DFd}{Degrees of Freedom in the denominator (a.k.a. DFerror).}
\item{SSn}{Sum of Squares in the numerator (a.k.a. SSeffect).}
\item{SSd}{Sum of Squares in the denominator (a.k.a. SSerror).}
\item{F}{F-value.}
\item{p}{p-value (probability of the data given the null hypothesis).}
\item{p<.05}{Highlights p-values less than the traditional alpha level of .05.}
\item{ges}{Generalized Eta-Squared measure of effect size (see in references below: Bakeman, 2005).}
\item{GGe}{Greenhouse-Geisser epsilon.}
\item{p[GGe]}{p-value after correction using Greenhouse-Geisser epsilon.}
\item{p[GGe]<.05}{Highlights p-values (after correction using Greenhouse-Geisser epsilon) less than the traditional alpha level of .05.}
\item{HFe}{Huynh-Feldt epsilon.}
\item{p[HFe]}{p-value after correction using Huynh-Feldt epsilon.}
\item{p[HFe]<.05}{Highlights p-values (after correction using Huynh-Feldt epsilon) less than the traditional alpha level of .05.}
\item{W}{Mauchly's W statistic}
}
}
\description{
*Note that this adapted from the `ez` R package available here: https://github.com/mike-lawrence/ez
It appears the `ez` package is no longer maintained so the function was copied here.*
}
\details{
This function provides easy analysis of data from factorial experiments, including purely 
within-Ss designs (a.k.a. “repeated measures”), purely between-Ss designs, and mixed 
within-and-between-Ss designs, yielding ANOVA results, generalized effect sizes and 
assumption checks.



ANCOVA is implemented by first regressing the DV against each covariate (after collapsing the 
data to the means of that covariate's levels per subject) and subtracting from the raw data the 
fitted values from this regression (then adding back the mean to maintain scale). These 
regressions are computed across Ss in the case of between-Ss covariates and computed within 
each Ss in the case of within-Ss covariates.

**Warning**

Prior to running (though after obtaining running ANCOVA regressions as described in the details 
section), dv is collapsed to a mean for each cell defined by the combination of wid and any 
variables supplied to within and/or between and/or diff. Users are warned that while convenient 
when used properly, this automatic collapsing can lead to inconsistencies if the pre-collapsed 
data are unbalanced (with respect to cells in the full design) and only the partial design is 
supplied to ezANOVA. When this is the case, use within_full to specify the full design to ensure 
proper automatic collapsing.
}
\references{
Bakeman, R. (2005). Recommended effect size statistics for repeated measures designs. 
Behavior Research Methods, 37 (3), 379-384.
}
\author{
Mike Lawrence
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.multibalance.R
\name{multibalance.plot}
\alias{multibalance.plot}
\title{Multiple covariate balance assessment plot.}
\usage{
multibalance.plot(tpsa, tmatch, grid = TRUE, cols)
}
\arguments{
\item{tpsa}{results of [trips()].}

\item{tmatch}{results of [trimatch()].}

\item{grid}{if TRUE, then a grid of three plots for each model will be displayed.}

\item{cols}{character vector of covariates (i.e. column names) from the original 
data to include in the plot. By default all covariates used in the
logistic regression model are used.}
}
\value{
a \code{ggplot2} figure.
}
\description{
A graphic based upon [cv.bal.psa()] function in the `PSAgraphics`
package. This graphic plots the effect sizes for multiple covariates before and
after propensity score adjustment.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.parallel.R
\name{parallel.plot}
\alias{parallel.plot}
\title{Parallel coordinate plot for the three groups and dependent variable.}
\usage{
parallel.plot(tmatch, outcome)
}
\arguments{
\item{tmatch}{results from [trimatch()].}

\item{outcome}{vector of the outcome}
}
\description{
Creates a \code{ggplot2} figure of a parallel coordinate plot.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.utils.R
\name{perpPt}
\alias{perpPt}
\title{Internal method for plotting.
Finds a point d distance from x, y}
\usage{
perpPt(x, y, d = 0.05)
}
\arguments{
\item{x}{x coordinate}

\item{y}{y coordinate}

\item{d}{the distance}
}
\description{
Internal method for plotting.
Finds a point d distance from x, y
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/merge.triangle.matches.R
\name{merge.triangle.matches}
\alias{merge.triangle.matches}
\title{Merges outcomes with the matched set.}
\usage{
\method{merge}{triangle.matches}(x, y, ...)
}
\arguments{
\item{x}{the result of [trimatch()]}

\item{y}{another data frame or vector to merge with.}

\item{...}{unused}
}
\value{
\code{x} with the additional column(s) added.
}
\description{
The \code{y} parameter should be a subset of the original data used.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/merge.triangle.psa.R
\name{merge.triangle.psa}
\alias{merge.triangle.psa}
\title{Merges covariate(s) with the results of [trips()].}
\usage{
\method{merge}{triangle.psa}(x, y, ...)
}
\arguments{
\item{x}{the result of [trips()]}

\item{y}{another data frame or vector to merge with.}

\item{...}{unused}
}
\value{
\code{x} with the additional column(s) added.
}
\description{
The \code{y} parameter should be a subset of the original data used.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.balance.R
\name{plot.balance.plots}
\alias{plot.balance.plots}
\title{Prints a grid of balance plots.}
\usage{
\method{plot}{balance.plots}(x, rows, cols, byrow = TRUE, plot.sequence = seq_along(bplots), ...)
}
\arguments{
\item{x}{the results of [balance.plot()] when a data frame is specified.}

\item{rows}{if \code{covar} is a data frame of covariates, the number of
rows in the grid of figures.}

\item{cols}{if \code{covar} is a data frame of covariates, the number of
columns in the grid of figures.}

\item{byrow}{if TRUE (default), plots will be drawn by rows, otherwise by columns.}

\item{plot.sequence}{the sequence (or subset) of plots to draw.}

\item{...}{currently unused.}
}
\description{
Prints a grid of balance plots.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/triangle.match.R
\name{OneToN}
\alias{OneToN}
\title{This method will use a M1-to-M2-to-1 matching.}
\usage{
OneToN(tmatch, M1 = 2, M2 = 1, ...)
}
\arguments{
\item{tmatch}{initial results from [trimatch()] that contains all
possible matches within the specified caliper.}

\item{M1}{a scaler indicating the number of unique subjects in group one to
retain. This applies only to the first group in the matching order.}

\item{M2}{a scaler indicating the number of unique matches to retain. This applies
to the first two groups in the matching order.}

\item{...}{currently unused.}
}
\description{
In this method, \code{M2} corresponds to the number of times a treat1 unit can be
matched with a treat2 unit. The \code{M1} parameter corresponds to the number of
times a treat1 unit can be used in total.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/triangle.psa.R
\name{trips}
\alias{trips}
\title{Estimates propensity scores for three groups}
\usage{
trips(
  thedata,
  treat,
  formu = ~.,
  groups = unique(treat),
  nstrata = 5,
  method = "logistic",
  ...
)
}
\arguments{
\item{thedata}{the data frame.}

\item{treat}{vector or factor indicating the treatment/control assignment for
\code{thedata}. Length must be equal to \code{nrow(thedata)}.}

\item{formu}{the logistic regression formula. Note that the dependent variable
should not be specified and will be modified.}

\item{groups}{a vector of exactly length three corresponding the values in
\code{treat} for each control/treatment.}

\item{nstrata}{the number of strata marks to plot on the edge.}

\item{method}{the method to use to estimate the propensity scores. Current options are logistic
or randomForest.}

\item{...}{other parameters passed to [stats::glm()].}
}
\description{
The propensity score is \deqn{e(X)=P({ W }=1|X)}
This function will estimate the propensity scores for each pair of groups
(e.g. two treatments and one control).
}
\details{
\deqn{{ PS }_{ 1 }=e({ X }_{ { T }_{ 1 }C })=Pr(z=1|{ X }_{ { T }_{ 1 }C })}

\deqn{{ PS }_{ 2 }=e({ X }_{ { T }_{ 2 }C })=Pr(z=1|{ X }_{ { T }_{ 2 }C })}

\deqn{{ PS }_{ 3 }=e({ X }_{ { T }_{ 2 }{ T }_{ 1 } })=Pr(z=1|{ X }_{ { T }_{ 2 }{ T }_{ 1 } })}
}
\examples{
\dontrun{
data(tutoring)
formu <- ~ Gender + Ethnicity + Military + ESL + EdMother + EdFather + Age +
     Employment + Income + Transfer + GPA
tpsa <- trips(tutoring, tutoring$treat, formu)
head(tpsa)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.distances.R
\name{distances.plot}
\alias{distances.plot}
\title{Barplot for the sum of distances.}
\usage{
distances.plot(tmatch, caliper = 0.25, label = FALSE)
}
\arguments{
\item{tmatch}{the results of [trimatch()].}

\item{caliper}{a vector indicating where vertical lines should be drawn as a
factor of the standard deviation. Rosenbaum and Rubin (1985) suggested
one quarter of one standard deviation.}

\item{label}{label the bars that exceed the minimum caliper.}
}
\description{
Barplot for the sum of distances.
}
\seealso{
triangle.match
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.loess.R
\name{loess3.plot}
\alias{loess3.plot}
\title{Loess plot for matched triplets.}
\usage{
loess3.plot(
  tmatch,
  outcome,
  model,
  ylab = "Outcome",
  plot.connections = FALSE,
  connections.color = "black",
  connections.alpha = 0.2,
  plot.points = geom_point,
  points.alpha = 0.1,
  points.palette = "Dark2",
  ...
)
}
\arguments{
\item{tmatch}{the results of [trimatch()].}

\item{outcome}{a vector representing the outcomes.}

\item{model}{an integer between 1 and 3 indicating from which model the propensity
scores will be used.}

\item{ylab}{the label for the y-axis.}

\item{plot.connections}{boolean indicating whether lines will be drawn connecting
each matched triplet.}

\item{connections.color}{the line color of connections.}

\item{connections.alpha}{number between 0 and 1 representing the alpha levels for 
connection lines.}

\item{plot.points}{a \code{ggplot2} function for plotting points. Usually 
[ggplot2::geom_point()] or [ggplot2::geom_jitter()]. If `NULL` no points
will be drawn.}

\item{points.alpha}{number between 0 and 1 representing the alpha level for the points.}

\item{points.palette}{the color palette to use. See [ggplot2::scale_colour_brewer()]
and \url{https://colorbrewer2.org} for more information.}

\item{...}{other parameters passed to [ggplot2::geom_smooth()] and 
[ggplot2::stat_smooth()].}
}
\value{
a `ggplot2` figure.
}
\description{
This function will create a \code{ggplot2} figure with propensity scores on the
x-axis and the outcome on the y-axis. Three Loess regression lines will be plotted
based upon the propensity scores from \code{model}. Since each model produces
propensity scores for two of the three groups, the propensity score for the third
group in each matched triplet will be the mean of the other two. If \code{model}
is not specified, the default will be to use the model that estimates the propensity
scores for the first two groups in the matching order.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.triangle.matches.R
\name{summary.triangle.matches}
\alias{summary.triangle.matches}
\title{Provides a summary of the matched triplets including analysis of outcome
measure if provided.}
\usage{
\method{summary}{triangle.matches}(object, outcome, p = 0.05, ordering = attr(object, "match.order"), ...)
}
\arguments{
\item{object}{result of [trimatch()].}

\item{outcome}{vector representing the outcome measure.}

\item{p}{threshold of the p value to perform a}

\item{ordering}{specify the order for doing the paired analysis, that is
analysis will be conducted as:
\code{ordering[1] - ordering[2]}, \code{ordering[1] - ordering[3]},
and \code{ordering[2] - ordering[3]}.}

\item{...}{parameters passed to other statistical tests.}
}
\value{
a trimatch.summary object.
}
\description{
If an outcome measure is provided this function will perform a Freidman
Rank Sum Test and repeated measures ANOVA. If either test has a statistically
significant difference (as determined by the value of the \code{p} parameter),
a Pairwise Wilcoxon Rank Sum Test will also be provided.
}
\seealso{
[stats::friedman.test()], ez::ezANOVA(), [stats::pairwise.wilcox.test()]
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.boxdiff.R
\name{boxdiff.plot}
\alias{boxdiff.plot}
\title{Returns a \code{ggplot2} box plot of the differences.}
\usage{
boxdiff.plot(
  tmatch,
  out,
  plot.mean = TRUE,
  ordering = attr(tmatch, "match.order"),
  ci.width = 0.5
)
}
\arguments{
\item{tmatch}{the results from [trimatch()].}

\item{out}{a vector of the outcome measure of interest.}

\item{plot.mean}{logical indicating whether the means should be plotted.}

\item{ordering}{specify the order for doing the paired analysis, that is
analysis will be conducted as:
\code{ordering[1] - ordering[2]}, \code{ordering[1] - ordering[3]},
and \code{ordering[2] - ordering[3]}.}

\item{ci.width}{the width for the confidence intervals.}
}
\value{
a \code{ggplot2} boxplot of the differences.
}
\description{
A boxplot of differences between each pair of treatments.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.balance.R
\name{print.balance.plots}
\alias{print.balance.plots}
\title{Print the results of [balance.plot()] for a data frame of covariates.}
\usage{
\method{print}{balance.plots}(x, ...)
}
\arguments{
\item{x}{the results of [balance.plot()] when a data frame is specified.}

\item{...}{parameters passed to [plot.balance.plots()] and
[summary.balance.plots()].}
}
\description{
Print the results of [balance.plot()] for a data frame of covariates.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.triangle.psa.R
\name{summary.triangle.psa}
\alias{summary.triangle.psa}
\title{Prints the summary results of the logistic regression models.}
\usage{
\method{summary}{triangle.psa}(object, ...)
}
\arguments{
\item{object}{the results of [trips()].}

\item{...}{currently unused.}
}
\description{
The [trips()] function estimates three separate logistic regression
models for each pair of groups. This function will print a combined table
of the three summaries.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/unmatched.R
\name{unmatched}
\alias{unmatched}
\title{Returns rows from [trips()] that were not matched by [trimatch()].}
\usage{
unmatched(tmatch)
}
\arguments{
\item{tmatch}{the results of [trimatch()].}
}
\value{
a data frame of unmatched rows.
}
\description{
This function returns a subset of [trips()] that were not matched by
[trimatch()]. All data frame methods work with the returned object
but special `summary` function will provided relevant information.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/unmatched.R
\name{summary.unmatched}
\alias{summary.unmatched}
\title{Provides a summary of unmatched subjects.}
\usage{
\method{summary}{unmatched}(object, digits = 3, ...)
}
\arguments{
\item{object}{results of [unmatched()].}

\item{digits}{number of digits to print.}

\item{...}{currently unused.}
}
\value{
a list of summary results.
}
\description{
Will return as a list and print the percentage of total unmatched rows and
percent by treatment.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as.data.frame.list.R
\name{data.frame.to.list}
\alias{data.frame.to.list}
\title{Convert a list of vectors to a data frame.}
\usage{
data.frame.to.list(...)
}
\arguments{
\item{...}{other parameters passed to [base::data.frame()].}
}
\value{
a data frame.
}
\description{
This function will convert a list of vectors to a data frame. This function
will handle three different types of lists of vectors. First, if all the elements
in the list are named vectors, the resulting data frame will have have a number
of columns equal to the number of unique names across all vectors. In cases
where some vectors do not have names in other vectors, those values will be
filled with \code{NA}.
}
\details{
The second case is when all the vectors are of the same length. In this case,
the resulting data frame is equivalent to applying \code{rbind} across all elements.

The third case handled is when there are varying vector lengths and not all the
vectors are named. This condition should be avoided. However, the function will
attempt to convert this list to a data frame. The resulting data frame will have
a number of columns equal to the length of the longest vector. For vectors with
length less than this will fill the row with \code{NA}s. Note that this function
will print a warning if this condition occurs.
}
\examples{
    test1 <- list( c(a='a',b='b',c='c'), c(a='d',b='e',c='f'))
    as.data.frame(test1)
    
    test2 <- list( c('a','b','c'), c(a='d',b='e',c='f'))
    as.data.frame(test2)
    
    test3 <- list('Row1'=c(a='a',b='b',c='c'), 'Row2'=c(var1='d',var2='e',var3='f'))
    as.data.frame(test3)
    
    \dontrun{
    #This will print a warning.
    test4 <- list('Row1'=letters[1:5], 'Row2'=letters[1:7], 'Row3'=letters[8:14])
    as.data.frame(test4)
    }
    
    test5 <- list(letters[1:10], letters[11:20])
    as.data.frame(test5)
    
    \dontrun{
    #This will throw an error.
    test6 <- list(list(letters), letters)
    as.data.frame(test6)
    }
}
\references{
\url{http://stackoverflow.com/questions/4227223/r-list-to-data-frame}
}
\author{
Jason Bryer \href{mailto:jason@bryer.org}{jason@bryer.org}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.balance.R
\name{summary.balance.plots}
\alias{summary.balance.plots}
\title{Prints a summary table of the test statistics of each balance plot.}
\usage{
\method{summary}{balance.plots}(object, ...)
}
\arguments{
\item{object}{the results of [balance.plot()] when a data frame is specified.}

\item{...}{currently unused.}
}
\value{
a data frame
}
\description{
The [balance.plot()] function will create a grid of balance plots
if a data frame is provided. The returned object is a list of \code{ggplot2}
figures with the statistical tests (i.e. Friedmen Rank Sum tests and if a
continuous variable, repeated measures ANOVA as well) saved as attributes.
This function will return a data frame combining all of those results.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TriMatch-package.R
\docType{data}
\name{nmes}
\alias{nmes}
\title{Results from the 1987 National Medical Expenditure Study}
\format{
a data frame with 9,708 observations of 12 variables.
}
\source{
http://imai.princeton.edu/research/pscore.html
}
\description{
This file was originally prepared by Anders Corr (corr@fas.harvard.edu) who
reports on December 8, 2007 that the resulting numbers closely
match with those reported in the published article. It was later modified by
Jason Bryer (jason@bryer.org) to an R data object to be included in this 
package. See \url{http://imai.princeton.edu/research/pscore.html} for more
information
}
\references{
National Center For Health Services Research, 1987. National 
        Medical Expenditure Survey. Methods II. Questionnaires and data 
        collection methods for the household survey and the Survey of 
        American Indians and Alaska Natives. National Center for Health 
        Services Research and Health Technology Assessment.
        
        Imai, K., & van Dyk, D.A. (2004). Causal Inference With General 
        Treatment Regimes: Generalizing the Propensity Score, Journal of the 
        American Statistical Association, 99(467), pp. 854-866.
        
        Elizabeth Johnson, E., Dominici, F., Griswold, M., & Zeger, S.L. (2003).
        Disease cases and their medical costs attributable to smoking: An 
        analysis of the national medical expenditure survey. Journal of 
        Econometrics, 112.
}
\author{
United States Department of Health and Human Services. 
        Agency for Health Care Policy and Research
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/star.R
\name{star}
\alias{star}
\title{Returns significance level.}
\usage{
star(x)
}
\arguments{
\item{x}{p-value.}
}
\description{
Returns the significance level as stars, or NA if a non-numeric value is
passed in.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.triangle.psa.R
\name{plot.triangle.psa}
\alias{plot.triangle.psa}
\title{Triangle plot.}
\usage{
\method{plot}{triangle.psa}(
  x,
  point.alpha = 0.3,
  point.size = 1.5,
  legend.title = "Treatment",
  text.size = 4,
  draw.edges = FALSE,
  draw.segments = TRUE,
  edge.alpha = 0.2,
  edge.color = "grey",
  edge.labels = c("Model 1", "Model 2", "Model 3"),
  sample = c(1),
  ...
)
}
\arguments{
\item{x}{the results from [trips()].}

\item{point.alpha}{alpha level for points.}

\item{point.size}{point size.}

\item{legend.title}{title for the legend.}

\item{text.size}{text size.}

\item{draw.edges}{draw edges of the triangle.}

\item{draw.segments}{draw segments connecting points across two models.}

\item{edge.alpha}{alpha level for edges if drawn.}

\item{edge.color}{the color for edges if drawn.}

\item{edge.labels}{the labels to use for each edge of the triangle.}

\item{sample}{a vector of length 1 or 3 representing the sample of points to plot.
The position of each element corresponds to the groups as returned
by \code{attr(tpsa,'groups')}. If equal to one, all points will be plotted.
Values less than one will plot a percentage of points. Values greater
than one exactly that number of points will be plotted.}

\item{...}{currently unused.}
}
\value{
ggplot2 figure
}
\description{
Triangle plot showing the fitted values (propensity scores) for three different
models.
}
\seealso{
triangle.psa
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/triangle.match.R
\name{trimatch}
\alias{trimatch}
\title{Creates matched triplets.}
\usage{
trimatch(
  tpsa,
  caliper = 0.25,
  nmatch = c(15),
  match.order,
  exact,
  method = maximumTreat,
  ...
)
}
\arguments{
\item{tpsa}{the results from [trips()]}

\item{caliper}{a vector of length one or three indicating the caliper to use 
for matching within each step. This is expressed in standardized units such 
that .25 means that matches must be within .25 of one standard deviation 
to be kept, otherwise the match is dropped.}

\item{nmatch}{number of closest matches to retain before moving to next edge. This can
be \code{Inf} in which case all matches within the caliper will be retained
through to the next step. For large datasets, evaluating all possible
matches within the caliper could be time consuming.}

\item{match.order}{character vector of length three indicating the order in 
which the matching algorithm will processes. The default is to use start
with the group the middle number of subjects, followed by the smallest,
and then the largest.}

\item{exact}{a vector or data frame of representing covariates for exact matching.
That is, matched triplets will first be matched exactly on these covariates
before evaluating distances.}

\item{method}{This is a function that specifies which matched triplets will be
retained. If `NULL`, all matched triplets within the specified
caliper will be returned (equivalent to caliper matching in two group
matching). The default is [maximumTreat()] that
attempts include each treatment at least once. 
Another option is [OneToN()] which mimicks the one-to-n 
matching where treatments are matched to multiple control units.}

\item{...}{other parameters passed to `method`.}
}
\description{
Create matched triplets by minimizing the total distance between matched triplets
within a specified caliper.
}
\details{
The [trips()] function will estimate the propensity scores
for three models. This method will then find the best matched triplets based
upon minimizing the summed differences between propensity scores across the
three models. That is, the algorithm works as follows:

\itemize{
\item The first subject from model 1 is selected.
\item The \code{nmatch[1]} smallest distances are selected using propensity scores from
 model 1.
\item For each of the matches identified, the subjects propensity score from model
 2 is retrieved.
\item The \code{nmatch[2]} smallest distances are selected using propensity score from
 model 3.
\item For each of those matches identified, the subjects propensity score from model
 2 is retrieved.
\item The distances is calculated from the first and last subjects propensity scores
 from model 2.
\item The three distances are summed.
\item The triplet with the smallest overall distance is selected and returned.
}
}
\examples{
\dontrun{
data(turoing)
formu <- ~ Gender + Ethnicity + Military + ESL + EdMother + EdFather + Age +
     Employment + Income + Transfer + GPA
tpsa <- trips(tutoring, tutoring$treat, formu)
tmatch <- trimatch(tpsa, status=FALSE)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/trimatch.apply2.R
\name{trimatch.apply2}
\alias{trimatch.apply2}
\title{Recursive function to find possible matched triplets using the apply functions.}
\usage{
trimatch.apply2(tpsa, caliper, nmatch, match.order, sd1, sd2, sd3)
}
\arguments{
\item{tpsa}{the results from [trips()]}

\item{caliper}{a vector of length one or three indicating the caliper to use 
for matching within each step. This is expressed in standardized units such 
that .25 means that matches must be within .25 of one standard deviation 
to be kept, otherwise the match is dropped.}

\item{nmatch}{number of closest matches to retain before moving to next edge. This can
be \code{Inf} in which case all matches within the caliper will be retained
through to the next step. For large datasets, evaluating all possible
matches within the caliper could be time consuming.}

\item{match.order}{character vector of length three indicating the order in 
which the matching algorithm will processes. The default is to use start
with the group the middle number of subjects, followed by the smallest,
and then the largest.}

\item{sd1}{standard deviation for propensity scores from model 1.}

\item{sd2}{standard deviation for propensity scores from model 2.}

\item{sd3}{standard deviation for propensity scores from model 3.}
}
\description{
Internal method. This version does not use the exact matching. Instead, this
function should be called separately for each grouping.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TriMatch-package.R
\docType{data}
\name{tutoring}
\alias{tutoring}
\title{Results from a study examining the effects of tutoring services on course grades.}
\format{
a data frame with 17 variables.
}
\description{
\itemize{
\item \code{treat} Treatment indicator.
\item \code{Course} The course id the student was enrolled in.
\item \code{Grade} The course grade the student earned (4=A, 3=B, 2=C, 1=D, 0=F or W).
\item \code{Gender} Gender of the student.
\item \code{Ethnicity} Ethnicity of the student, either White, Black, or Other.
\item \code{Military} Is the student an active military student.
\item \code{ESL} English second language student.
\item \code{EdMother} Education level of the mother (1 = did not finish high school;
2 = high school grad; 3 = some college; 4 = earned associate degree;
5 = earned baccalaureate degree; 6 = Earned Master's degree; 7 = earned doctorate).
\item \code{EdFather} Education level of the father (levels same as EdMother).
\item \code{Age} Age at the start of the course.
\item \code{Employment} Employment level at college enrollment (1 = No; 2 = part-time;
3 = full-time).
\item \code{Income} Household income level at college enrollment (1 = <25K; 2 = <35K; 3 = <45K;
4 = <55K; 5 = <70K; 6 = <85K; 7 = <100K; 8 = <120K; 9 = >120K).
\item \code{Transfer} Number of transfer credits at the start of the course.
\item \code{GPA} GPA as of the start of the course.
\item \code{GradeCode} Letter grade.
\item \code{Level} Level of the course, either Lower or Upper.
\item \code{ID} Randomly assigned student ID.
}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/covariateBalance.R
\name{covariateBalance}
\alias{covariateBalance}
\title{Calculate covariate effect size differences before and after stratification.}
\usage{
covariateBalance(
  covariates,
  treatment,
  propensity,
  strata = NULL,
  int = NULL,
  tree = FALSE,
  minsize = 2,
  universal.psd = TRUE,
  trM = 0,
  absolute.es = TRUE,
  trt.value = NULL,
  use.trt.var = FALSE,
  verbose = FALSE,
  xlim = NULL,
  plot.strata = TRUE,
  ...
)
}
\arguments{
\item{covariates}{dataframe of interest}

\item{treatment}{binary vector of 0s and 1s (necessarily? what if character, or 1, 2?)}

\item{propensity}{PS scores from some method or other.}

\item{strata}{either a vector of strata number for each row of covariate, or 
one number n in which case it is attempted to group rows by ps scores into 
n strata of size approximately 1/n.  This does not seem to work well in the 
case of few specific propensity values, as from a tree.}

\item{int}{either a number m used to divide [0,1] into m equal length subintervals, 
or a vector of cut points between 0 an    1 defining the subintervals (perhaps 
as suggested by loess.psa).  In either case these subintervals define strata, 
so strata can be of any size.}

\item{tree}{logical, if unique ps scores are few, as from a recursively partitioned 
tree, then TRUE will force each ps value to define a stratum.}

\item{minsize}{smallest allowable stratum-treatment size.  If violated, strata is 
removed.}

\item{universal.psd}{If 'TRUE', forces standard deviations used to be unadjusted 
for stratification.}

\item{trM}{trimming proportion for mean calculations.}

\item{absolute.es}{logical, if 'TRUE' routine uses absolute values of all effect sizes.}

\item{trt.value}{allows user to specify which value is active treatment, if desired.}

\item{use.trt.var}{logical, if true then Rubin-Stuart method using only treatment 
variance with be used in effect size calculations.}

\item{verbose}{logical, controls output that is visibly returned.}

\item{xlim}{limits for the x-axis.}

\item{plot.strata}{logical indicating whether to print strata.}

\item{...}{currently unused.}
}
\description{
This function is modified from the [PSAgraphics::cv.bal.psa()] function in the 
`PSAgrpahics` package.
}
\details{
Note: effect sizes are calculated as treatment 1 - treatment 0, or treatment B - treatment A.
}
\author{
Robert M. Pruzek RMPruzek@yahoo.com

James E. Helmreich James.Helmreich@Marist.edu

KuangNan Xiong harryxkn@yahoo.com
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.triangle.psa.R
\name{print.triangle.plot}
\alias{print.triangle.plot}
\title{Print method for [plot.triangle.psa()]. The primary purpose is to 
suppress the "Removed n rows containing missing values" warning printed
by `ggplot2`.}
\usage{
\method{print}{triangle.plot}(x, ...)
}
\arguments{
\item{x}{a plot from [plot.triangle.psa()].}

\item{...}{other parameters passed to ggplot2.}
}
\description{
Print method for [plot.triangle.psa()]. The primary purpose is to 
suppress the "Removed n rows containing missing values" warning printed
by `ggplot2`.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.triangle.matches.R
\name{plot.triangle.matches}
\alias{plot.triangle.matches}
\title{Triangle plot drawing matched triplets.}
\usage{
\method{plot}{triangle.matches}(
  x,
  sample = 0.05,
  rows = sample(nrow(tmatch), nrow(tmatch) * sample),
  line.color = "black",
  line.alpha = 0.5,
  point.color = "black",
  point.size = 3,
  p,
  ...
)
}
\arguments{
\item{x}{matched triplets from [triangle.match()].}

\item{sample}{an number between 0 and 1 representing the percentage of matched
triplets to draw.}

\item{rows}{an integer vector corresponding to the rows in `tmatch` to draw.}

\item{line.color}{the line color.}

\item{line.alpha}{the alpha for the lines.}

\item{point.color}{color of matched triplet points.}

\item{point.size}{point size for matched triplets.}

\item{p}{a ggplot to add the match lines. If NULL, then [plot.triangle.psa()].}

\item{...}{other parameters passed to [plot.triangle.psa()].}
}
\value{
a `ggplot2` graphic.
}
\description{
This plot function adds a layer to [plot.triangle.psa()] drawing 
matched triplets. If `p` is supplied, this function will simply draw on
top of the pre-existing plot, otherwise [plot.triangle.psa()] will
be called first.
}
\details{
If this function calls [plot.triangle.psa()], it will only draw
line segments and points for those data rows that were used in the matching
procedure. That is, data elements not matched will be excluded from the
figure. To plot all segments and points regardless if used in matching, set
`p = plot(tpsa)`.
}
\seealso{
plot.triangle.psa

triangle.match
}
